<?php
/*
 * metal_compiler.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_compiler.php,v 1.91 2005/05/26 03:58:13 mlemos Exp $
 *
 */

$metal_xml_parsers=array();
$metal_xml_parser_signatures=array();

class metal_execution_context_class
{
	var $path;
	var $file;
	var $result;
};

class metal_compiler_class
{
	var $error="";
	var $error_file="";
	var $error_line=0;
	var $error_column=0;
	var $error_byte_index=0;
	var $parser;
	var $objects=array();
	var $functions=array();
	var $bindings=array();
	var $bindings_file="";
	var $language=array();
	var $cache_directory="";
	var $cache_parsed_files=1;
	var $timer=0.0;
	var $included_input=array();
	var $context_functions=array();
	var $class_context_functions=array();
	var $declared_context_functions=array();
	var $simplified_xml=1;
	var $fail_on_non_simplified_xml=1;
	var $include_path=array(".");
	var $arguments=array();
	var $sequences=array("XML"=>1);

	var $next_token="";
	var $signature="";

	Function strtok($string,$separator="")
	{
		if(!strcmp($separator,""))
		{
			$separator=$string;
			$string=$this->next_token;
		}
		for($character=0;$character<strlen($separator);$character++)
		{
			if(GetType($position=strpos($string,$separator[$character]))=="integer")
			{
				$this->next_token=substr($string,$position+1);
				return(substr($string,0,$position));
			}
		}
		$this->next_token="";
		return($string);
	}

	Function SetError($file,$line,$column,$byte_index,$error)
	{
		$this->error=$error;
		$this->error_file=$file;
		$this->error_line=$line;
		$this->error_column=$column;
		$this->error_byte_index=$byte_index;
		return(0);
	}

	Function SetParserError($file,&$parser,$error)
	{
		return($this->SetError($file,$parser->error_line,$parser->error_column,$parser->error_byte_index,$error));
	}

	Function SetParsedElementError($file,&$parser,$path,$error)
	{
		return($this->SetError($file,$parser->positions[$path]["Line"],$parser->positions[$path]["Column"],$parser->positions[$path]["Byte"],$error));
	}

	Function SetElementError($file,$path,$error)
	{
		global $metal_xml_parsers;

		return($this->SetParsedElementError($file,$metal_xml_parsers[$file],$path,$error));
	}

	Function GetParser(&$parser,$file)
	{
		global $metal_xml_parsers;

		if(!IsSet($metal_xml_parsers[$file]))
			return(0);
		$parser=$metal_xml_parsers[$file];
		return(1);
	}

	Function CreateClassObject(&$object, $class)
	{
		$object=new $class;
	}

	Function CopyEmptyContext(&$from,&$to)
	{
		$this->CreateClassObject($to,"metal_execution_context_class");
		$to->file=$from->file;
		$to->path=$from->path;
		$to->result=array();
	}

	Function CreateFileContext(&$context,$file)
	{
		$this->CreateClassObject($context,"metal_execution_context_class");
		$context->file=$file;
		$context->path="0";
		$context->result=array();
	}

	Function CreateOutputDirectory($directory)
	{
		if($directory!=""
		&& $directory[0]=="/")
		{
			$path="/";
			$directory=substr($directory,1);
		}
		else
			$path="";
		for($remaining=$directory;strcmp($remaining,"");)
		{
			$directory=$this->strtok($remaining,"/");
			$remaining=$this->strtok("");
			$path.=$directory;
			if(@!is_dir($path)
			&& @!mkdir($path,~umask()))
				return(0);
			$path.="/";
		}
		return(1);
	}

	Function CreateContextOutputDirectory($directory,&$context,$error)
	{
		if(!$this->CreateOutputDirectory($directory))
		{
			$this->SetElementError($context->file,$context->path,$error);
			return(0);
		}
		return(1);
	}

	Function GetSequenceNextValue($sequence)
	{
		return(IsSet($this->sequences[$sequence]) ? ++$this->sequences[$sequence] : 0);
	}

	Function GetXMLNode()
	{
		return($this->GetSequenceNextValue("XML"));
	}

	Function RelativePath($from,$to)
	{
		if(strcmp($from,"")
		&& $from[strlen($from)-1]=="/")
			$from=substr($from,0,strlen($from)-1);
		$from_parts=explode("/",$from);
		if(strcmp($to,"")
		&& $to[strlen($to)-1]=="/")
			$to=substr($to,0,strlen($to)-1);
		$to_parts=explode("/",$to);
		for($common=0;$common<count($from_parts) && $common<count($to_parts) && !strcmp($from_parts[$common],$to_parts[$common]);$common++);
		if($common==count($from_parts))
			$relative_path=".";
		else
		{
			for($relative_path="",$up=count($from_parts);$up>$common;$up--)
			{
				if($relative_path!="")
					$relative_path.="/";
				$relative_path.="..";
			}
		}
		for(;$common<count($to_parts);$common++)
			$relative_path.="/".$to_parts[$common];
		return($relative_path);
	}

	Function CheckParsedWhiteSpace($file,&$parser,$data,$path)
	{
		$line=$parser->positions[$path]["Line"];
		$column=$parser->positions[$path]["Column"];
		$byte_index=$parser->positions[$path]["Byte"];
		for($previous_return=0,$position=0;$position<strlen($data);$position++)
		{
			switch($data[$position])
			{
				case " ":
				case "\t":
					$column++;
					$byte_index++;
					$previous_return=0;
					break;
				case "\n":
					if(!$previous_return)
						$line++;
					$column=1;
					$byte_index++;
					$previous_return=0;
					break;
				case "\r":
					$line++;
					$column=1;
					$byte_index++;
					$previous_return=1;
					break;
				default:
					$this->SetError($file,$line,$column,$byte_index,"data is not white space");
					return(0);
			}
		}
		return(1);
	}

	Function CheckWhiteSpace($file,$data,$path)
	{
		global $metal_xml_parsers;

		return($this->CheckParsedWhiteSpace($file,$metal_xml_parsers[$file],$data,$path));
	}

	Function GetValidParsedData($file,&$parser,$path,&$name,$error)
	{
		$elements=$parser->structure[$path]["Elements"];
		for($name="",$element=0;$element<$elements;$element++)
		{
			$part_path=$path.",$element";
			$part=$parser->structure[$part_path];
			if(GetType($part)!="string")
			{
				$data=array();
				if(!$this->GetOutputParsedData($file,$parser,$part_path,$data,$error,0,1))
					return(0);
				for($data_part=0;$data_part<count($data);$data_part++)
				{
					if(strcmp($data[$data_part]["Type"],"DATA"))
					{
						$this->SetParsedElementError($file,$parser,$part_path,$error);
						return(0);
					}
					$name.=$data[$data_part]["Data"];
				}
			}
			else
				$name.=$part;
		}
		return(1);
	}

	Function GetValidData($file,$path,&$name,$error)
	{
		global $metal_xml_parsers;

		return($this->GetValidParsedData($file,$metal_xml_parsers[$file],$path,$name,$error));
	}

	Function ValidateVariableName($name)
	{
		return(eregi("^[A-Z][A-Z0-9_]*\$",$name,$match));
	}

	Function GetValidParsedName($file,&$parser,$path,&$name,$error)
	{
		if(!$this->GetValidParsedData($file,$parser,$path,$name,$error))
			return(0);
		if(!$this->ValidateVariableName($name))
		{
			$this->SetParsedElementError($file,$parser,$path,$error);
			return(0);
		}
		return(1);
	}

	Function GetValidName($file,$path,&$name,$error)
	{
		global $metal_xml_parsers;

		return($this->GetValidParsedName($file,$metal_xml_parsers[$file],$path,$name,$error));
	}

	Function StripIndentation($data)
	{
		for($stripped="";$data!="";)
		{
			$length=strcspn($data,"\n\r");
			$stripped.=substr($data,0,$length);
			if($length==strlen($data))
				break;
			$stripped.=" ";
			$data=substr($data,$length);
			if(($skip=strspn($data,"\n\r\t ")))
				$data=substr($data,$skip);
		}
		return($stripped);
	}

	Function GetOutputParsedPartData($file,&$parser,$part_path,&$data,$error,$skip_white_space)
	{
		$part=$parser->structure[$part_path];
		if(GetType($part)=="string")
		{
			if($skip_white_space)
			{
				if(!$this->CheckParsedWhiteSpace($file,$parser,$part,$part_path))
					return(0);
			}
			else
			{
				$data[]=array(
					"Data"=>$part,
					"Type"=>"DATA"
				);
			}
		}
		else
		{
			$this->CreateFileContext($context,$file);
			$context->path=$part_path;
			if(!$this->ExecuteOutputObject($parser,$context))
				return(0);
			if(IsSet($context->result))
			{
				for($result=0;$result<count($context->result);$result++)
					$data[]=$context->result[$result];
			}
		}
		return(1);
	}

	Function GetOutputParsedData($file,&$parser,$path,&$data,$error,$skip_white_space,$part=0)
	{
		if($part)
			return($this->GetOutputParsedPartData($file,$parser,$path,$data,$error,$skip_white_space));
		$elements=$parser->structure[$path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			if(!$this->GetOutputParsedPartData($file,$parser,$path.",$element",$data,$error,$skip_white_space))
				return(0);
		}
		return(1);
	}

	Function GetOutputData($file,$path,&$data,$error,$skip_white_space,$part=0)
	{
		global $metal_xml_parsers;

		return($this->GetOutputParsedData($file,$metal_xml_parsers[$file],$path,$data,$error,$skip_white_space,$part));
	}

	Function DoComposeExpression(&$bindings,&$context,$type,$side,&$expression,$error)
	{
		return($bindings->ComposeExpression($this,$context,$type,$side,$expression,$error));
	}

	Function ComposeExpression(&$context,$type,$side,&$expression,$error)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoComposeExpression($this->language["object"],$context,$type,$side,$expression,$error));
	}

	Function GetValidParsedExpression(&$context,&$parser,$type,$side,$error,$skip_white_space,$part=0)
	{
		$context->result=$expression=array();
		if(!$this->GetOutputParsedData($context->file,$parser,$context->path,$expression,$error,$skip_white_space,$part)
		|| !$this->ComposeExpression($context,$type,$side,$expression,$error))
			return(0);
		return(1);
	}

	Function GetValidExpression(&$context,$type,$side,$error,$skip_white_space,$part=0)
	{
		global $metal_xml_parsers;

		return($this->GetValidParsedExpression($context,$metal_xml_parsers[$context->file],$type,$side,$error,$skip_white_space,$part));
	}

	Function DoConvertTypedData(&$bindings,&$context,$type,&$data)
	{
		return($bindings->ConvertTypedData($this,$context,$type,$data));
	}

	Function ConvertTypedData(&$context,$type,$data)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoConvertTypedData($this->language["object"],$context,$type,$data));
	}

	Function ConvertData(&$context,$type,$data,&$converted)
	{
		$this->CopyEmptyContext($context,$convert_context);
		if(!$this->ConvertTypedData($convert_context,$type,$data))
			return(0);
		$converted=$convert_context->result[0]["Data"]["Value"];
		return(1);
	}

	Function GetValidParsedTypedExpression(&$context,&$parser,$type,$error,$part=0,$skip_white_space=0)
	{
		$context->result=$expression=array();
		if(!$this->GetOutputParsedData($context->file,$parser,$context->path,$expression,$error,$skip_white_space,$part))
			return(0);
		for($data="",$initializations=array(),$part=0;$part<count($expression);$part++)
		{
			switch($expression[$part]["Type"])
			{
				case "INITIALIZATION":
				case "FILE":
					$initializations[]=$expression[$part];
					break;
				case "DATA":
					$data.=$expression[$part]["Data"];
					break;
				default:
					return($this->ComposeExpression($context,$type,"ANY",$expression,$error));
			}
		}
		for($part=0;$part<count($initializations);$part++)
			$context->result[]=$initializations[$part];
		return($this->ConvertTypedData($context,$type,$data));
	}

	Function GetValidTypedExpression(&$context,$type,$error,$part=0,$skip_white_space=0)
	{
		global $metal_xml_parsers;

		return($this->GetValidParsedTypedExpression($context,$metal_xml_parsers[$context->file],$type,$error,$part,$skip_white_space));
	}

	Function DoGetExpressionValue(&$context,&$bindings,&$expression,&$expression_value,&$expression_type)
	{
		return($bindings->GetExpressionValue($this,$context,$expression,$expression_value,$expression_type));
	}

	Function GetExpressionValue(&$context,&$expression,&$expression_value,&$expression_type)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGetExpressionValue($context,$this->language["object"],$expression,$expression_value,$expression_type));
	}

	Function GetArguments(&$code,&$context,&$values,&$types,$class,$function)
	{
		$elements=$code->structure[$context->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$context->path.",$element";
			$data=$code->structure[$element_path];
			if(GetType($data)=="array")
			{
				if(IsSet($types[$data["Tag"]]))
				{
					$type=$this->strtok($types[$data["Tag"]],":");
					$side=$this->strtok("");
					if($side!="MULTIPLE"
					&& IsSet($values[$data["Tag"]]))
						return($this->SetElementError($context->file,$element_path,"it was specified the $class object $function parameter \"".$data["Tag"]."\" more than once"));
					switch($type)
					{
						case "DATA":
							if(!$this->GetValidData($context->file,$element_path,$value,"it was not specified a valid $class $function ".$data["Tag"]." argument"))
								return(0);
							break;
						case "BOOLEANDATA":
							if(!$this->GetValidData($context->file,$element_path,$value,"it was not specified a valid $class $function ".$data["Tag"]." argument"))
								return(0);
							switch($value)
							{
								case "0":
								case "1":
									break;
								default:
									return($this->SetElementError($context->file,$element_path,$value." is not a valid $class object $function boolean data parameter"));
							}
							$value=intval($value);
							break;
						case "INTEGERDATA":
							if(!$this->GetValidData($context->file,$element_path,$value,"it was not specified a valid $class $function ".$data["Tag"]." argument"))
								return(0);
							if(strcmp(strval(intval($value)),$value))
								return($this->SetElementError($context->file,$element_path,$value." is not a valid $class object $function integer data parameter"));
							$value=intval($value);
							break;
						case "NAME":
							if(!$this->GetValidName($context->file,$element_path,$value,"it was not specified a valid $class $function ".$data["Tag"]." name argument"))
								return(0);
							break;
						case "TEXT":
							if(!$this->GetValidData($context->file,$element_path,$value,"it was not specified a valid $class $function ".$data["Tag"]." text argument"))
								return(0);
							$value=$this->StripIndentation($value);
							break;
						case "CONTEXT":
							$this->CopyEmptyContext($context,$value);
							$value->path=$element_path;
							break;
						default:
							$this->CopyEmptyContext($context,$evaluate_context);
							$evaluate_context->path=$element_path;
							if(!($side==""
							? $this->GetValidTypedExpression($evaluate_context,$types[$data["Tag"]],"it was not specified a valid $class $function ".$data["Tag"]." argument")
							: $this->GetValidExpression($evaluate_context,$type,$side,"it was not specified a valid $class $function ".$data["Tag"]." argument",1,0))
							|| !$this->GetExpressionValue($context,$evaluate_context->result,$value,$expression_type))
								return(0);
							break;
					}
					if(strcmp($type,"CONTEXT"))
					{
						if($side=="MULTIPLE")
							$values[$data["Tag"]][]=$value;
						else
							$values[$data["Tag"]]=$value;
					}
					else
					{
						if($side=="MULTIPLE")
							$this->CopyEmptyContext($value,$values[$data["Tag"]][]);
						else
							$this->CopyEmptyContext($value,$values[$data["Tag"]]);
					}
				}
				else
				{
					$this->SetElementError($context->file,$element_path,$data["Tag"]." is not a supported $class object $function parameter");
					return(0);
				}
			}
			else
			{
				if(!$this->CheckWhiteSpace($context->file,$data,$element_path))
					return(0);
			}
		}
		return(1);
	}

	Function GetContextArguments(&$context,&$values,&$types,$class,$function)
	{
		global $metal_xml_parsers;

		return($this->GetArguments($metal_xml_parsers[$context->file],$context,$values,$types,$class,$function));
	}

	Function GetOptionalArgumentValues(&$context,&$values,&$required_types,&$optional_types,$class,$function,$final)
	{
		$types=$required_types;
		for($type=0,Reset($optional_types);$type<count($optional_types);Next($optional_types),$type++)
		{
			$name=Key($optional_types);
			if(IsSet($types[$name]))
			{
				$this->SetElementError($context->file,$context->path,IsSet($required_types[$name]) ? "it specified a argument (\"$name\") either as optional and required" : "it specified a repeated optional argument (\"$name\")");
				return(0);
			}
			$types[$name]=$optional_types[$name];
		}
		if(!$this->GetContextArguments($context,$values,$types,$class,$function))
			return(0);
		if($final)
		{
			for($type=0,Reset($required_types);$type<count($required_types);Next($required_types),$type++)
			{
				if(!IsSet($values[Key($required_types)]))
				{
					$this->strtok($required_types[Key($required_types)],":");
					$multiple=!strcmp($this->strtok(""),"MULTIPLE");
					$this->SetElementError($context->file,$context->path,"it was not specified ".($multiple ? "any" : "the")." ".$class." ".$function." ".Key($required_types)." argument");
					return(0);
				}
			}
		}
		return(1);
	}

	Function GetArgumentValues(&$arguments_code,&$arguments,&$code,&$context,&$values,&$types,$class,$function)
	{
		$values=array();
		$optional_types=array();
		return($this->GetOptionalArgumentValues($arguments,$values,$types,$optional_types,$class,$function,0)
		&& $this->GetOptionalArgumentValues($context,$values,$types,$optional_types,$class,$function,1));
	}

	Function GetIncludePathFromList($file, &$path_list)
	{
		for($include_path="",$path=0;$path<count($path_list);$path++)
		{
			if(file_exists($path_list[$path]."/".$file))
			{
				$include_path=$path_list[$path]."/";
				break;
			}
		}
		return($include_path);
	}

	Function GetIncludePath($file)
	{
		return($this->GetIncludePathFromList($file,$this->include_path));
	}

	Function GetBaseIncludePath($file,&$base_path)
	{
		for($include_path="",$base=0;$base<count($base_path);$base++)
		{
			for($path=0;$path<count($this->include_path);$path++)
			{
				if(file_exists($this->include_path[$path]."/".$base_path[$base]."/".$file))
				{
					$include_path=$base_path[$base]."/";
					break 2;
				}
			}
		}
		return($include_path);
	}

	Function ParseDefinitionFileFromPath($file, &$path)
	{
		global $metal_xml_parsers,$metal_xml_parser_signatures;

		$include_path=$this->GetIncludePathFromList($file, $path);
		if(strlen($file)==0
		|| !file_exists($include_path.$file))
		{
			$this->SetError($file,0,0,0,"XML file \"$file\" does not exist");
			return(0);
		}
		if($this->cache_parsed_files)
		{
			$current_dir=getcwd();
			if(!chdir(dirname($include_path.$file)))
			{
				$this->SetError($file,0,0,0,"could not access the file \"$file\" directory");
				return(0);
			}
			$cache_path=getcwd();
			if(!chdir($current_dir))
			{
				$this->SetError($file,0,0,0,"could not access the current directory");
				return(0);
			}
			if(defined("PHP_OS")
			&& !strcmp(substr(PHP_OS,0,3),"WIN"))
				$cache_path=ereg_replace('^([A-Za-z]+)(:)', '/\1', str_replace("\\", "/", $cache_path));
			$dir_path=substr($cache_path,1);
			$file_path=basename($cache_path."/".$file);
			$cache_file=($this->cache_directory=="" ? "" : $this->cache_directory."/").$dir_path."/".$file_path.".cache";
			if(!$this->CreateOutputDirectory(dirname($cache_file)))
			{
				$this->SetError($cache_file,0,0,0,"could not create the XML cache file directory \"".dirname($cache_file)."\"");
				return(0);
			}
		}
		if(IsSet($metal_xml_parsers[$file])
		|| !strcmp($error=XMLParseFile($metal_xml_parsers[$file],$include_path.$file,1,$this->cache_parsed_files ? $cache_file : "",0,"ISO-8859-1",$this->simplified_xml,$this->fail_on_non_simplified_xml),""))
		{
			if(!IsSet($metal_xml_parser_signatures[$file])
			|| strlen($metal_xml_parser_signatures[$file])==0)
				$metal_xml_parser_signatures[$file]=$this->signature;
			return(1);
		}
		if(!IsSet($metal_xml_parsers[$file])
		|| GetType($metal_xml_parsers[$file])!="object")
		{
			$this->CreateClassObject($metal_xml_parsers[$file],"xml_parser_class");
			$metal_xml_parsers[$file]->simplified_xml=$this->simplified_xml;
			$metal_xml_parsers[$file]->fail_on_non_simplified_xml=$this->fail_on_non_simplified_xml;
		}
		$this->SetParserError($file,$metal_xml_parsers[$file],$error);
		Unset($metal_xml_parsers[$file]);
		return(0);
	}

	Function ParseDefinitionFile($file)
	{
		return($this->ParseDefinitionFileFromPath($file, $this->include_path));
	}

	Function AddContextFunctions(&$context,$object,&$functions,&$level)
	{
		if(!IsSet($this->objects[$object]))
		{
			$this->SetElementError($context->file,$context->path,"$object is not a known object");
			return(0);
		}
		$class=$this->objects[$object]["class"];
		$declared_functions=array();
		for($function=0;$function<count($functions);$function++)
		{
			if(IsSet($this->class_context_functions[$class])
			&& IsSet($this->class_context_functions[$class][$functions[$function]]))
				$declared_functions[$this->class_context_functions[$class][$functions[$function]]]=$functions[$function];
		}
		$level=count($this->declared_context_functions);
		$this->declared_context_functions[$level]=array(
			"Object"=>$object,
			"Functions"=>$declared_functions
		);
		return(1);

	}

	Function RemoveContextFunctions(&$context,$object,&$level)
	{
		if(!IsSet($this->objects[$object]))
		{
			$this->SetElementError($context->file,$context->path,"$object is not a known object");
			return(0);
		}
		if($level!=count($this->declared_context_functions)-1)
		{
			$this->SetElementError($context->file,$context->path,"the specified context level ($level) does not match the current context functions top level (".(count($this->declared_context_functions)-1).")");
			return(0);
		}
		if(strcmp($object,$this->declared_context_functions[$level]["Object"]))
		{
			$this->SetElementError($context->file,$context->path,"the specified object ($object) does not match the current context functions top level object (".$this->declared_context_functions[$level]["Object"].")");
			return(0);
		}
		Unset($this->declared_context_functions[$level]);
		return(1);
	}

	Function DoSetupObject(&$object,&$attributes,$name)
	{
		global $metal_xml_parsers;

		$object->object_name=$name;
		$object->initialized=array();
		return($object->Setup($this,$metal_xml_parsers[$attributes->file],$attributes));
	}

	Function SetupObject($object)
	{
		if(!IsSet($this->objects[$object]["object"]))
		{
			$class=$this->bindings[$this->objects[$object]["class"]];
			for($files=$class["Files"],$file=0;$file<count($files);$file++)
			{
				if(!IsSet($files[$file]["symbol"])
				|| !defined($files[$file]["symbol"]))
				{
					$include_path=$this->GetIncludePath($files[$file]["path"]);
					include($include_path.$files[$file]["path"]);
				}
			}
			$this->CreateClassObject($this->objects[$object]["object"],$class["target"]);
			if(IsSet($class["languagebindings"]))
			{
				$this->CreateClassObject($this->objects[$object]["object"]->language_bindings,$class["languagebindings"]);
				$this->objects[$object]["object"]->language_bindings->language=$this->language["name"];
			}
			if(!$this->DoSetupObject($this->objects[$object]["object"],$this->objects[$object]["attributes"],$object))
			{
				Unset($this->objects[$object]["object"]);
				return(0);
			}
		}
		return(1);
	}

	Function ExecuteObject(&$object,&$code,$function,&$context,&$arguments)
	{
		global $metal_xml_parsers;

		return($object->Execute($this,$code,$function,$context,$metal_xml_parsers[$arguments->file],$arguments));
	}

	Function LookupObjectFunction(&$code,&$context,&$object,&$function_context)
	{
		$data=$code->structure[$context->path];
		$object=$function="unknown";
		if(GetType($data=$code->structure[$context->path])!="array")
		{
			$this->SetElementError($context->file,$context->path,"it was not specified a valid output object ($object) function ($function)");
			return(0);
		}
		$function=$data["Tag"];
		for($level=count($this->declared_context_functions);$level>0;)
		{
			$level--;
			if(IsSet($this->declared_context_functions[$level]["Functions"][$function]))
			{
				$object=$this->declared_context_functions[$level]["Object"];
				$class=$this->objects[$object]["class"];
				$call=$this->declared_context_functions[$level]["Functions"][$function];
				$function_context=array(
					"call"=>$call,
					"arguments"=>$this->context_functions[$this->class_context_functions[$class][$call]]["arguments"]
				);
				return(1);
			}
		}
		if(!IsSet($this->functions[$function])
		|| !IsSet($this->objects[$object=$this->functions[$function]["object"]]))
		{
			$this->SetElementError($context->file,$context->path,"it was not specified a valid output object ($object) function ($function)");
			return(0);
		}
		$function_context=$this->functions[$function];
		return(1);
	}

	Function ExecuteOutputObject(&$code,&$context)
	{
		if(!$this->LookupObjectFunction($code,$context,$object,$function_context)
		|| !$this->SetupObject($object))
			return(0);
		return($this->ExecuteObject($this->objects[$object]["object"],$code,$function_context["call"],$context,$function_context["arguments"]));
	}

	Function ExecuteOutput(&$code,$output_file,&$result,$output=1)
	{
		$this->CreateFileContext($context,$output_file);
		$context->path="0";
		if(GetType($code->structure[$context->path])!="array"
		|| ($output
		&& strcmp($code->structure[$context->path]["Tag"],"output")))
		{
			$this->SetElementError($context->file,$context->path,"it was not specified a valid output definition file");
			return(0);
		}
		if($output)
		{
			$elements=$code->structure[$context->path]["Elements"];
			$path=$context->path;
			for($element=0;$element<$elements;$element++)
			{
				$element_path=$path.",$element";
				$data=$code->structure[$element_path];
				if(GetType($data)=="array")
				{
					$context->path=$element_path;
					if(!$this->ExecuteOutputObject($code,$context))
						return(0);
				}
				else
				{
					if(!$this->CheckWhiteSpace($context->file,$data,$element_path))
						return(0);
				}
			}
		}
		else
		{
			if(!$this->ExecuteOutputObject($code,$context))
				return(0);
		}
		for($part=0;$part<count($context->result);$part++)
			$result[]=$context->result[$part];
		return(1);
	}

	Function ExecuteOutputFile($output_file,&$result,$output=1)
	{
		global $metal_xml_parsers;

		if(!$this->ParseDefinitionFile($output_file)
		|| !$this->ExecuteOutput($metal_xml_parsers[$output_file],$output_file,$result,$output))
			return(0);
		$include_path=$this->GetIncludePath($output_file);
		$result[]=array(
			"Data"=>array(
				"Name"=>$include_path.$output_file,
				"Type"=>"DEPENDENCY"
			),
			"Type"=>"FILE"
		);
		return(1);
	}

	Function GetBindingsFileData(&$parser,$path,$bindings_file,&$bindings_file_data)
	{
		$file_elements=$parser->structure[$path]["Elements"];
		for($bindings_file_data=array(),$file_element=0;$file_element<$file_elements;$file_element++)
		{
			$file_element_path=$path.",$file_element";
			$data=$parser->structure[$file_element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "path":
						if(IsSet($bindings_file_data["path"]))
						{
							$this->SetElementError($bindings_file,$file_element_path,"it was specified a path already specified for the bindings file");
							return(0);
						}
						if(!$this->GetValidData($bindings_file,$file_element_path,$bindings_file_data["path"],"it was not specified a valid bindings class source name"))
							return(0);
						$include_path=$this->GetIncludePath($bindings_file_data["path"]);
						if(!file_exists($include_path.$bindings_file_data["path"]))
						{
							$this->SetElementError($bindings_file,$file_element_path,"it was specified a path for a bindings file (".$bindings_file_data["path"].") that does not exist");
							return(0);
						}
						break;
					case "symbol":
						if(IsSet($bindings_file_data["symbol"]))
						{
							$this->SetElementError($bindings_file,$file_element_path,"it was specified a symbol already specified for the bindings file");
							return(0);
						}
						if(!$this->GetValidName($bindings_file,$file_element_path,$bindings_file_data["symbol"],"it was not specified a valid bindings class source name"))
							return(0);
						break;
					default:
						$this->SetElementError($bindings_file,$file_element_path,$data["Tag"]."is not a valid bindings file argument");
						return(0);
				}
			}
		}
		if(!IsSet($bindings_file_data["path"]))
		{
			$this->SetElementError($bindings_file,$path,"it were not specified the path of a file that implements the bindings");
			return(0);
		}
		return(1);
	}

	Function ParseBindings(&$parser,$bindings_file,$target_language,&$result)
	{
		$path="0";
		if(GetType($parser->structure[$path])!="array"
		|| strcmp($parser->structure[$path]["Tag"],"bindings"))
		{
			$this->SetElementError($bindings_file,$path,"it was not specified a valid bindings definition file");
			return(0);
		}

		$elements=$parser->structure[$path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$path.",$element";
			$data=$parser->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "language":
						if(count($this->language))
						{
							$this->SetElementError($bindings_file,$element_path,"it was already specified the bindings language");
							return(0);
						}
						$language_elements=$parser->structure[$element_path]["Elements"];
						for($language=array(),$source="",$language_element=0;$language_element<$language_elements;$language_element++)
						{
							$language_element_path=$element_path.",$language_element";
							$data=$parser->structure[$language_element_path];
							if(GetType($data)=="array")
							{
								switch($data["Tag"])
								{
									case "name":
										if(IsSet($language["name"]))
										{
											$this->SetElementError($bindings_file,$language_element_path,"it was already specified the bindings language name");
											return(0);
										}
										if(!$this->GetValidName($bindings_file,$language_element_path,$language["name"],"it was not specified a valid bindings language name"))
											return(0);
										if(strcmp($target_language,$language["name"]))
										{
											$this->SetElementError($bindings_file,$element_path,"it were not specified bindings definitions for the requested target language");
											return(0);
										}
										break;
									case "class":
										if(IsSet($language["class"]))
										{
											$this->SetElementError($bindings_file,$language_element_path,"it was specified bindings class already specified for the language");
											return(0);
										}
										if(!$this->GetValidName($bindings_file,$language_element_path,$language["class"],"it was not specified a valid bindings language class source name"))
											return(0);
										break;
									case "file":
										if(!$this->GetBindingsFileData($parser,$language_element_path,$bindings_file,$bindings_file_data))
											return(0);
										$language["Files"][]=$bindings_file_data;
										break;
									default:
										$this->SetElementError($bindings_file,$language_element_path,"it was not specified a valid language bindings definition argument");
										return(0);
								}
							}
							else
							{
								if(!$this->CheckWhiteSpace($bindings_file,$data,$language_element_path))
									return(0);
							}
						}
						if(!IsSet($language["name"]))
						{
							$this->SetElementError($bindings_file,"0","it was not specified the target language for this bindings definition");
							return(0);
						}
						$this->language=$language;
						$this->CreateFileContext($this->language["context"],$bindings_file);
						$this->language["context"]->path=$element_path;
						break;
					case "class":
						$class_elements=$parser->structure[$element_path]["Elements"];
						for($bindings_values=array(),$source="",$class_element=0;$class_element<$class_elements;$class_element++)
						{
							$class_element_path=$element_path.",$class_element";
							$data=$parser->structure[$class_element_path];
							if(GetType($data)=="array")
							{
								switch($data["Tag"])
								{
									case "source":
										if(!$this->GetValidName($bindings_file,$class_element_path,$source,"it was not specified a valid bindings class source name"))
											return(0);
										if(IsSet($this->bindings[$source]))
										{
											$this->SetElementError($bindings_file,$class_element_path,"it was specified bindings source already specified for the class");
											return(0);
										}
										break;
									case "target":
									case "languagebindings":
										if(IsSet($bindings_values[$data["Tag"]]))
										{
											$this->SetElementError($bindings_file,$class_element_path,"it was specified bindings target already specified for the class");
											return(0);
										}
										if(!$this->GetValidName($bindings_file,$class_element_path,$bindings_values[$data["Tag"]],"it was not specified a valid bindings class source name"))
											return(0);
										break;
									case "file":
										if(!$this->GetBindingsFileData($parser,$class_element_path,$bindings_file,$bindings_file_data))
											return(0);
										$bindings_values["Files"][]=$bindings_file_data;
										break;
									default:
										$this->SetElementError($bindings_file,$class_element_path,"it was not specified a valid class bindings definition argument");
										return(0);
								}
							}
							else
							{
								if(!$this->CheckWhiteSpace($bindings_file,$data,$class_element_path))
									return(0);
							}
						}
						if(!strcmp($source,""))
						{
							$this->SetElementError($bindings_file,$element_path,"it was not specified the source name for the bindings for this class");
							return(0);
						}
						if(!IsSet($bindings_values["Files"]))
						{
							$this->SetElementError($bindings_file,$element_path,"it were not specified any file to implement the bindings");
							return(0);
						}
						$this->bindings[$source]=$bindings_values;
						break;
					default:
						$this->SetElementError($bindings_file,$element_path,"it was not specified a valid bindings definition tag");
						return(0);
				}
			}
			else
			{
				if(!$this->CheckWhiteSpace($bindings_file,$data,$element_path))
					return(0);
			}
		}
		$include_path=$this->GetIncludePath($bindings_file);
		$result[]=array(
			"Data"=>array(
				"Name"=>$include_path.$bindings_file,
				"Type"=>"DEPENDENCY"
			),
			"Type"=>"FILE"
		);
		return(1);
	}

	Function ParseInput(&$parser,$input_file,&$result)
	{
		global $metal_xml_parsers;

		$path="0";
		if(GetType($parser->structure[$path])!="array"
		|| strcmp($parser->structure[$path]["Tag"],"input"))
		{
			$this->SetElementError($input_file,$path,"it was not specified a valid input file");
			return(0);
		}

		$elements=$parser->structure[$path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$path.",$element";
			$data=$parser->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "object":
						$object_elements=$parser->structure[$element_path]["Elements"];
						$this->CreateFileContext($attributes,$input_file);
						$attributes->path="";
						for($name=$class="",$object_element=0;$object_element<$object_elements;$object_element++)
						{
							$object_element_path=$element_path.",$object_element";
							$data=$parser->structure[$object_element_path];
							if(GetType($data)=="array")
							{
								$property=$data["Tag"];
								switch($property)
								{
									case "class":
										if(strcmp($class,""))
										{
											$this->SetElementError($input_file,$object_element_path,"it was attempted to redefine the object class name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$object_element_path,$class,"it was not specified a valid object class name"))
											return(0);
										break;
									case "name":
										if(strcmp($name,""))
										{
											$this->SetElementError($input_file,$object_element_path,"it was attempted to redefine the object name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$object_element_path,$name,"it was not specified a valid object name"))
											return(0);
										break;
									case "attributes":
										if(strcmp($attributes->path,""))
										{
											$this->SetElementError($input_file,$object_element_path,"it was attempted to redefine the object attributes");
											return(0);
										}
										$attributes->path=$object_element_path;
										break;
									default:
										$this->SetElementError($input_file,$object_element_path,"it was not specified a valid input object property ($property)");
										return(0);
								}
							}
							else
							{
								if(!$this->CheckWhiteSpace($input_file,$data,$element_path))
									return(0);
							}
						}
						if(!strcmp($name,""))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the object name");
							return(0);
						}
						if(!strcmp($class,""))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the object class");
							return(0);
						}
						if(!strcmp($attributes->path,""))
						{
							$this->SetElementError($input_file,$element_path,"it were not specified the object attributes");
							return(0);
						}
						if(IsSet($this->objects[$name]))
						{
							$this->SetElementError($input_file,$element_path,"it was specified the name of an already defined object");
							return(0);
						}
						if(!IsSet($this->bindings[$class]))
						{
							$this->SetElementError($input_file,$element_path,"it were not specified any bindings for this object class ($class)");
							return(0);
						}
						$this->objects[$name]=array(
							"class"=>$class,
							"attributes"=>$attributes
						);
						break;
					case "function":
						$function_elements=$parser->structure[$element_path]["Elements"];
						$this->CreateFileContext($arguments,$input_file);
						$arguments->path="";
						for($define_once=0,$names=array(),$object=$call="",$function_element=0;$function_element<$function_elements;$function_element++)
						{
							$function_element_path=$element_path.",$function_element";
							$data=$parser->structure[$function_element_path];
							if(GetType($data)=="array")
							{
								$property=$data["Tag"];
								switch($property)
								{
									case "object":
										if(strcmp($object,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the function object name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$function_element_path,$object,"it was not specified a valid function object name"))
											return(0);
										break;
									case "name":
										if(!$this->GetValidName($input_file,$function_element_path,$names[count($names)],"it was not specified a valid function tag name"))
											return(0);
										break;
									case "call":
										if(strcmp($call,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the function call name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$function_element_path,$call,"it was not specified a valid function call name"))
											return(0);
										break;
									case "defineonce":
										$define_once=1;
										break;
									case "arguments":
										if(strcmp($arguments->path,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the function arguments");
											return(0);
										}
										$arguments->path=$function_element_path;
										break;
									default:
										$this->SetElementError($input_file,$function_element_path,"it was not specified a valid input function property ($property)");
										return(0);
								}
							}
							else
							{
								if(!$this->CheckWhiteSpace($input_file,$data,$element_path))
									return(0);
							}
						}
						if(count($names)==0)
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the function name");
							return(0);
						}
						if(!strcmp($object,""))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the function object");
							return(0);
						}
						if(!strcmp($arguments->path,""))
						{
							$this->SetElementError($input_file,$element_path,"it were not specified the function arguments");
							return(0);
						}
						if(!IsSet($this->objects[$object]))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified a valid object for this function ($object)");
							return(0);
						}
						for($name=0;$name<count($names);$name++)
						{
							if(IsSet($this->functions[$names[$name]]))
							{
								if($define_once)
								{
									if(count($names)>1)
										return($this->SetElementError($input_file,$element_path,"it was defined more than once a function with multiple names"));
								}
								else
								{
									$this->SetElementError($input_file,$element_path,"it was specified the name of an already defined function");
									return(0);
								}
							}
							$this->functions[$names[$name]]=array(
								"object"=>$object,
								"call"=>$call,
								"arguments"=>$arguments
							);
						}
						break;
					case "contextfunction":
						$function_elements=$parser->structure[$element_path]["Elements"];
						$this->CreateFileContext($arguments,$input_file);
						$arguments->path="";
						for($names=array(),$class=$call="",$function_element=0;$function_element<$function_elements;$function_element++)
						{
							$function_element_path=$element_path.",$function_element";
							$data=$parser->structure[$function_element_path];
							if(GetType($data)=="array")
							{
								$property=$data["Tag"];
								switch($property)
								{
									case "class":
										if(strcmp($class,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the context function class name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$function_element_path,$class,"it was not specified a valid function class name"))
											return(0);
										break;
									case "name":
										if(!$this->GetValidName($input_file,$function_element_path,$names[count($names)],"it was not specified a valid context function tag name"))
											return(0);
										break;
									case "call":
										if(strcmp($call,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the context function call name");
											return(0);
										}
										if(!$this->GetValidName($input_file,$function_element_path,$call,"it was not specified a valid context function call name"))
											return(0);
										break;
									case "arguments":
										if(strcmp($arguments->path,""))
										{
											$this->SetElementError($input_file,$function_element_path,"it was attempted to redefine the context function arguments");
											return(0);
										}
										$arguments->path=$function_element_path;
										break;
									default:
										$this->SetElementError($input_file,$function_element_path,"it was not specified a valid input context function property ($property)");
										return(0);
								}
							}
							else
							{
								if(!$this->CheckWhiteSpace($input_file,$data,$element_path))
									return(0);
							}
						}
						if(count($names)==0)
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the context function name");
							return(0);
						}
						if(!strcmp($class,""))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified the context function class");
							return(0);
						}
						if(!strcmp($arguments->path,""))
						{
							$this->SetElementError($input_file,$element_path,"it were not specified the context function arguments");
							return(0);
						}
						if(!IsSet($this->bindings[$class]))
						{
							$this->SetElementError($input_file,$element_path,"it was not specified a valid class for this context function ($class)");
							return(0);
						}
						for($name=0;$name<count($names);$name++)
						{
							if(IsSet($this->context_functions[$names[$name]]))
							{
								$this->SetElementError($input_file,$element_path,"it was specified the name of an already defined context function");
								return(0);
							}
							$this->context_functions[$names[$name]]=array(
								"class"=>$class,
								"call"=>$call,
								"arguments"=>$arguments
							);
							$this->class_context_functions[$class][$call]=$names[$name];
						}
						break;
					case "includeonce":
					case "include":
						if(!$this->GetValidData($input_file,$element_path,$more_input_file,"it was not specified a valid input file name to include"))
							return(0);
						$include_path=$this->GetIncludePath($more_input_file);
						if(!file_exists($include_path.$more_input_file))
						{
							$this->SetElementError($input_file,$element_path,"it was specified an inexisting input file (\"$more_input_file\") to include");
							return(0);
						}
						if(IsSet($this->included_input[$more_input_file]))
						{
							if($data["Tag"]=="includeonce")
								break;
							$include=$this->included_input[$more_input_file];
							$position=$metal_xml_parsers[$include["file"]]->positions[$include["path"]];
							$this->SetElementError($input_file,$element_path,"input file \"$more_input_file\" was already included from file \"".$include["file"]."\" (line ".$position["Line"]." column ".$position["Column"]." byte ".$position["Byte"].")");
							return(0);
						}
						if(!$this->ParseDefinitionFile($more_input_file))
							return(0);
						$this->included_input[$more_input_file]=array(
							"file"=>$input_file,
							"path"=>$element_path
						);
						if(!$this->ParseInput($metal_xml_parsers[$more_input_file],$more_input_file,$result))
							return(0);
						break;
					default:
						$this->SetElementError($input_file,$element_path,"it was not specified a valid input declaration");
						return(0);
				}
			}
			else
			{
				if(!$this->CheckWhiteSpace($input_file,$data,$element_path))
					return(0);
			}
		}
		$include_path=$this->GetIncludePath($input_file);
		$result[]=array(
			"Data"=>array(
				"Name"=>$include_path.$input_file,
				"Type"=>"DEPENDENCY"
			),
			"Type"=>"FILE"
		);
		return(1);
	}

	Function SetupBindingsObject()
	{
		if(!IsSet($this->language["object"]))
		{
			for($files=$this->language["Files"],$file=0;$file<count($files);$file++)
			{
				if(!IsSet($files[$file]["symbol"])
				|| !defined($files[$file]["symbol"]))
				{
					$include_path=$this->GetIncludePath($files[$file]["path"]);
					include($include_path.$files[$file]["path"]);
				}
			}
			$class=$this->language["class"];
			$this->CreateClassObject($this->language["object"],$class);
			if(!$this->DoSetupObject($this->language["object"],$this->language["context"],$this->language["name"]))
			{
				Unset($this->language["object"]);
				return(0);
			}
		}
		return(1);
	}

	Function DoGenerateOutput(&$context,&$bindings,&$data,&$output,$indent,$evaluate)
	{
		return($bindings->GenerateOutput($this,$context,$data,$output,$indent,$evaluate));
	}

	Function GenerateOutput(&$context,&$data,&$output,$indent,$evaluate)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGenerateOutput($context,$this->language["object"],$data,$output,$indent,$evaluate));
	}

	Function DoGetBindingsProperty(&$context,&$bindings,$property,&$value)
	{
		return($bindings->GetBindingsProperty($this,$context,$property,$value));
	}

	Function GetBindingsProperty(&$context,$property,&$value)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGetBindingsProperty($context,$this->language["object"],$property,$value));
	}

	Function DoStartScope(&$context,&$bindings,&$scope,$type)
	{
		return($bindings->StartScope($this,$context,$scope,$type));
	}

	Function StartScope(&$context,&$scope,$type)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoStartScope($context,$this->language["object"],$scope,$type));
	}

	Function DoEndScope(&$context,&$bindings,$scope)
	{
		return($bindings->EndScope($this,$context,$scope));
	}

	Function EndScope(&$context,$scope)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoEndScope($context,$this->language["object"],$scope));
	}

	Function DoGetScopeVariables(&$context,&$bindings,$scope,&$variables)
	{
		return($bindings->GetScopeVariables($this,$context,$scope,$variables));
	}

	Function GetScopeVariables(&$context,$scope,&$variables)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGetScopeVariables($context,$this->language["object"],$scope,$variables));
	}

	Function DoGetScopeExceptions(&$context,&$bindings,$scope,&$exceptions)
	{
		return($bindings->GetScopeExceptions($this,$context,$scope,$exceptions));
	}

	Function GetScopeExceptions(&$context,$scope,&$exceptions)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGetScopeExceptions($context,$this->language["object"],$scope,$exceptions));
	}

	Function DoAllocateTemporaryVariable(&$context,&$bindings,&$variable)
	{
		return($bindings->AllocateTemporaryVariable($this,$context,$variable));
	}

	Function AllocateTemporaryVariable(&$context,&$variable)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoAllocateTemporaryVariable($context,$this->language["object"],$variable));
	}

	Function DoQueryVariable(&$context,&$bindings,&$variable)
	{
		return($bindings->QueryVariable($this,$context,$variable));
	}

	Function QueryVariable(&$context,&$variable)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoQueryVariable($context,$this->language["object"],$variable));
	}

	Function DoDeclareVariable(&$context,&$bindings,&$variable)
	{
		return($bindings->DeclareVariable($this,$context,$variable));
	}

	Function DeclareVariable(&$context,&$variable)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoDeclareVariable($context,$this->language["object"],$variable));
	}

	Function DoDeclareException(&$context,&$bindings,&$exception)
	{
		return($bindings->DeclareException($this,$context,$exception));
	}

	Function DeclareException(&$context,&$exception)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoDeclareException($context,$this->language["object"],$exception));
	}

	Function DoGenerateLocalVariables(&$context,&$bindings,&$variables,&$result)
	{
		return($bindings->GenerateLocalVariables($this,$context,$variables,$result));
	}

	Function GenerateLocalVariables(&$context,&$variables,&$result)
	{
		if(!$this->SetupBindingsObject())
			return(0);
		return($this->DoGenerateLocalVariables($context,$this->language["object"],$variables,$result));
	}

	Function DoObjectInitialization(&$object,&$context,$object_context)
	{
		if(!IsSet($object->initialized[$object_context]))
		{
			if(!$object->Initialization($this,$context,$object_context))
				return(0);
			$object->initialized[$object_context]=1;
		}
		return(1);
	}

	Function OutputObjectInitialization(&$context,$object,$object_context,&$output,$indent,$evaluate)
	{
		if(!$this->SetupObject($object)
		|| !$this->DoObjectInitialization($this->objects[$object]["object"],$context,$object_context)
		|| !$this->GenerateOutput($context,$context->result,$output,$indent,$evaluate))
			return(0);
		return(1);
	}

	Function Compile($arguments,&$result)
	{
		global $metal_xml_parsers,$metal_xml_parser_signatures;

		$this->timer=doubleval($this->strtok(microtime()," "))+doubleval($this->strtok(""));
		$this->signature=md5(uniqid(microtime().rand(0,getrandmax())));
		if(!IsSet($result))
			$result=array();
		if(!IsSet($arguments["TargetLanguage"]))
		{
			$this->error="it was not specified a valid target language";
			return(0);
		}

		if(!IsSet($arguments["Bindings"]))
		{
			$this->error="it was not specified a valid bindings definition file";
			return(0);
		}
		$this->bindings_file=$arguments["Bindings"];

		if(IsSet($arguments["CacheDirectory"]))
			$this->cache_directory=$arguments["CacheDirectory"];

		if(IsSet($arguments["CacheParsedFiles"]))
			$this->cache_parsed_files=$arguments["CacheParsedFiles"];

		if(IsSet($arguments["IncludePath"]))
			$this->include_path=$arguments["IncludePath"];

		if(IsSet($arguments["InsertIncludePath"]))
		{
			$insert_include_path=$arguments["InsertIncludePath"];
			for($path=0;$path<count($this->include_path);$path++)
				$insert_include_path[]=$this->include_path[$path];
			$this->include_path=$insert_include_path;
		}

		if(IsSet($arguments["Arguments"]))
			$this->arguments=$arguments["Arguments"];

		if(!$this->ParseDefinitionFile($this->bindings_file)
		|| !$this->ParseBindings($metal_xml_parsers[$this->bindings_file],$this->bindings_file,$arguments["TargetLanguage"],$result))
			return(0);

		if(!IsSet($arguments["Input"]))
		{
			$this->error="it was not specified a valid input definition file";
			return(0);
		}
		$input_file=$arguments["Input"];
		if(!$this->ParseDefinitionFile($input_file))
			return(0);
		$this->included_input=array(
			$input_file=>array(
				"file"=>$input_file,
				"path"=>"0"
		));
		if(!$this->ParseInput($metal_xml_parsers[$input_file],$input_file,$result))
			return(0);

		if(!IsSet($arguments["Output"]))
		{
			$this->error="it was not specified a valid output definition file";
			return(0);
		}
		if(!$this->ExecuteOutputFile($arguments["Output"],$result))
			return(0);
		for(Reset($metal_xml_parser_signatures);GetType($file=Key($metal_xml_parser_signatures))=="string";Next($metal_xml_parser_signatures))
		{
			if(!strcmp($metal_xml_parser_signatures[$file],$this->signature))
			{
				Unset($metal_xml_parsers[$file]);
				$metal_xml_parser_signatures[$file]="";
			}
		}
		$this->timer=doubleval($this->strtok(microtime()," "))+doubleval($this->strtok(""))-$this->timer;
		return(1);
	}
};

?>