<?php
if(!defined("METAL_FLOW_INCLUDED"))
{
	define("METAL_FLOW_INCLUDED",1);

/*
 * metal_flow.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_flow.php,v 1.34 2004/05/10 20:21:27 mlemos Exp $
 *
 */
 
class metal_flow_class extends metal_base_class
{
	Function GetCommands(&$compiler,&$context,$path,&$commands,$error,$skip_white_space)
	{
		$output=array();
		if(!$compiler->GetOutputData($context->file,$path,$output,$error,$skip_white_space))
			return(0);
		for($command_count=$statement=0;$statement<count($output);$statement++)
		{
			switch($output[$statement]["Type"])
			{
				case "INITIALIZATION":
					$context->result[]=$output[$statement];
					break;
				case "COMMAND":
					$command_count++;
					break;
				default:
					$command_count+=2;
					break;
			}
		}
		if($command_count!=1)
		{
			$commands[]=array(
				"Data"=>"{",
				"Type"=>"COMMAND"
			);
		}
		$commands[]=array(
			"Data"=>1,
			"Type"=>"INDENT"
		);
		for($statement=0;$statement<count($output);$statement++)
		{
			if(strcmp($output[$statement]["Type"],"INITIALIZATION"))
				$commands[]=$output[$statement];
		}
		$commands[]=array(
			"Data"=>1,
			"Type"=>"OUTDENT"
		);
		if($command_count!=1)
		{
			$commands[]=array(
				"Data"=>"}",
				"Type"=>"COMMAND"
			);
		}
		return(1);
	}

	Function GetFunctionArguments(&$compiler,&$arguments,&$arguments_code,&$skip_white_space)
	{
		$skip_white_space=1;
		$elements=$arguments_code->structure[$arguments->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$arguments->path.",$element";
			$data=$arguments_code->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "skipwhitespace":
						if(!$compiler->GetValidData($arguments->file,$element_path,$skip_white_space,"it was not specified a valid loop skipwhitespace argument"))
							return(0);
						if(strcmp($skip_white_space,"1")
						&& strcmp($skip_white_space,"0"))
						{
							$compiler->SetElementError($arguments->file,$element_path,"it was not specified a valid boolean loop skipwhitespace argument value");
							return(0);
						}
						break;
					default:
						$compiler->SetElementError($arguments->file,$element_path,$data["Tag"]." is not a value flow object loop argument");
						return(0);
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($arguments->file,$data,$element_path))
					return(0);
			}
		}
		return(1);
	}

	Function GetMacroArguments(&$compiler,&$code,&$context,&$macro,$arguments)
	{
		$elements=$code->structure[$context->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$context->path.",$element";
			$data=$code->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "do":
					case "arguments":
					case "context":
						if(IsSet($macro[$data["Tag"]]))
						{
							$compiler->SetElementError($context->file,$element_path,"it was defined the flow macro ".$data["Tag"]." argument more than once");
							return(0);
						}
						$macro[$data["Tag"]]=array(
							"path"=>$element_path,
							"file"=>$context->file
						);
						break;
					case "singleargument":
					case "onlyarguments":
						if($arguments)
						{
							if(!strcmp($data["Tag"],"singleargument"))
							{
								if(!$compiler->GetValidName($context->file,$element_path,$macro[$data["Tag"]],"it was not specified a valid flow macro ".$data["Tag"]." name"))
									return(0);
							}
							else
							{
								if(!$compiler->GetValidData($context->file,$element_path,$macro[$data["Tag"]],"it was not specified a valid flow macro ".$data["Tag"]." argument"))
									return(0);
								if(strcmp($macro[$data["Tag"]],"1")
								&& strcmp($macro[$data["Tag"]],"0"))
								{
									$compiler->SetElementError($context->file,$element_path,"it was not specified a valid boolean flow macro ".$data["Tag"]." argument value");
									return(0);
								}
							}
							break;
						}
					default:
						$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a supported flow macro attribute");
						return(0);
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
					return(0);
			}
		}
		return(1);
	}

	Function GetDoCommands(&$compiler,&$context,$path,&$commands,$error,$skip_white_space)
	{
		$do_commands=array();
		if(!$compiler->GetOutputData($context->file,$path,$do_commands,$error,$skip_white_space))
			return(0);
		for($command=0;$command<count($do_commands);$command++)
		{
			switch($do_commands[$command]["Type"])
			{
				case "INITIALIZATION":
					$context->result[]=$do_commands[$command];
					break;
				case "DEFERRED":
					switch($do_commands[$command]["Data"]["Command"])
					{
						case "break":
						case "continue":
							if((++$do_commands[$command]["Data"]["CurrentLevel"])==$do_commands[$command]["Data"]["Level"])
							{
								$message=array(
									"commands"=>array(),
									"level"=>$do_commands[$command]["Data"]["Level"]
								);
								if(!$this->language_bindings->Execute($compiler,$context,$this,$do_commands[$command]["Data"]["Command"],$message))
									return(0);
								for($statement=0;$statement<count($message["commands"]);$statement++)
									$commands[]=$message["commands"][$statement];
								break 2;
							}
					}
				default:
					$commands[]=$do_commands[$command];
					break;
			}
		}
		return(1);
	}

	Function Execute(&$compiler,&$code,$function,&$context,&$arguments_code,&$arguments)
	{
		switch($compiler->language["name"])
		{
			case "Java":
				switch($function)
				{
					case "loop":
					case "switch":
					case "break":
					case "continue":
					case "if":
					case "macro":
					case "macroargument":
					case "comment":
						break;
					default:
						$compiler->SetElementError($context->file,$context->path,"$function function of the flow class is not implemented for the language ".$compiler->language["name"]);
						return(0);
				}
				break;
			case "PHP":
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$compiler->language["name"]." is not a language supported by the flow class");
				return(0);
		}
		switch($function)
		{
			case "loop":
				$message=array(
					"loopstart"=>0,
					"commands"=>array()
				);
				if(!$this->GetFunctionArguments($compiler,$arguments,$arguments_code,$message["skipwhitespace"]))
					return(0);
				$elements=$code->structure[$context->path]["Elements"];
				for($element=0;$element<$elements;$element++)
				{
					$element_path=$context->path.",$element";
					$data=$code->structure[$element_path];
					if(GetType($data)=="array")
					{
						switch($data["Tag"])
						{
							case "while":
								$compiler->CopyEmptyContext($context,$evaluate_context);
								$evaluate_context->path=$element_path;
								if(!$compiler->GetValidExpression($evaluate_context,"BOOLEAN","NOTLEFT","it was not specified a valid flow while BOOLEAN expression",$message["skipwhitespace"])
								|| !$compiler->GetExpressionValue($context,$evaluate_context->result,$message["condition"],$expression_type))
									return(0);
								if(!$this->language_bindings->Execute($compiler,$context,$this,"loop.while",$message))
									return(0);
								break;
							case "do":
								$message["path"]=$element_path;
								if(!$this->language_bindings->Execute($compiler,$context,$this,"loop.do",$message))
									return(0);
								break;
							default:
								$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a valid flow object loop argument");
								return(0);
						}
					}
					else
					{
						if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
							return(0);
					}
				}
				if(!$this->language_bindings->Execute($compiler,$context,$this,"loop.end",$message))
					return(0);
				for($statement=0;$statement<count($message["commands"]);$statement++)
					$context->result[]=$message["commands"][$statement];
				break;
			case "break":
			case "continue":
				$context->result[]=array(
					"Data"=>array(
						"Command"=>$function,
						"Level"=>1,
						"CurrentLevel"=>0,
						"File"=>$context->file,
						"Line"=>$code->positions[$context->path]["Line"],
						"Column"=>$code->positions[$context->path]["Column"],
						"Byte"=>$code->positions[$context->path]["Byte"]
					),
					"Type"=>"DEFERRED"
				);
				break;
			case "if":
				$message=array(
					"commands"=>array()
				);
				if(!$this->GetFunctionArguments($compiler,$arguments,$arguments_code,$message["skipwhitespace"]))
					return(0);
				$elements=$code->structure[$context->path]["Elements"];
				for($condition_set=$then_set=$else_set=$element=0;$element<$elements;$element++)
				{
					$element_path=$context->path.",$element";
					$data=$code->structure[$element_path];
					if(GetType($data)=="array")
					{
						switch($data["Tag"])
						{
							case "istrue":
							case "isfalse":
								if($condition_set)
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified more than one condition block for the if function");
									return(0);
								}
								$compiler->CopyEmptyContext($context,$evaluate_context);
								$evaluate_context->path=$element_path;
								if(!$compiler->GetValidExpression($evaluate_context,"BOOLEAN","NOTLEFT","it was not specified a valid flow if BOOLEAN expression",$message["skipwhitespace"]))
									return(0);
								if(!$compiler->GetExpressionValue($context,$evaluate_context->result,$message[$data["Tag"]],$expression_type))
									return(0);
								$condition_set=1;
								break;
							case "then":
								if(!$condition_set)
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified a then block before setting the condition block for the if function");
									return(0);
								}
								if($then_set)
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified more than one then block for the if function");
									return(0);
								}
								$message["then"]=$element_path;
								$then_set=1;
								break;
							case "else":
								if(!$then_set)
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified a else block before setting the then and the condition block for the if function");
									return(0);
								}
								if($else_set)
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified more than one else block for the if function");
									return(0);
								}
								$message["else"]=$element_path;
								$else_set=1;
								break;
							default:
								$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a value flow object if argument");
								return(0);
						}
					}
					else
					{
						if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
							return(0);
					}
				}
				if(!$condition_set)
				{
					$compiler->SetElementError($context->file,$context->path,"it was not specified the condition block for the if function");
					return(0);
				}
				if(!$then_set)
				{
					$compiler->SetElementError($context->file,$element_path,"it was not specified the then block for the if function");
					return(0);
				}	
				if(!$this->language_bindings->Execute($compiler,$context,$this,"if",$message))
					return(0);
				for($statement=0;$statement<count($message["commands"]);$statement++)
					$context->result[]=$message["commands"][$statement];
				break;
			case "macro":
				$macro=array("onlyarguments"=>"0");
				if(!$this->GetMacroArguments($compiler,$arguments_code,$arguments,$macro,1)
				|| (!IsSet($macro["singleargument"])
				&& !strcmp($macro["onlyarguments"],"0")
				&& !$this->GetMacroArguments($compiler,$code,$context,$macro,0)))
					return(0);
				if(!IsSet($macro["do"]))
				{
					$compiler->SetElementError($context->file,$context->path,"it was not specified the flow macro do section");
					return(0);
				}
				if(IsSet($macro["context"]))
				{
					if(!$compiler->GetValidName($macro["context"]["file"],$macro["context"]["path"],$context_name,"it was not specified a valid flow macro context"))
						return(0);
					if(IsSet($this->macros[$context_name]))
					{
						$compiler->SetElementError($macro["context"]["file"],$macro["context"]["path"],"it was specified a flow macro context of an eventually nested macro");
						return(0);
					}
					$this->macros[$context_name]=array();
					if(IsSet($macro["singleargument"]))
					{
						$this->macros[$context_name][$macro["singleargument"]]=array(
							"file"=>$context->file,
							"path"=>$context->path
						);
					}
					else
					{
						if(IsSet($macro["arguments"])
						|| !strcmp($macro["onlyarguments"],"1"))
						{
							if(!strcmp($macro["onlyarguments"],"1"))
							{
								$file=$context->file;
								$path=$context->path;
							}
							else
							{
								$file=$macro["arguments"]["file"];
								$path=$macro["arguments"]["path"];
							}
							$elements=$code->structure[$path]["Elements"];
							for($element=0;$element<$elements;$element++)
							{
								$element_path="$path,$element";
								$data=$code->structure[$element_path];
								if(GetType($data)=="array")
								{
									$this->macros[$context_name][$data["Tag"]]=array(
										"file"=>$file,
										"path"=>$element_path
									);
								}
								else
								{
									if(!$compiler->CheckWhiteSpace($file,$data,$element_path))
										return(0);
								}
							}
						}
					}
				}
				else
				{
					if(IsSet($macro["arguments"]))
					{
						$compiler->SetElementError($context->file,$context->path,"it were specified flow macro arguments but it was not specified the macro context");
						return(0);
					}
					$context_name="";
				}
				$functions=array(
					"macroargument"
				);
				if(!$compiler->AddContextFunctions($context,$this->object_name,$functions,$level))
					return(0);
				if(!$compiler->GetOutputData($macro["do"]["file"],$macro["do"]["path"],$context->result,"it was not defined valid macro data",1))
					return(0);
				if(!$compiler->RemoveContextFunctions($context,$this->object_name,$level))
					return(0);
				if(strcmp($context_name,""))
					Unset($this->macros[$context_name]);
				break;
			case "macroargument":
				$types=array(
					"argument"=>"DATA",
					"context"=>"DATA"
				);
				$optional_types=array(
					"optional"=>"BOOLEANDATA",
					"skipwhitespace"=>"DATA"
				);
				$values=array();
				if(!$compiler->GetOptionalArgumentValues($arguments,$values,$types,$optional_types,"flow",$function,0)
				|| !$compiler->GetOptionalArgumentValues($context,$values,$types,$optional_types,"flow",$function,1))
					return(0);
				if(!IsSet($this->macros[$values["context"]]))
				{
					$compiler->SetElementError($context->file,$context->path,"it was not specified a defined flow macroargument context");
					return(0);
				}
				$macro_context=$this->macros[$values["context"]];
				if(!IsSet($macro_context[$values["argument"]]))
				{
					if(IsSet($values["optional"])
					&& $values["optional"])
						break;
					$compiler->SetElementError($context->file,$context->path,"it was specified a flow macroargument argument (\"".$values["argument"]."\") that was not defined");
					return(0);
				}
				$macro_argument=$macro_context[$values["argument"]];
				$skip_white_space=(!IsSet($values["skipwhitespace"]) || strcmp($values["skipwhitespace"],"0"));
				if(IsSet($this->macros[$values["context"]][$values["argument"]]["invoked"]))
				{
					$compiler->SetElementError($context->file,$context->path,"the specified a flow macroargument argument (\"".$values["argument"]."\") is invoking itself");
					return(0);
				}
				$this->macros[$values["context"]][$values["argument"]]["invoked"]=1;
				$result=$compiler->GetOutputData($macro_argument["file"],$macro_argument["path"],$context->result,"it was not defined valid macroargument data",$skip_white_space);
				Unset($this->macros[$values["context"]][$values["argument"]]["invoked"]);
				if(!$result)
					return(0);
				break;
			case "switch":
				$message=array(
					"valuetype"=>"",
					"case"=>array()
				);
				$elements=$code->structure[$context->path]["Elements"];
				for($element=0;$element<$elements;$element++)
				{
					$element_path=$context->path.",$element";
					$data=$code->structure[$element_path];
					if(GetType($data)=="array")
					{
						switch($data["Tag"])
						{
							case "value":
								if(strcmp($message["valuetype"],""))
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified the flow object switch value argument more than once");
									return(0);
								}
								$compiler->CopyEmptyContext($context,$evaluate_context);
								$evaluate_context->path=$element_path;
								if(!$compiler->GetValidExpression($evaluate_context,"ANY","NOTLEFT","it was not specified a valid flow switch value argument",1)
								|| !$compiler->GetExpressionValue($context,$evaluate_context->result,$message["value"],$message["valuetype"]))
									return(0);
								switch($message["valuetype"])
								{
									case "STRING":
									case "INTEGER":
										break;
									default:
										$compiler->SetElementError($context->file,$element_path,"it was specified a flow switch value argument expression of a type (\"".$message["valuetype"]."\") that is neither INTEGER nor STRING");
										return(0);
								}
								break;
							case "case":
							case "default":
								if(!strcmp($message["valuetype"],""))
								{
									$compiler->SetElementError($context->file,$element_path,"it was specified the flow object switch ".$data["Tag"]." argument before defining the value argument");
									return(0);
								}
								$case=array();
								$case_elements=$code->structure[$element_path]["Elements"];
								for($case_element=0;$case_element<$case_elements;$case_element++)
								{
									$case_element_path=$element_path.",$case_element";
									$case_data=$code->structure[$case_element_path];
									if(GetType($case_data)=="array")
									{
										switch($case_data["Tag"])
										{
											case "do":
												if(IsSet($case["do"]))
												{
													$compiler->SetElementError($context->file,$case_element_path,"it was specified the flow object switch ".$data["Tag"]." do argument more than once");
													return(0);
												}
												$case["do"]=$case_element_path;
												break;
											case "value":
												if($data["Tag"]=="case")
												{
													if(IsSet($case["value"]))
													{
														$compiler->SetElementError($context->file,$case_element_path,"it was specified the flow object switch case value argument more than once");
														return(0);
													}
													$compiler->CopyEmptyContext($context,$evaluate_context);
													$evaluate_context->path=$case_element_path;
													if(!$compiler->GetValidData($context->file,$case_element_path,$value,"it was not specified a valid loop skipwhitespace argument"))
														return(0);
													if(IsSet($message["case"][$value]))
													{
														$compiler->SetElementError($context->file,$case_element_path,"it was specified flow object switch case for value \"$value\" more than once");
														return(0);
													}
													if(!$compiler->ConvertData($evaluate_context,$message["valuetype"],$value,$case["value"]))
														return(0);
													break;
												}
											default:
												$compiler->SetElementError($context->file,$case_element_path,$case_data["Tag"]." is not a flow object switch ".$data["Tag"]." argument");
												return(0);
										}
									}
									else
									{
										if(!$compiler->CheckWhiteSpace($context->file,$case_data,$case_element_path))
											return(0);
									}
								}
								if(($data["Tag"]=="case"
								&& !IsSet($case[$argument="value"]))
								|| !IsSet($case[$argument="do"]))
								{
									$compiler->SetElementError($context->file,$element_path,"it was not a specified the flow switch ".$data["Tag"]." $argument argument");
									return(0);
								}
								if($data["Tag"]=="case")
									$message["case"][$value]=$case;
								else
									$message["default"]=$case["do"];
								break;
							default:
								$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a flow object switch argument");
								return(0);
						}
					}
					else
					{
						if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
							return(0);
					}
				}
				if(count($message["case"])==0
				&& !IsSet($message["default"]))
				{
					$compiler->SetElementError($context->file,$element_path,"it were not specified any flow object switch cases");
					return(0);
				}
				if(!$this->language_bindings->Execute($compiler,$context,$this,"switch",$message))
					return(0);
				break;
			case "comment":
				$message=array(
				);
				$types=array(
				);
				$optional_types=array(
					"linesbefore"=>"INTEGER",
					"linesafter"=>"INTEGER",
					"columns"=>"INTEGER"
				);
				if(!$compiler->GetOptionalArgumentValues($arguments,$message,$types,$optional_types,"flow",$function,1)
				|| !$compiler->GetValidData($context->file,$context->path,$message["comment"],"it was not specified a valid comment"))
					return(0);
				if(!IsSet($message["linesbefore"]))
					$message["linesbefore"]=0;
				if(!IsSet($message["linesafter"]))
					$message["linesafter"]=0;
				if(!IsSet($message["columns"]))
					$message["columns"]=0;
				if(!$this->language_bindings->Execute($compiler,$context,$this,$function,$message))
					return(0);
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$function." is not a supported flow object function");
				return(0);
		}
		return(1);
	}

};
 
}
?>