<?php
if(!defined("METAL_PERL_BINDINGS_INCLUDED"))
{
	define("METAL_PERL_BINDINGS_INCLUDED",1);

/*
 * metal_perl_bindings.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_perl_bindings.php,v 1.4 2003/03/09 19:55:16 mlemos Exp $
 *
 */
 
class metal_perl_bindings_class extends metal_base_bindings_class
{
	Function ConvertTypedData(&$compiler,&$context,$type,$data)
	{
		switch($type)
		{
			case "STRING":
				for($converted_data="\"",$position=0;$position<strlen($data);$position++)
				{
					switch($character=$data[$position])
					{
						case "\r":
							$converted_data.="\\r";
							break;
						case "\n":
							$converted_data.="\\n";
							break;
						case "\t":
							$converted_data.="\\t";
							break;
						case "\$":
							$converted_data.="\\\$";
							break;
						case "\"":
							$converted_data.="\\\"";
							break;
						case "\\":
							$converted_data.="\\\\";
							break;
						default:
							$order=Ord($character);
							if($order<32
							|| $order>127)
								$converted_data.=sprintf("\\x{%X}",$order);
							else
								$converted_data.=$character;
							break;
					}
				}
				$converted_data.="\"";
				break;
			case "INTEGER":
				if(strval(intval($data))!=$data)
				{
					$compiler->SetElementError($context->file,$context->path,"\"$data\" is not a valid INTEGER value");
					return(0);
				}
				$converted_data="$data";
				break;
			case "FLOAT":
				if(strval(doubleval($data))!=$data)
				{
					$compiler->SetElementError($context->file,$context->path,"\"$data\" is not a valid FLOAT value");
					return(0);
				}
				$converted_data="$data";
				break;
			case "BOOLEAN":
				switch(strtolower($data))
				{
					case "1":
					case "true":
						$converted_data="1";
						break;
					case "0":
					case "false":
						$converted_data="0";
					break;
					default:
						$compiler->SetElementError($context->file,$context->path,"\"$data\" is not a valid BOOLEAN value");
						return(0);
				}
				break;
			case "HASH":
			case "ARRAY":
				if(strcmp($data,""))
				{
					$compiler->SetElementError($context->file,$context->path,"\"$data\" is not a valid $type value");
					return(0);
				}
				$converted_data="()";
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$type." is not a valid type to be converted from data");
				return(0);
		}
		$context->result[]=array(
			"Data"=>array(
				"Value"=>$converted_data,
				"Type"=>$type,
				"Side"=>"RIGHT"
			),
			"Type"=>"EXPRESSION"
		);
		return(1);
	}

	Function GenerateCommand(&$compiler,&$context,$evaluate,&$output,$indent,&$part)
	{
		if($evaluate)
		{
			$compiler->SetElementError($context->file,$context->path,"evaluation of the output of a command is not supported for language ".$compiler->language["name"]);
			return(0);
		}
		$output.=$indent.$part["Data"]."\n";
		return(1);
	}

	Function GenerateOutput(&$compiler,&$context,$data,&$output,$indent,$evaluate)
	{
		$compiler->CopyEmptyContext($context,$output_context);
		for($part=0;$part<count($data);$part++)
		{
			switch($data[$part]["Type"])
			{
				case "INITIALIZATION":
					$this->nesting_level++;
					$compiler->CopyEmptyContext($context,$initialization_context);
					$success=$compiler->OutputObjectInitialization($initialization_context,$data[$part]["Data"]["Object"],$data[$part]["Data"]["Context"],$output,$indent,$evaluate);
					$this->nesting_level--;
					if(!$success)
						return(0);
					for($output_part=0;$output_part<count($initialization_context->result);$output_part++)
						$output_context->result[]=$initialization_context->result[$output_part];
					break;
				case "DATA":
					$output.=$data[$part]["Data"];
					break;
				case "COMMAND":
					if(!$this->GenerateCommand($compiler,$context,$evaluate,$output,$indent,$data[$part]))
						return(0);
					break;
				case "EXPRESSION":
					if($evaluate)
					{
						$compiler->SetElementError($context->file,$context->path,"evaluation of the output of an expression is not supported for language ".$compiler->language["name"]);
						return(0);
					}
					else
					{
						switch($data[$part]["Data"]["Type"])
						{
							case "STRING":
							case "INTEGER":
							case "FLOAT":
							case "BOOLEAN":
								break;
							default:
								$compiler->SetElementError($context->file,$context->path,$data[$part]["Data"]["Type"]." is not a valid expression type to output");
								return(0);				
						}
						$output.=$indent."print(".$data[$part]["Data"]["Value"].");\n";
					}
					break;
				case "INDENT":
					for($level=$data[$part]["Data"];$level;$indent.="\t",$level--);
					break;
				case "OUTDENT":
					$indent=substr($indent,0,strlen($indent)-$data[$part]["Data"]);
					break;
				case "DEFERRED":
					$command_data=$data[$part]["Data"];
					$error="it was specified a misplaced ".$command_data["Command"]." command";
					if(IsSet($command_data["File"])
					&& IsSet($command_data["Line"])
					&& IsSet($command_data["Column"])
					&& IsSet($command_data["Byte"]))
						$compiler->SetError($command_data["File"],$command_data["Line"],$command_data["Column"],$command_data["Byte"],$error);
					else
						$compiler->SetElementError($context->file,$context->path,$error);
					return(0);
				case "FILE":
					if(!file_exists($data[$part]["Data"]["Name"]))
					{
						$compiler->SetElementError($context->file,$context->path,"it was specified a ".$data[$part]["Data"]["Type"]." file (\"".$data[$part]["Data"]["Name"]."\") that does not exist");
						return(0);
					}
					$output_context->result[]=$data[$part];
					break;
				case "RESETINDENT":
					$this->indent_stack[count($this->indent_stack)]=$indent;
					for($indent="",$indent_level=$data[$part]["Data"];$indent_level;$indent.="\t",$indent_level--);
					break;
				case "RESTOREINDENT":
					if(count($this->indent_stack)==0)
					{
						$compiler->SetElementError($context->file,$context->path,"it was specified a RESTOREINDENT command when there is no indentation text values on the stack");
						return(0);
					}
					$indent=$this->indent_stack[count($this->indent_stack)-1];
					Unset($this->indent_stack[count($this->indent_stack)-1]);
					break;
				case "CONTROL":
					if($data[$part]["Data"]["Type"]!="FLUSH")
					{
						$compiler->SetElementError($context->file,$context->path,$data[$part]["Data"]["Type"]." is not a supported type of CONTROL");
						return(0);
					}
					$command=array(
						"Data"=>"\$|=1;",
						"Type"=>"COMMAND"
					);
					if(!$this->GenerateCommand($compiler,$context,$evaluate,$output,$indent,$command))
						return(0);
					$command=array(
						"Data"=>"\$|=0;",
						"Type"=>"COMMAND"
					);
					if(!$this->GenerateCommand($compiler,$context,$evaluate,$output,$indent,$command))
						return(0);
					break;
				default:
					$compiler->SetElementError($context->file,$context->path,$data[$part]["Type"]." is not a supported output type");
					return(0);
			}
		}
		$context->result=$output_context->result;
		return(1);
	}

	Function GetBindingsProperty(&$compiler,&$context,$property,&$value)
	{
		switch($property)
		{
			case "filenameextension":
				$value=".pl";
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,"\"$property\" is not a supported bindings property");
				return(0);
		}
		return(1);
	}
};
 
}
?>