<?php
if(!defined("METAL_PERSISTENCE_INCLUDED"))
{
	define("METAL_PERSISTENCE_INCLUDED",1);

/*
 * metal_persistence.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_persistence.php,v 1.140 2005/05/27 20:04:51 mlemos Exp $
 *
 */

class metal_persistence_class extends metal_base_class
{
	var $generator;
	var $factory_generator;
	var $schema_generator;
	var $form_generator;
	var $report_generator;
	var $components=array();
	var $component_name="";
	var $dependencies=array();
	var $current_class="";
	var $current_form="";
	var $definitions="";
	var $requirements=array();
	var $themes=array();

	var $priorities=array(
		"multiplyby"=>4,
		"divideby"=>4,
		"plus"=>3,
		"minus"=>3,
		"equalto"=>2,
		"differentfrom"=>2,
		"morethan"=>2,
		"lessthan"=>2,
		"moreorequalthan"=>2,
		"lessorequalthan"=>2,
		"and"=>1,
		"or"=>1
	);

	Function ConcatenateResults(&$result, &$concatenate)
	{
		for($part=0;$part<count($concatenate);$part++)
			$result[]=$concatenate[$part];
	}

	Function EvaluateOperation(&$compiler, &$context, &$left, &$right, &$operator, $node)
	{
		$left_type=$left["Data"]["Type"];
		$right_type=$right["Data"]["Type"];
		switch($operator["Data"]["Type"])
		{
			case "multiplyby":
			case "divideby":
			case "plus":
			case "minus":
			case "equalto":
			case "differentfrom":
			case "and":
			case "or":
			case "morethan":
			case "lessthan":
			case "moreorequalthan":
			case "lessorequalthan":
				switch($operator["Data"]["Type"])
				{
					case "equalto":
						$type="ANY";
						$operation="=";
						$result_type="boolean";
						break;
					case "differentfrom":
						$type="ANY";
						$operation="<>";
						$result_type="boolean";
						break;
					case "and":
						$type="boolean";
						$operation=" AND ";
						$result_type="boolean";
						break;
					case "or":
						$type="boolean";
						$operation=" OR ";
						$result_type="boolean";
						break;
					case "morethan":
					case "lessthan":
					case "moreorequalthan":
					case "lessorequalthan":
						switch($left_type)
						{
							case "integer":
							case "text":
							case "float":
							case "decimal":
							case "time":
							case "date":
							case "timestamp":
								$type=$left_type;
								break;
							default:
								$compiler->SetElementError($operator["Data"]["File"],$operator["Data"]["Path"],$operator["Data"]["Type"]." operator does not support ".$left_type." operands");
								return(0);
						}
						$result_type="boolean";
						switch($operator["Data"]["Type"])
						{
							case "morethan":
								$operation=">";
								break;
							case "lessthan":
								$operation="<";
								break;
							case "moreorequalthan":
								$operation=">=";
								break;
							case "lessorequalthan":
								$operation="<=";
								break;
						}
						break;
					case "multiplyby":
					case "divideby":
					case "plus":
					case "minus":
						switch($left_type)
						{
							case "integer":
							case "float":
							case "decimal":
								$type=$left_type;
								break;
							default:
								$compiler->SetElementError($operator["Data"]["File"],$operator["Data"]["Path"],$operator["Data"]["Type"]." operator does not support ".$left_type." operands");
								return(0);
						}
						$result_type=$type;
						switch($operator["Data"]["Type"])
						{
							case "multiplyby":
								$operation="*";
								break;
							case "divideby":
								$operation="/";
								break;
							case "plus":
								$operation="+";
								break;
							case "minus":
								$operation="-";
								break;
						}
						break;
				}
				if(!strcmp($left_type,"ANY"))
					$left_type=$type;
				if(!strcmp($right_type,"ANY"))
					$right_type=$type;
				if(strcmp($left_type,$right_type))
				{
					$compiler->SetElementError($operator["Data"]["File"],$operator["Data"]["Path"],"it were specified expressions of different types ($left_type and $right_type) for the values on each side of the operator ".$operator["Data"]["Type"]);
					return(0);
				}
				if(strcmp($type,"ANY")
				&& strcmp($left_type,$type))
				{
					$compiler->SetElementError($operator["Data"]["File"],$operator["Data"]["Path"],"it were specified non-$type expressions for comparision");
					return(0);
				}
				$operator_code=array(
					array(
						"Data"=>array(
							"Data"=>$operation,
							"Parent"=>$node
						),
						"Type"=>"XMLDATA"
					)
				);
				$this->ConcatenateResults($left["Data"]["Value"],$operator_code);
				$this->ConcatenateResults($left["Data"]["Value"],$right["Data"]["Value"]);
				$left["Data"]["Type"]=$result_type;
				$left["Data"]["Side"]="RIGHT";
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$operator["Data"]["Type"]." is not a supported OPERATOR");
				return(0);
		}
		return(1);
	}

	Function EvaluateExpression(&$compiler, &$context, &$expression, $position, &$result, $node)
	{
		for(;$position<count($expression);$position++)
		{
			switch($expression[$position]["Type"])
			{
				case "INITIALIZATION":
				case "FILE":
					$context->result[]=$expression[$position];
					break;
				case "EXPRESSION":
					if(IsSet($left))
					{
						if(!IsSet($operator)
						|| IsSet($right))
						{
							$compiler->SetElementError($context->file,$context->path,"it was specified an expression with incorrect syntax");
							return(0);
						}
						$right=$expression[$position];
						$right_position=$position;
					}
					else
						$left=$expression[$position];
					break;
				case "OPERATOR":
					if(!IsSet($left))
					{
						$compiler->SetElementError($context->file,$context->path,"it was specified an operator where a expression was expected");
						return(0);
					}
					if(!IsSet($this->priorities[$expression[$position]["Data"]["Type"]]))
					{
						$compiler->SetElementError($context->file,$context->path,"it was specified an operator (".$expression[$position]["Data"]["Type"].") with unknown priority");
						return(0);
					}
					$new_priority=$this->priorities[$expression[$position]["Data"]["Type"]];
					if(IsSet($operator))
					{
						if(IsSet($right))
						{
							if($new_priority>$priority)
							{
								if(!$this->EvaluateExpression($compiler, $context, $expression, $right_position, $right, $node)
								|| !$this->EvaluateOperation($compiler, $context, $left, $right, $operator, $node))
									return(0);
								Unset($operator);
								Unset($right);
								break 2;
							}
							else
							{
								if(!$this->EvaluateOperation($compiler, $context, $left, $right, $operator, $node))
									return(0);
								Unset($operator);
								Unset($right);
								$operator=$expression[$position];
								$priority=$new_priority;
							}
						}
						else
						{
							$compiler->SetElementError($context->file,$context->path,"it were specified two consecutive operators without an expression between them");
							return(0);
						}
					}
					else
					{
						$operator=$expression[$position];
						$priority=$new_priority;
					}
					break;
				default:
					$compiler->SetElementError($context->file,$context->path,"it was specified a ".$expression[$position]["Type"]." within an expression");
					return(0);
			}
		}
		if(IsSet($right))
		{
			if(!IsSet($operator))
			{
				$compiler->SetElementError($context->file,$context->path,"it was specified an incomplete expression");
				return(0);
			}
			if(!$this->EvaluateOperation($compiler, $context, $left, $right, $operator, $node))
				return(0);
			Unset($operator);
			Unset($right);
		}
		if(!IsSet($left))
		{
			$compiler->SetElementError($context->file,$context->path,"it was specified an empty expression");
			return(0);
		}
		if(IsSet($operator))
		{
			$compiler->SetElementError($operator["Data"]["File"],$operator["Data"]["Path"],"it was specified an incomplete expression");
			return(0);
		}
		$result=$left;
		return(1);
	}

	Function ComputeArbitraryFilterExpression(&$compiler, &$parser, &$context, $default_component, &$classes, &$tables, &$function_arguments, &$result, $node, &$expression)
	{
		$expression=array();
		$elements=$parser->structure[$context->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$context->path.",$element";
			$data=$parser->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "isnull":
						$values=array();
						$required_types=array(
							"variable"=>"NAME"
						);
						$optional_types=array(
							"object"=>"NAME"
						);
						$compiler->CopyEmptyContext($context,$isnull_context);
						$isnull_context->path=$element_path;
						if(!$compiler->GetOptionalArgumentValues($isnull_context, $values, $required_types, $optional_types, "persistence", "factory",1))
							return(0);
						$variable_class=(IsSet($values["object"]) ? $values["object"] : "");
						if(!IsSet($classes[$variable_class]["class"]))
							return($compiler->SetElementError($context->file,$element_path,"it was attempted to access a variable of an object of an ambiguous class"));
						$class=$classes[$variable_class]["class"];
						$table=(IsSet($classes[$variable_class]["table"]) ? $classes[$variable_class]["table"]."." : "");
						$component=(IsSet($classes[$variable_class]["component"]) ? $classes[$variable_class]["component"] : $default_component);
						if(!IsSet($this->components[$component]["classes"][$class]["variables"][$values["variable"]]))
							return($compiler->SetElementError($context->file,$element_path,"it was specified an inexisting filter isnull class variable"));
						$expression[]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Data"=>$table.$values["variable"]." IS NULL",
											"Parent"=>$node
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>"boolean",
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "variable":
						$values=array();
						$required_types=array(
						);
						$optional_types=array(
							"name"=>"NAME",
							"object"=>"NAME",
							"oid"=>"DATA"
						);
						$compiler->CopyEmptyContext($context,$name_context);
						$name_context->path=$element_path;
						if(!$compiler->GetOptionalArgumentValues($name_context, $values, $required_types, $optional_types, "persistence", "factory",1))
							return(0);
						$variable_class=(IsSet($values["object"]) ? $values["object"] : "");
						if(!IsSet($classes[$variable_class]["class"]))
							return($compiler->SetElementError($context->file,$element_path,"it was attempted to access a variable of an object of an ambiguous class"));
						$class=$classes[$variable_class]["class"];
						$table=(IsSet($classes[$variable_class]["table"]) ? $classes[$variable_class]["table"]."." : "");
						$component=(IsSet($classes[$variable_class]["component"]) ? $classes[$variable_class]["component"] : $default_component);
						if(IsSet($values["name"]))
						{
							if(!IsSet($this->components[$component]["classes"][$class]["variables"][$values["name"]]))
								return($compiler->SetElementError($context->file,$element_path,$values["name"]." is not existing filter class variable name"));
							if(IsSet($this->components[$component]["classes"][$class]["variables"][$values["name"]]["type"]))
								$type=$this->components[$component]["classes"][$class]["variables"][$values["name"]]["type"];
							else
								$type=(IsSet($values["oid"]) ? "OID:" : "OBJECT:").$this->components[$component]["classes"][$class]["variables"][$values["name"]]["class"];
							$field=$values["name"];
						}
						elseif(IsSet($values["oid"]))
						{
							$type="OID:".$class;
							$field="id";
						}
						else
							return($compiler->SetElementError($context->file,$element_path,"it was not specified the filter class variable name"));
						$expression[]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Data"=>$table.$field,
											"Parent"=>$node
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>$type,
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "count":
						$values=array();
						$required_types=array(
						);
						$optional_types=array(
							"variable"=>"NAME",
							"object"=>"NAME",
							"oid"=>"DATA"
						);
						$compiler->CopyEmptyContext($context,$name_context);
						$name_context->path=$element_path;
						if(!$compiler->GetOptionalArgumentValues($name_context, $values, $required_types, $optional_types, "persistence", "factory",1))
							return(0);
						if(IsSet($values["variable"]))
						{
							$variable_class=(IsSet($values["object"]) ? $values["object"] : "");
							if(!IsSet($classes[$variable_class]["class"]))
								return($compiler->SetElementError($context->file,$element_path,"it was attempted to access a variable of an object of an ambiguous class"));
							$class=$classes[$variable_class]["class"];
							$table=(IsSet($classes[$variable_class]["table"]) ? $classes[$variable_class]["table"]."." : "");
							$component=(IsSet($classes[$variable_class]["component"]) ? $classes[$variable_class]["component"] : $default_component);
							if(!IsSet($this->components[$component]["classes"][$class]["variables"][$values["variable"]]))
								return($compiler->SetElementError($context->file,$element_path,$values["variable"]." is not existing filter class variable name"));
							$column=$table.$values["variable"];
						}
						else
							$column="*";
						$expression[]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Data"=>"COUNT(".$column.")",
											"Parent"=>$node
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>"integer",
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "argument":
						if(!$compiler->GetValidData($context->file, $element_path, $argument, "it was not specified a valid function argument name"))
							return(0);
						if(!IsSet($function_arguments["arguments"]))
							return($compiler->SetElementError($context->file,$element_path,"the function does not take any arguments"));
						if(!IsSet($function_arguments["arguments"][$argument]))
							return($compiler->SetElementError($context->file,$element_path,$argument." is not a valid argument name"));
						$argument_values=array(
							array(
								"Data"=>array(
									"Parent"=>$node,
									"Node"=>($field_value=$compiler->GetXMLNode()),
									"Tag"=>"databaseget".(IsSet($function_arguments["arguments"][$argument]["type"]) ? $function_arguments["arguments"][$argument]["type"] : "integer")."fieldvalue",
									"Indent"=>1
								),
								"Type"=>"XMLTAG"
							),
							array(
								"Data"=>array(
									"Parent"=>$field_value,
									"Node"=>($value_node=$compiler->GetXMLNode()),
									"Tag"=>"value"
								),
								"Type"=>"XMLTAG"
							)
						);
						if(IsSet($function_arguments["arguments"][$argument]["type"]))
						{
							$argument_values[]=array(
								"Data"=>array(
									"Parent"=>$value_node,
									"Tag"=>"argument",
									"Data"=>$argument,
								),
								"Type"=>"XMLTAG"
							);
						}
						else
						{
							if(IsSet($function_arguments["arguments"][$argument]["oid"]))
							{
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$value_node,
										"Node"=>($argument_node=$compiler->GetXMLNode()),
										"Tag"=>"argument",
										"Data"=>$argument
									),
									"Type"=>"XMLTAG"
								);
							}
							else
							{
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$value_node,
										"Node"=>($class_variable_node=$compiler->GetXMLNode()),
										"Tag"=>$function_arguments["arguments"][$argument]["class"]."classvariable",
										"Indent"=>1
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$class_variable_node,
										"Node"=>($object_node=$compiler->GetXMLNode()),
										"Tag"=>"object"
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$object_node,
										"Node"=>($argument_node=$compiler->GetXMLNode()),
										"Tag"=>"argument",
										"Data"=>$argument
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$class_variable_node,
										"Tag"=>"variable",
										"Data"=>"id"
									),
									"Type"=>"XMLTAG"
								);
							}
						}
						$expression[]=array(
							"Data"=>array(
								"Value"=>$argument_values,
								"Type"=>(IsSet($function_arguments["arguments"][$argument]["type"]) ? $function_arguments["arguments"][$argument]["type"] : (IsSet($function_arguments["arguments"][$argument]["oid"]) ? "OID:" : "OBJECT:").$function_arguments["arguments"][$argument]["class"]),
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						$function_arguments["arguments"][$argument]["Uses"]++;
						break;
					case "object":
						$values=array();
						$required_types=array(
							"name"=>"NAME"
						);
						$optional_types=array(
							"class"=>"NAME",
							"component"=>"NAME"
						);
						$compiler->CopyEmptyContext($context,$object_context);
						$object_context->path=$element_path;
						if(!$compiler->GetOptionalArgumentValues($object_context, $values, $required_types, $optional_types, "persistence", "factory",1))
							return(0);
						$name=$values["name"];
						if(IsSet($values["class"]))
						{
							if(IsSet($values["component"]))
							{
								$component=$values["component"];
								if(!IsSet($this->components[$component]))
									return($compiler->SetElementError($context->file,$element_path,$component." is not an existing component"));
							}
							else
								$component=$default_component;
							$class=$values["class"];
							if(!IsSet($this->components[$component]["classes"][$class]))
								return($compiler->SetElementError($context->file,$element_path,$class." is not an existing component class"));
							if(IsSet($classes[$name]))
								return($compiler->SetElementError($context->file,$element_path,"it was attempted to redefine the filter object named \"".$name."\""));
							if(IsSet($tables[$name]))
								return($compiler->SetElementError($context->file,$element_path,"it was attempted to define an object with the name \"".$name."\" that is already used by distinct object database table"));
							$classes[$name]=array(
								"class"=>$class,
								"table"=>$name,
								"component"=>$component
							);
							$tables[$name]=$class;
						}
						elseif(IsSet($classes[$name]))
							$class=$classes[$name]["class"];
						else
							return($compiler->SetElementError($context->file,$element_path,"it was not declared the object with the name \"".$name."\""));
						$expression[]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Parent"=>$node,
											"Data"=>$name.".id"
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>"OBJECT:".$class,
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "text":
						if(!$compiler->GetValidData($context->file, $element_path, $value, "it was not specified a valid constant ".$data["Tag"]." expression"))
							return(0);
						$part=count($expression);
						$expression[$part]=array(
							"Data"=>array(
								"Type"=>"text",
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						if(GetType(strpos($value,"'"))=="integer"
						|| GetType(strpos($value,"\\"))=="integer")
						{
							$expression[$part]["Data"]["Value"]=array(
								array(
									"Data"=>array(
										"Parent"=>$node,
										"Node"=>($field_value=$compiler->GetXMLNode()),
										"Tag"=>"databasegettextfieldvalue",
										"Indent"=>1
									),
									"Type"=>"XMLTAG"
								),
								array(
									"Data"=>array(
										"Parent"=>$field_value,
										"Tag"=>"value",
										"Data"=>$value,
									),
									"Type"=>"XMLTAG"
								)
							);
						}
						else
						{
							$expression[$part]["Data"]["Value"]=array(
								array(
									"Data"=>array(
										"Parent"=>$node,
										"Data"=>"'".$value."'",
									),
									"Type"=>"XMLDATA"
								)
							);
						}
						break;
					case "integer":
						if(!$compiler->GetValidData($context->file, $element_path, $value, "it was not specified a valid constant ".$data["Tag"]." expression"))
							return(0);
						$expression[]=array(
							"Data"=>array(
								"Type"=>$data["Tag"],
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path,
								"Value"=>array(
									array(
										"Data"=>array(
											"Data"=>$value,
											"Parent"=>$node
										),
										"Type"=>"XMLDATA"
									)
								)
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "float":
					case "decimal":
					case "boolean":
					case "date":
					case "time":
					case "timestamp":
						if(!$compiler->GetValidData($context->file, $element_path, $value, "it was not specified a valid constant ".$data["Tag"]." expression"))
							return(0);
						$expression[]=array(
							"Data"=>array(
								"Type"=>$data["Tag"],
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path,
								"Value"=>array(
									array(
										"Data"=>array(
											"Parent"=>$node,
											"Node"=>($field_value=$compiler->GetXMLNode()),
											"Tag"=>"databaseget".$data["Tag"]."fieldvalue",
											"Indent"=>1
										),
										"Type"=>"XMLTAG"
									),
									array(
										"Data"=>array(
											"Parent"=>$field_value,
											"Tag"=>"value",
											"Data"=>$value,
										),
										"Type"=>"XMLTAG"
									)
								)
							),
							"Type"=>"EXPRESSION"
						);
						break;
					case "not":
						$compiler->CopyEmptyContext($context,$not_context);
						$not_context->path=$element_path;
						if(!$this->EvaluateFilterExpression($compiler, $parser, $not_context, $default_component, $classes, $tables, $function_arguments, $not_expression, $node, "boolean"))
							return(0);
						$part=count($expression);
						$expression[$part]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Parent"=>$node,
											"Data"=>"NOT ("
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>"boolean",
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						$this->ConcatenateResults($expression[$part]["Data"]["Value"], $not_expression);
						$expression[$part]["Data"]["Value"][]=array(
							"Data"=>array(
								"Parent"=>$node,
								"Data"=>")"
							),
							"Type"=>"XMLDATA"
						);
						break;
					case "group":
						$compiler->CopyEmptyContext($context,$group_context);
						$group_context->path=$element_path;
						if(!$this->EvaluateArbitraryFilterExpression($compiler, $parser, $group_context, $default_component, $classes, $tables, $function_arguments, $group_expression, $node))
							return(0);
						$part=count($expression);
						$expression[$part]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Parent"=>$node,
											"Data"=>"("
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>$group_expression["Data"]["Type"],
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						$this->ConcatenateResults($expression[$part]["Data"]["Value"], $group_expression["Data"]["Value"]);
						$expression[$part]["Data"]["Value"][]=array(
							"Data"=>array(
								"Parent"=>$node,
								"Data"=>")"
							),
							"Type"=>"XMLDATA"
						);
						break;
					case "maximum":
					case "minimum":
						$compiler->CopyEmptyContext($context,$summary_context);
						$summary_context->path=$element_path;
						if(!$this->EvaluateArbitraryFilterExpression($compiler, $parser, $summary_context, $default_component, $classes, $tables, $function_arguments, $result, $node))
							return(0);
						switch($result["Data"]["Type"])
						{
							case "integer":
							case "decimal":
							case "float":
							case "date":
							case "time":
							case "timestamp":
								break;
							default:
								return($compiler->SetElementError($summary_context->file,$summary_context->path,"the ".$data["Tag"]." expression is of an invalid type (".$result["Data"]["Type"].")"));
						}
						$functions=array(
							"maximum"=>"MAX",
							"minimum"=>"MIN",
						);
						$part=count($expression);
						$expression[$part]=array(
							"Data"=>array(
								"Value"=>array(
									array(
										"Data"=>array(
											"Parent"=>$node,
											"Data"=>$functions[$data["Tag"]]."("
										),
										"Type"=>"XMLDATA"
									)
								),
								"Type"=>"integer",
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						$this->ConcatenateResults($expression[$part]["Data"]["Value"], $result["Data"]["Value"]);
						$expression[$part]["Data"]["Value"][]=array(
							"Data"=>array(
								"Parent"=>$node,
								"Data"=>")"
							),
							"Type"=>"XMLDATA"
						);
						break;
					case "lessthan":
					case "lessorequalthan":
					case "morethan":
					case "moreorequalthan":
					case "equalto":
					case "differentfrom":
					case "and":
					case "or":
					case "multiplyby":
					case "divideby":
					case "plus":
					case "minus":
						$values=array();
						$expression[]=array(
							"Data"=>array(
								"Type"=>$data["Tag"],
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"OPERATOR"
						);
						break;
					default:
						return($compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a valid filter argument"));
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
					return(0);
			}
		}
		return(1);
	}

	Function EvaluateArbitraryFilterExpression(&$compiler, &$parser, &$context, $default_component, &$classes, &$tables, &$function_arguments, &$result, $node)
	{
		if(!$this->ComputeArbitraryFilterExpression($compiler, $parser, $context, $default_component, $classes, $tables, $function_arguments, $result, $node, $expression))
			return(0);
		return($this->EvaluateExpression($compiler,$context,$expression,0,$result,$node));
	}

	Function EvaluateFilterExpression(&$compiler, &$parser, &$context, $default_component, &$classes, &$tables, &$function_arguments, &$filter_expression, $node, $expression_type)
	{
		if(!$this->EvaluateArbitraryFilterExpression($compiler, $parser, $context, $default_component, $classes, $tables, $function_arguments, $result, $node))
			return(0);
		if($result["Data"]["Type"]!=$expression_type)
			return($compiler->SetElementError($context->file,$context->path,"this filter expression is of type ".$result["Data"]["Type"]." and not ".$expression_type." as expected"));
		$filter_expression=$result["Data"]["Value"];
		return(1);
	}

	Function EvaluateFilter(&$compiler, &$parser, &$context, $default_component, &$classes, &$tables, &$function_arguments, &$filter_condition, $node)
	{
		$tables=array();
		return($this->EvaluateFilterExpression($compiler, $parser, $context, $default_component, $classes, $tables, $function_arguments, $filter_condition, $node, "boolean"));
	}

	Function ComputeArbitraryRuntimeExpression(&$compiler, &$parser, &$context, &$function_arguments, &$result, $node, &$expression)
	{
		$expression=array();
		$elements=$parser->structure[$context->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$context->path.",$element";
			$data=$parser->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "argument":
						if(!$compiler->GetValidData($context->file, $element_path, $argument, "it was not specified a valid function argument name"))
							return(0);
						if(!IsSet($function_arguments["arguments"]))
							return($compiler->SetElementError($context->file,$element_path,"the function does not take any arguments"));
						if(!IsSet($function_arguments["arguments"][$argument]))
							return($compiler->SetElementError($context->file,$element_path,$argument." is not a valid argument name"));
						$argument_values=array(
						);
						if(IsSet($function_arguments["arguments"][$argument]["type"]))
						{
							$argument_values[]=array(
								"Data"=>array(
									"Parent"=>$node,
									"Tag"=>"argument",
									"Data"=>$argument,
								),
								"Type"=>"XMLTAG"
							);
						}
						else
						{
							if(IsSet($function_arguments["arguments"][$argument]["oid"]))
							{
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$node,
										"Node"=>($argument_node=$compiler->GetXMLNode()),
										"Tag"=>"argument",
										"Data"=>$argument
									),
									"Type"=>"XMLTAG"
								);
							}
							else
							{
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$node,
										"Node"=>($class_variable_node=$compiler->GetXMLNode()),
										"Tag"=>$function_arguments["arguments"][$argument]["class"]."classvariable",
										"Indent"=>1
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$class_variable_node,
										"Node"=>($object_node=$compiler->GetXMLNode()),
										"Tag"=>"object"
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$object_node,
										"Node"=>($argument_node=$compiler->GetXMLNode()),
										"Tag"=>"argument",
										"Data"=>$argument
									),
									"Type"=>"XMLTAG"
								);
								$argument_values[]=array(
									"Data"=>array(
										"Parent"=>$class_variable_node,
										"Tag"=>"variable",
										"Data"=>"id"
									),
									"Type"=>"XMLTAG"
								);
							}
						}
						$expression[]=array(
							"Data"=>array(
								"Value"=>$argument_values,
								"Type"=>(IsSet($function_arguments["arguments"][$argument]["type"]) ? $function_arguments["arguments"][$argument]["type"] : (IsSet($function_arguments["arguments"][$argument]["oid"]) ? "OID:" : "OBJECT:").$function_arguments["arguments"][$argument]["class"]),
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path
							),
							"Type"=>"EXPRESSION"
						);
						$function_arguments["arguments"][$argument]["Uses"]++;
						break;
					case "integer":
						if(!$compiler->GetValidData($context->file, $element_path, $value, "it was not specified a valid constant ".$data["Tag"]." expression"))
							return(0);
						$expression[]=array(
							"Data"=>array(
								"Type"=>$data["Tag"],
								"Side"=>"RIGHT",
								"File"=>$context->file,
								"Path"=>$element_path,
								"Value"=>array(
									array(
										"Data"=>array(
											"Data"=>$value,
											"Parent"=>$node
										),
										"Type"=>"XMLDATA"
									)
								)
							),
							"Type"=>"EXPRESSION"
						);
						break;
					default:
						return($compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a valid argument"));
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
					return(0);
			}
		}
		return(1);
	}

	Function EvaluateArbitraryRuntimeExpression(&$compiler, &$parser, &$context, &$function_arguments, &$result, $node)
	{
		if(!$this->ComputeArbitraryRuntimeExpression($compiler, $parser, $context, $function_arguments, $result, $node, $expression))
			return(0);
		return($this->EvaluateExpression($compiler,$context,$expression,0,$result,$node));
	}

	Function EvaluateRuntimeExpression(&$compiler, &$parser, &$context, &$function_arguments, &$expression, $node, $expression_type)
	{
		if(!$this->EvaluateArbitraryRuntimeExpression($compiler, $parser, $context, $function_arguments, $result, $node))
			return(0);
		if($result["Data"]["Type"]!=$expression_type)
			return($compiler->SetElementError($context->file,$context->path,"this expression is of type ".$result["Data"]["Type"]." and not ".$expression_type." as expected"));
		$expression=$result["Data"]["Value"];
		return(1);
	}
	
	Function ValidateType(&$compiler,&$context,$component,$values,&$definition)
	{
		if(IsSet($values["type"])
		&& IsSet($values["class"]))
			return($compiler->SetElementError($context->file,$context->path,"it was specified data member to be of simultaneously of an elementar type and a class object reference"));
		if(!IsSet($values["type"])
		&& !IsSet($values["class"]))
			return($compiler->SetElementError($context->file,$context->path,"it was not specified the data member type"));
		if(IsSet($values["type"]))
		{
			if(IsSet($values["component"]))
				return($compiler->SetElementError($context->file,$context->path,"it was defined a variable associated to an external component"));
			$validate=array(
				"integer"=>"^-?[0-9]+\$",
				"text"=>".*",
				"float"=>"^[+-]?[0-9]+(\\.[0-9]*)?(e[+-]?[0-9]+)?\$",
				"decimal"=>"^[+-]?[0-9]+(\\.[0-9]*)?\$",
				"boolean"=>"^0|1|true|false|yes|no\$",
				"time"=>"^([0-9]{2}):([0-9]{2}):([0-9]{2})\$",
				"date"=>"^([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})\$",
				"timestamp"=>"^([0-9]{4})-([0-9]{1,2})-([0-9]{1,2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})\$"
			);
			if(!IsSet($validate[$values["type"]]))
				return($compiler->SetElementError($context->file,$context->path,"it was not specified a valid data member type"));
			if(IsSet($values["optional"])
			&& $values["optional"])
				$definition["optional"]=1;
			if(IsSet($values["initialvalue"]))
			{
				if(IsSet($values[$auto_value="autocreate"])
				|| IsSet($values[$auto_value="autoupdate"]))
					return($compiler->SetElementError($context->file,$context->path,"it was defined a variable with either initialvalue and ".$auto_value));
				$value=$values["initialvalue"];
				if(!eregi($validate[$values["type"]],$value,$matches))
					return($compiler->SetElementError($context->file,$context->path,"it was not specified a valid ".$values["type"]." initial value"));
				switch($values["type"])
				{
					case "time":
						if(intval($matches[1])>=24
						|| intval($matches[2])>=60
						|| intval($matches[3])>=60)
							return($compiler->SetElementError($context->file,$context->path,"it was not specified a valid time initial value"));
						break;
					case "date":
						if(intval($matches[2])<1
						|| intval($matches[2])>12
						|| intval($matches[3])<1
						|| intval($matches[3])>31)
							return($compiler->SetElementError($context->file,$context->path,"it was not specified a valid date initial value"));
						break;
					case "boolean":
						switch(strtolower($value))
						{
							case "true":
							case "yes":
								$value="1";
								break;
							case "false":
							case "no":
								$value="0";
								break;
						}
						break;
				}
				$definition["initialvalue"]=$value;
			}
			elseif(IsSet($values[$auto_value="autocreate"])
			|| IsSet($values[$auto_value="autoupdate"]))
			{
				if(IsSet($values["autocreate"])
				&& IsSet($values["autoupdate"]))
					return($compiler->SetElementError($context->file,$context->path,"it was defined a variable with either autocreate and autoupdate"));
				if(IsSet($definition["optional"]))
					return($compiler->SetElementError($context->file,$context->path,$auto_value." variables may not be optional"));
				$validate=array(
					"utctime"=>array(
						"time"=>1,
						"timestamp"=>1
					),
					"localtime"=>array(
						"time"=>1,
						"timestamp"=>1
					),
					"utcdate"=>array(
						"date"=>1
					),
					"localdate"=>array(
						"date"=>1
					),
				);
				if(!IsSet($validate[$values[$auto_value]][$values["type"]]))
					return($compiler->SetElementError($context->file,$context->path,$values[$auto_value]." is not a valid ".$auto_value." value for a ".$values["type"]." variable"));
				$definition[$auto_value]=$values[$auto_value];
			}
			$definition["type"]=$values["type"];
		}
		else
		{
			if(IsSet($values[$auto_value="initialvalue"])
			|| IsSet($values[$auto_value="autocreate"])
			|| IsSet($values[$auto_value="autoupdate"]))
				return($compiler->SetElementError($context->file,$context->path,"it was defined the reference variable with a ".$auto_value));
			if(IsSet($values["component"]))
			{
				if(!IsSet($this->components[$component]["components"][$values["component"]]))
					return($compiler->SetElementError($context->file,$context->path,"it was defined a reference variable of a class of an unknown external component"));
				$definition["component"]=$values["component"];
			}
			if(IsSet($values["optional"])
			&& $values["optional"])
				$definition["optional"]=1;
			$definition["class"]=$values["class"];
		}
		return(1);
	}

	Function ParseFunctionsDefinition(&$compiler,&$functions,$component,&$supported,&$definition)
	{
		if(count($functions)==0)
		{
			$definition=array();
			return(1);
		}
		for($function=0;$function<count($functions);$function++)
		{
			$function_values=array();
			$required_types=array(
				"name"=>"NAME",
				"type"=>"DATA"
			);
			$optional_types=array(
				"id"=>"DATA",
				"parameters"=>"CONTEXT",
				"argument"=>"CONTEXT:MULTIPLE"
			);
			$compiler->CopyEmptyContext($functions[$function],$function_context);
			if(!$compiler->GetOptionalArgumentValues($function_context,$function_values,$required_types,$optional_types,"persistence","function",1))
				return(0);
			$function_name=$function_values["name"];
			if(IsSet($definition[$function_name]))
				return($compiler->SetElementError($function_context->file,$function_context->path,"it was specified a function with the name \"$function_name\" more than once"));
			if(!IsSet($supported[$function_values["type"]]))
				return($compiler->SetElementError($function_context->file,$function_context->path,$function_values["type"]." is not a supported function type"));
			if(IsSet($function_values["id"])
			&& !$this->SetComponentId($compiler, $function_context, $component, $function_values["id"], $function_name, "function"))
				return(0);
			$definition[$function_name]=array(
				"function"=>$function_context,
				"type"=>$function_values["type"],
			);
			if(IsSet($function_values["argument"]))
			{
				$definition[$function_name]["arguments"]=array();
				$required_types=array(
					"name"=>"NAME",
				);
				$optional_types=array(
					"type"=>"NAME",
					"class"=>"NAME",
					"oid"=>"DATA",
					"component"=>"NAME"
				);
				for($argument=0;$argument<count($function_values["argument"]);$argument++)
				{
					$argument_values=array();
					$compiler->CopyEmptyContext($function_values["argument"][$argument],$argument_context);
					if(!$compiler->GetOptionalArgumentValues($argument_context, $argument_values, $required_types, $optional_types, "persistence", "function", 1))
						return(0);
					$name=$argument_values["name"];
					if(IsSet($definition[$function_name]["arguments"][$name]))
					{
						return($compiler->SetElementError($argument_context->file, $argument_context->path, "it was defined a function argument with the name ".$name." more than once"));
					}
					if(IsSet($argument_values["type"])
					&& IsSet($argument_values["class"]))
						return($compiler->SetElementError($argument_context->file,$argument_context->path,"it was specified simultaneously an elementar type and a class for the argument"));
					if(!IsSet($argument_values["type"])
					&& !IsSet($argument_values["class"]))
						return($compiler->SetElementError($argument_context->file,$argument_context->path,"it was not specified the argument type"));
					$definition[$function_name]["arguments"][$name]=array(
						"Uses"=>0
					);
					if(IsSet($argument_values["type"]))
					{
						$type=$argument_values["type"];
						switch($type)
						{
							case "integer":
							case "text":
							case "float":
							case "decimal":
							case "boolean":
							case "time":
							case "date":
							case "timestamp":
								break;
							default:
								return($compiler->SetElementError($argument_context->file, $argument_context->path,"it was not specified a valid function argument type"));
						}
						$definition[$function_name]["arguments"][$name]["type"]=$type;
					}
					else
					{
						if(IsSet($argument_values["component"]))
						{
							$class_component=$argument_values["component"];
							if(!IsSet($this->components[$class_component]))
							{
								$component_file=$this->components[$component]["components"][$class_component]["file"];
								$this->GetComponentPath($compiler, $function_context->file, $component_path);
								if(!$compiler->ParseDefinitionFileFromPath($component_file,$component_path))
									return(0);
								$compiler->CreateFileContext($component_context,$component_file);
								if(!$this->ParseComponentDefinition($compiler,$component_context,$class_component))
									return(0);
								$this->dependencies[]=$compiler->GetIncludePathFromList($component_file,$component_path).$component_file;
							}
							if(!IsSet($this->components[$class_component]))
								return($compiler->SetElementError($argument_context->file,$argument_context->path,$argument_values["component"]." is not a valid component"));
						}
						else
							$class_component=$component;
						if(!IsSet($this->components[$class_component]["classes"][$argument_values["class"]]))
							return($compiler->SetElementError($argument_context->file,$argument_context->path,$argument_values["class"]." is not a valid class of component ".$class_component));
						$definition[$function_name]["arguments"][$name]["class"]=$argument_values["class"];
						$definition[$function_name]["arguments"][$name]["component"]=$class_component;
						if(IsSet($argument_values["oid"]))
							$definition[$function_name]["arguments"][$name]["oid"]=$argument_values["oid"];
					}
					$compiler->CopyEmptyContext($argument_context, $definition[$function_name]["arguments"][$name]["Context"]);
				}
			}
			if(IsSet($function_values["parameters"]))
				$compiler->CopyEmptyContext($function_values["parameters"],$definition[$function_name]["parameters"]);
			$compiler->CopyEmptyContext($functions[$function],$definition[$function_name]["Context"]);
		}
		return(1);
	}

	Function GetClassGetReferenceParameters(&$compiler,$component,$current_class,&$function_arguments,$function,&$reference,&$reference_class)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"variable"=>"NAME"
		);
		$optional_types=array(
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		$reference=$parameter_values["variable"];
		if(!IsSet($this->components[$component]["classes"][$current_class]["variables"][$reference]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$reference." is not a valid component class variable"));
		if(!IsSet($this->components[$component]["classes"][$current_class]["variables"][$reference]["class"]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$reference." is not a valid component class reference variable"));
		if(IsSet($this->components[$component]["classes"][$current_class]["variables"][$reference]["component"]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,"getreference of a variable of a class of a different component is not yet supported"));
		$reference_class=$this->components[$component]["classes"][$current_class]["variables"][$reference]["class"];
		return(1);
	}

	Function GetClassGetCollectionParameters(&$compiler,$component,$current_class,&$function_arguments,$function,&$collection_reference,&$collection_class,&$collection_component, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"collection"=>"NAME"
		);
		$optional_types=array(
			"filter"=>"CONTEXT",
			"object"=>"CONTEXT:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		$collection=$parameter_values["collection"];
		if(!IsSet($this->components[$component]["classes"][$current_class]["collections"][$collection]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$collection." is not a valid component class collection"));
		$collection_reference=$this->components[$component]["classes"][$current_class]["collections"][$collection]["reference"];
		$collection_class=$this->components[$component]["classes"][$current_class]["collections"][$collection]["class"];
		$collection_component=(IsSet($this->components[$component]["classes"][$current_class]["collections"][$collection]["component"]) ? $this->components[$component]["classes"][$current_class]["collections"][$collection]["component"] : $component);
		return(1);
	}

	Function SetComponentId(&$compiler, &$context, $component, $id, $name, $type)
	{
		if(IsSet($this->components[$component]["id"][$id]))
			return($compiler->SetElementError($context->file, $context->path, "the id of the ".$type." ".$name." is set to \"".$id."\" which is the same of the ".$this->components[$component]["id"][$id]["type"]." ".$this->components[$component]["id"][$id]["name"]));
		$this->components[$component]["id"][$id]=array(
			"name"=>$name,
			"type"=>$type
		);
		return(1);
	}

	Function ParseComponentClass(&$compiler,$component,&$class_context)
	{
		$class_values=array();
		$required_types=array(
			"name"=>"NAME"
		);
		$optional_types=array(
			"id"=>"DATA",
			"variable"=>"CONTEXT:MULTIPLE",
			"collection"=>"CONTEXT:MULTIPLE",
			"function"=>"CONTEXT:MULTIPLE",
			"validation"=>"CONTEXT:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($class_context,$class_values,$required_types,$optional_types,"persistence","class",1))
			return(0);
		if(!strcmp($class_values["name"],$component))
			return($compiler->SetElementError($class_context->file,$class_context->path,"it was defined a class with the same name of the component"));
		if((!IsSet($class_values["variable"])
		|| count($class_values["variable"])==0)
		&& (!IsSet($class_values["collection"])
		|| count($class_values["collection"])==0))
			return($compiler->SetElementError($class_context->file,$class_context->path,"it were not specified any class variables"));
		$class_name=$class_values["name"];
		if(IsSet($this->components[$component]["classes"][$class_name]))
			return($compiler->SetElementError($class_context->file,$class_context->path,"it was specified a class with the name \"$class_name\" more than once"));
		if(IsSet($class_values["id"])
		&& !$this->SetComponentId($compiler, $class_context, $component, $class_values["id"], $class_name, "class"))
			return(0);
		$this->components[$component]["classes"][$class_name]=array(
			"file"=>$class_context->file,
			"path"=>$class_context->path,
			"variables"=>array(),
			"collections"=>array(),
			"functions"=>array(),
			"validations"=>array()
		);
		if(IsSet($class_values["variable"]))
		{
			for($variable=0;$variable<count($class_values["variable"]);$variable++)
			{
				$variable_values=array();
				$required_types=array(
					"name"=>"NAME"
				);
				$optional_types=array(
					"id"=>"DATA",
					"type"=>"NAME",
					"class"=>"NAME",
					"component"=>"NAME",
					"initialvalue"=>"DATA",
					"autocreate"=>"DATA",
					"autoupdate"=>"DATA",
					"multiline"=>"BOOLEAN",
					"length"=>"INTEGER",
					"optional"=>"BOOLEAN"
				);
				$compiler->CopyEmptyContext($class_values["variable"][$variable],$variable_context);
				if(!$compiler->GetOptionalArgumentValues($variable_context,$variable_values,$required_types,$optional_types,"persistence","variable",1))
					return(0);
				$variable_name=$variable_values["name"];
				if(IsSet($this->components[$component]["classes"][$class_name]["variables"][$variable_name]))
					return($compiler->SetElementError($variable_context->file,$variable_context->path,"it was specified a class variable with the name \"$variable_name\" more than once"));
				if(IsSet($this->components[$component]["classes"][$class_name]["collections"][$variable_name]))
					return($compiler->SetElementError($variable_context->file,$variable_context->path,"it was specified a class variable with the name \"$variable_name\" of an already defined collection"));
				$this->components[$component]["classes"][$class_name]["variables"][$variable_name]=array("Context"=>$variable_context);
				if(IsSet($variable_values["multiline"]))
					$this->components[$component]["classes"][$class_name]["variables"][$variable_name]["multiline"]=$variable_values["multiline"];
				if(!$this->ValidateType($compiler,$variable_context,$component,$variable_values,$this->components[$component]["classes"][$class_name]["variables"][$variable_name]))
					return(0);
				if(IsSet($variable_values["length"]))
				{
					if(!IsSet($this->components[$component]["classes"][$class_name]["variables"][$variable_name]["type"])
					|| strcmp($this->components[$component]["classes"][$class_name]["variables"][$variable_name]["type"],"text"))
						return($compiler->SetElementError($variable_context->file,$variable_context->path,"it was specified the length of class variable \"$variable_name\" which is not of type text"));
					if($variable_values["length"]<1)
						return($compiler->SetElementError($variable_context->file,$variable_context->path,"it was specified a length for class variable \"$variable_name\" that is less than 1 character"));
					$this->components[$component]["classes"][$class_name]["variables"][$variable_name]["length"]=$variable_values["length"];
				}
				if(IsSet($variable_values["id"])
				&& !$this->SetComponentId($compiler, $variable_context, $component, $variable_values["id"], $variable_name, "variable"))
					return(0);
			}
		}
		if(IsSet($class_values["collection"]))
		{
			for($collection=0;$collection<count($class_values["collection"]);$collection++)
			{
				$collection_values=array();
				$required_types=array(
					"name"=>"NAME",
					"class"=>"NAME",
					"reference"=>"NAME"
				);
				$optional_types=array(
					"id"=>"DATA",
					"component"=>"NAME"
				);
				$compiler->CopyEmptyContext($class_values["collection"][$collection],$collection_context);
				if(!$compiler->GetOptionalArgumentValues($collection_context,$collection_values,$required_types,$optional_types,"persistence","collection",1))
					return(0);
				$collection_name=$collection_values["name"];
				if(IsSet($this->components[$component]["classes"][$class_name]["collections"][$collection_name]))
					return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a class collection with the name \"$collection_name\" more than once"));
				if(IsSet($this->components[$component]["classes"][$class_name]["variables"][$collection_name]))
					return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a class collection with the name \"$collection_name\" of an already defined variable"));
				if(IsSet($collection_values["id"])
				&& !$this->SetComponentId($compiler, $collection_context, $component, $collection_values["id"], $collection_name, "collection"))
					return(0);
				$definition=array(
					"class"=>$collection_values["class"],
					"reference"=>$collection_values["reference"],
					"Context"=>$collection_context
				);
				if(IsSet($collection_values["component"]))
				{
					$collection_component=$collection_values["component"];
					if(!IsSet($this->components[$component]["components"][$collection_component]))
						return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was defined a collection of objects of a class of an unknown external component (".$collection_component.")"));
					$definition["component"]=$collection_component;
				}
				$this->components[$component]["classes"][$class_name]["collections"][$collection_name]=$definition;
			}
		}
		if(IsSet($class_values["validation"]))
		{
			for($validation=0;$validation<count($class_values["validation"]);$validation++)
			{
				$validation_values=array();
				$required_types=array(
					"type"=>"NAME",
					"errorcode"=>"INTEGERDATA",
					"parameters"=>"CONTEXT"
				);
				$optional_types=array(
					"id"=>"DATA"
				);
				if(!$compiler->GetOptionalArgumentValues($class_values["validation"][$validation],$this->components[$component]["classes"][$class_name]["validation"][$validation],$required_types,$optional_types,"persistence","validation",1))
					return(0);
				switch($validation_type=$this->components[$component]["classes"][$class_name]["validation"][$validation]["type"])
				{
					case "unique":
					case "notempty":
						break;
					default:
						return($compiler->SetElementError($class_values["validation"][$validation]->file,$class_values["validation"][$validation]->path,$validation_type." is not a supported validation type"));
				}
				if(IsSet($this->components[$component]["classes"][$class_name]["validation"][$validation]["id"])
				&& !$this->SetComponentId($compiler, $collection_context, $component, $this->components[$component]["classes"][$class_name]["validation"][$validation]["id"], strval($validation), "validation"))
					return(0);
			}
		}
		$supported=array(
			"addtocollection"=>1,
			"removefromcollection"=>1,
			"getcollection"=>1,
			"getreference"=>1,
			"persist"=>1,
			"setreference"=>1,
			"validate"=>1,
			"delete"=>1
		);
		if(IsSet($class_values["function"])
		&& !$this->ParseFunctionsDefinition($compiler,$class_values["function"],$component,$supported,$this->components[$component]["classes"][$class_name]["functions"]))
			return(0);
		Reset($this->components[$component]["classes"][$class_name]["functions"]);
		$end=(GetType($key=Key($this->components[$component]["classes"][$class_name]["functions"]))!="string");
		for(;!$end;)
		{
			switch($this->components[$component]["classes"][$class_name]["functions"][$key]["type"])
			{
				case "getreference":
					if(!$this->GetClassGetReferenceParameters($compiler,$component,$class_name,$this->components[$component]["classes"][$class_name]["functions"][$key],"class",$reference,$reference_class))
						return(0);
					$this->requirements[$component]["factory"]["GetObject"][$reference_class]=1;
					$this->requirements[$component]["class"][$reference_class]["Query"]=1;
					$this->requirements[$component]["class"][$reference_class]["Fetch"]=1;
					break;
				case "getcollection":
					if(!$this->GetClassGetCollectionParameters($compiler, $component, $class_name, $this->components[$component]["classes"][$class_name]["functions"][$key], "class", $collection_reference, $collection_class, $collection_component,$parameter_values))
						return(0);
					$this->requirements[$component]["factory"]["GetCollection"][$collection_class]=1;
					$this->requirements[$component]["class"][$collection_class]["Query"]=1;
					$this->requirements[$component]["class"][$collection_class]["Fetch"]=1;
					break;
			}
			Next($this->components[$component]["classes"][$class_name]["functions"]);
			$end=(GetType($key=Key($this->components[$component]["classes"][$class_name]["functions"]))!="string");
		}
		return(1);
	}

	Function GetFactoryGetAllObjectsParameters(&$compiler,$component,&$function_arguments,$function,&$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"class"=>"NAME"
		);
		$optional_types=array(
			"filter"=>"CONTEXT",
			"object"=>"CONTEXT:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["classes"][$parameter_values["class"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["class"]." is not a valid component class"));
		return(1);
	}

	Function GetFactoryGetObjectParameters(&$compiler,$component,&$function_arguments,$function,&$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"class"=>"NAME"
		);
		$optional_types=array(
			"id"=>"CONTEXT",
			"filter"=>"CONTEXT",
			"object"=>"CONTEXT:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($parameter_values["id"])
		&& !IsSet($parameter_values["filter"]))
			return($compiler->SetElementError($function_arguments["parameters"]->file, $function_arguments["parameters"]->path, "it was not specified the filter rule or id parameter of the object to get"));
		if(IsSet($parameter_values["id"])
		&& IsSet($parameter_values["filter"]))
			return($compiler->SetElementError($function_arguments["parameters"]->file, $function_arguments["parameters"]->path, "it was specified both the filter rule and the id parameter of the object to get"));
		if(!IsSet($this->components[$component]["classes"][$parameter_values["class"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file, $function_arguments["parameters"]->path, $values["class"]." is not a valid component class"));
		return(1);
	}

	Function GetReportOpenReportParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME"
		);
		$optional_types=array(
			"first"=>"CONTEXT",
			"limit"=>"CONTEXT",
			"count"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		if(IsSet($parameter_values["first"])
		&& !IsSet($parameter_values["limit"]))
			return($compiler->SetElementError($parameter_values["first"]->file,$parameter_values["first"]->path,"it was specified the report first row without specifying the row limit"));
		return(1);
	}

	Function GetReportGetReportDataParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME",
			"data"=>"CONTEXT",
			"endofdata"=>"CONTEXT"
		);
		$optional_types=array(
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		return(1);
	}

	Function GetReportGetAllReportDataParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME",
			"data"=>"CONTEXT"
		);
		$optional_types=array(
			"first"=>"CONTEXT",
			"limit"=>"CONTEXT",
			"count"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		if(IsSet($parameter_values["first"])
		&& !IsSet($parameter_values["limit"]))
			return($compiler->SetElementError($parameter_values["first"]->file,$parameter_values["first"]->path,"it was specified the report first row without specifying the row limit"));
		return(1);
	}

	Function GetReportGetReportFieldParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME",
			"field"=>"CONTEXT"
		);
		$optional_types=array(
			"first"=>"CONTEXT",
			"limit"=>"CONTEXT",
			"count"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		if(IsSet($parameter_values["first"])
		&& !IsSet($parameter_values["limit"]))
			return($compiler->SetElementError($parameter_values["first"]->file,$parameter_values["first"]->path,"it was specified the report first row without specifying the row limit"));
		return(1);
	}

	Function GetReportGetReportRowParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME",
			"data"=>"CONTEXT"
		);
		$optional_types=array(
			"first"=>"CONTEXT",
			"limit"=>"CONTEXT",
			"count"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		if(IsSet($parameter_values["first"])
		&& !IsSet($parameter_values["limit"]))
			return($compiler->SetElementError($parameter_values["first"]->file,$parameter_values["first"]->path,"it was specified the report first row without specifying the row limit"));
		return(1);
	}

	Function GetReportGetReportSQLParameters(&$compiler, $component, $report_name, &$function_arguments, $function, &$parameter_values)
	{
		if(!IsSet($function_arguments["parameters"]))
			return($compiler->SetElementError($function_arguments["function"]->file,$function_arguments["function"]->path,"it were not specified the function parameters"));
		$parameter_values=array();
		$required_types=array(
			"query"=>"NAME"
		);
		$optional_types=array(
			"condition"=>"CONTEXT",
			"tables"=>"CONTEXT",
			"columns"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($function_arguments["parameters"],$parameter_values,$required_types,$optional_types,"persistence",$function,1))
			return(0);
		if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$parameter_values["query"]]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,$parameter_values["query"]." is not a valid report query"));
		if(!IsSet($parameter_values["condition"])
		&& !IsSet($parameter_values["tables"])
		&& !IsSet($parameter_values["columns"]))
			return($compiler->SetElementError($function_arguments["parameters"]->file,$function_arguments["parameters"]->path,"it were not specified any report SQL query parts to return"));
		return(1);
	}

	Function ParseFormSubmitInput(&$compiler,&$submit_context,&$submit_values,$submit_type)
	{
		$submit_values=array();
		$required_types=array(
		);
		$optional_types=array(
			"value"=>"CONTEXT",
			"label"=>"CONTEXT",
			"accesskey"=>"CONTEXT",
			"field"=>"NAME",
			"image"=>"CONTEXT",
			"imagepath"=>"CONTEXT",
			"imagelabel"=>"CONTEXT",
			"imageaccesskey"=>"CONTEXT"
		);
		if(!$compiler->GetOptionalArgumentValues($submit_context,$submit_values,$required_types,$optional_types,"persistence","form",1))
			return(0);
		if(!IsSet($submit_values["value"])
		&& !IsSet($submit_values["image"]))
			return($compiler->SetElementError($submit_context->file,$submit_context->path,"it were not specified either the ".$submit_type." input value or image values"));
		return(1);
	}

	Function ParseFormDefinition(&$compiler,&$context,$component)
	{
		$form_values=array();
		$required_types=array(
			"name"=>"NAME",
			"type"=>"DATA",
			"parameters"=>"CONTEXT",
			"page"=>"CONTEXT"
		);
		$optional_types=array(
			"themefile"=>"DATA",
			"themename"=>"DATA",
			"theme"=>"CONTEXT:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($context,$form_values,$required_types,$optional_types,"persistence","form",1))
			return(0);
		$form_name=$form_values["name"];
		if(IsSet($this->components[$component]["forms"][$form_name]))
			return($compiler->SetElementError($context->file,$context->path,"it was defined a form with the name ".$form_name." more than once"));
		$theme_values=array();
		if(IsSet($form_values["themename"])
		|| IsSet($form_values["themefile"])
		|| IsSet($form_values["theme"]))
		{
			$required_types=array(
			);
			$optional_types=array(
				"formvariable"=>"CONTEXT:MULTIPLE",
				"skin"=>"CONTEXT",
				"errormessage"=>"CONTEXT",
				"marks"=>"CONTEXT",
				"template"=>"CONTEXT:MULTIPLE",
				"class"=>"CONTEXT",
				"style"=>"CONTEXT",
				"layout"=>"CONTEXT",
				"image"=>"CONTEXT:MULTIPLE",
				"imagespath"=>"CONTEXT"
			);
			$theme_path=array();
			if(IsSet($form_values["themename"]))
			{
				if(!IsSet($this->themes[$form_values["themename"]]))
					return($compiler->SetElementError($context->file,$context->path,$form_values["themename"]." is not listed theme name"));
				$theme_file=$this->themes[$form_values["themename"]];
				if(!$compiler->ParseDefinitionFile($theme_file))
					return(0);
				$compiler->GetParser($parser,$theme_file);
				if(strcmp($parser->structure["0"]["Tag"],"theme"))
					return($compiler->SetElementError($theme_file,"0","it was not specified a valid theme definition file"));
				$compiler->CreateFileContext($theme_context,$theme_file);
				if(!$compiler->GetOptionalArgumentValues($theme_context,$theme_values,$required_types,$optional_types,"persistence","form",!IsSet($form_values["theme"]) && !IsSet($form_values["themefile"])))
					return(0);
				$theme_path[]=dirname($theme_file);
				$this->dependencies[]=$compiler->GetIncludePath($theme_file).$theme_file;
			}
			if(IsSet($form_values["themefile"]))
			{
				$theme_file=$form_values["themefile"];
				if(!$compiler->ParseDefinitionFile($theme_file))
					return(0);
				$compiler->GetParser($parser,$theme_file);
				if(strcmp($parser->structure["0"]["Tag"],"theme"))
					return($compiler->SetElementError($theme_file,"0","it was not specified a valid theme definition file"));
				$compiler->CreateFileContext($theme_context,$theme_file);
				if(!$compiler->GetOptionalArgumentValues($theme_context,$theme_values,$required_types,$optional_types,"persistence","form",!IsSet($form_values["theme"])))
					return(0);
				$theme_path[]=dirname($theme_file);
				$this->dependencies[]=$compiler->GetIncludePath($theme_file).$theme_file;
			}
			if(IsSet($form_values["theme"]))
			{
				for($theme=0;$theme<count($form_values["theme"]);$theme++)
				{
					if(!$compiler->GetOptionalArgumentValues($form_values["theme"][$theme],$theme_values,$required_types,$optional_types,"persistence","form",1))
						return(0);
				}
			}
		}
		$form_variables=array();
		if(IsSet($theme_values["formvariable"]))
		{
			for($variable=0;$variable<count($theme_values["formvariable"]);$variable++)
			{
				$form_variable_values=array();
				$required_types=array(
					"name"=>"NAME"
				);
				$optional_types=array(
					"type"=>"NAME",
					"class"=>"NAME",
					"value"=>"DATA"
				);
				if(!$compiler->GetOptionalArgumentValues($theme_values["formvariable"][$variable],$form_variable_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				$variable_name=$form_variable_values["name"];
				if(IsSet($form_variables[$variable_name]))
					return($compiler->SetElementError($theme_values["formvariable"][$variable]->file,$theme_values["formvariable"][$variable]->path,"it was specified the form variable \"".$form_variable_values["name"]."\" more than once"));
				$form_variables[$variable_name]=array();
				if(IsSet($form_variable_values["type"]))
				{
					if(IsSet($form_variable_values["class"]))
						return($compiler->SetElementError($theme_values["formvariable"][$variable]->file,$theme_values["formvariable"][$variable]->path,"it was defined both the form variable type and class"));
					switch($form_variable_values["type"])
					{
						case "STRING":
						case "INTEGER":
						case "BOOLEAN":
						case "FLOAT":
							break;
						default:
							return($compiler->SetElementError($theme_values["formvariable"][$variable]->file,$theme_values["formvariable"][$variable]->path,$form_variable_values["type"]." is not a valid form variable type"));
					}
					$form_variables[$variable_name]["type"]=$form_variable_values["type"];
					if(IsSet($form_variable_values["value"]))
						$form_variables[$variable_name]["value"]=$form_variable_values["value"];
				}
				else
				{
					if(!IsSet($form_variable_values["class"]))
						return($compiler->SetElementError($theme_values["formvariable"][$variable]->file,$theme_values["formvariable"][$variable]->path,"it was not defined the form variable type"));
					if(!IsSet($this->components[$component]["classes"][$form_variable_values["class"]]))
						return($compiler->SetElementError($theme_values["formvariable"][$variable]->file,$theme_values["formvariable"][$variable]->path,"it was specified an inexisting class for the object form variable"));
					$form_variables[$variable_name]["class"]=$form_variable_values["class"];
				}
			}
		}
		$images=array();
		if(IsSet($theme_values["image"]))
		{
			if(!IsSet($theme_values["imagespath"]))
				return($compiler->SetElementError($theme_values["image"][0]->file,$theme_values["image"][0]->path,"it was defined a theme image but it was not specified the images path"));
			for($image=0;$image<count($theme_values["image"]);$image++)
			{
				$image_values=array();
				$required_types=array(
					"file"=>"DATA"
				);
				$optional_types=array(
					"role"=>"NAME"
				);
				if(!$compiler->GetOptionalArgumentValues($theme_values["image"][$image],$image_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				$image_file=$image_values["file"];
				if(IsSet($images[$image_file]))
					return($compiler->SetElementError($theme_values["image"][$image]->file,$theme_values["image"][$image]->path,"it was specified the image \"".$image_values["file"]."\" more than once"));
				$images[$image_file]=array("Base"=>$compiler->GetBaseIncludePath($image_file,$theme_path));
				if(IsSet($image_values["role"]))
				{
					switch($image_values["role"])
					{
						case "cancel":
							break;
						default:
							return($compiler->SetElementError($theme_values["image"][$image]->file,$theme_values["image"][$image]->path,$image_values["role"]." is not a supported image role"));
					}
					$images[$image_file]["role"]=$image_values["role"];
				}
				$compiler->CopyEmptyContext($theme_values["image"][$image],$images[$image_file]["Context"]);
			}
		}
		$page_form_values=array();
		$required_types=array(
			"template"=>"CONTEXT"
		);
		$optional_types=array(
		);
		if(!$compiler->GetOptionalArgumentValues($form_values["page"],$page_form_values,$required_types,$optional_types,"persistence","form",1))
			return(0);
		$page_template_form_values=array();
		$required_types=array();
		$optional_types=array(
			"clip"=>"DATA",
			"variables"=>"CONTEXT",
			"conditional"=>"CONTEXT"
		);
		if(IsSet($theme_values["layout"]))
			$optional_types["file"]="DATA";
		else
			$required_types["file"]="DATA";
		if(!$compiler->GetOptionalArgumentValues($page_form_values["template"],$page_template_form_values,$required_types,$optional_types,"persistence","form",1))
			return(0);
		if(!IsSet($theme_values["layout"]))
		{
			$file=$page_template_form_values["file"];
			$page_template_form_values["file"]=$compiler->GetBaseIncludePath($file,$theme_path).$file;
		}
		if(!IsSet($page_template_form_values["clip"]))
			$page_template_form_values["clip"]="form";
		$this->components[$component]["forms"][$form_name]=array(
			"type"=>$form_values["type"],
			"formvariables"=>$form_variables,
			"images"=>$images,
			"page"=>array(""=>array(
				"template"=>$page_template_form_values
			))
		);
		if(IsSet($theme_values["imagespath"]))
			$compiler->CopyEmptyContext($theme_values["imagespath"],$this->components[$component]["forms"][$form_name]["imagespath"]);
		if(IsSet($theme_values["skin"]))
		{
			$skin_form_values=array();
			$required_types=array(
				"file"=>"DATA"
			);
			$optional_types=array(
				"clip"=>"DATA",
				"bodyvariable"=>"DATA"
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["skin"],$skin_form_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			$file=$skin_form_values["file"];
			$skin_form_values["file"]=$compiler->GetBaseIncludePath($file,$theme_path).$file;
			if(!IsSet($skin_form_values["clip"]))
				$skin_form_values["clip"]="formskin";
			if(!IsSet($skin_form_values["bodyvariable"]))
				$skin_form_values["bodyvariable"]="body";
			$this->components[$component]["forms"][$form_name]["skin"]=$skin_form_values;
		}
		if(IsSet($theme_values["errormessage"]))
		{
			$error_message_values=array();
			$required_types=array(
				"formvariable"=>"NAME"
			);
			$optional_types=array(
				"templateconditional"=>"NAME"
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["errormessage"],$error_message_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			$compiler->CopyEmptyContext($theme_values["errormessage"],$error_message_values["Context"]);
			$this->components[$component]["forms"][$form_name]["errormessage"]=$error_message_values;
		}
		if(IsSet($theme_values["marks"]))
		{
			$mark_values=array();
			$required_types=array(
				"verify"=>"CONTEXT"
			);
			$optional_types=array(
				"required"=>"CONTEXT"
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["marks"],$mark_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			$compiler->CopyEmptyContext($theme_values["marks"],$mark_values["Context"]);
			$this->components[$component]["forms"][$form_name]["marks"]=$mark_values;
		}
		if(IsSet($theme_values["template"]))
		{
			$template_values=array();
			$required_types=array(
			);
			$optional_types=array(
				"variables"=>"CONTEXT:MULTIPLE",
				"conditional"=>"CONTEXT:MULTIPLE"
			);
			for($template=0;$template<count($theme_values["template"]);$template++)
			{
				if(!$compiler->GetOptionalArgumentValues($theme_values["template"][$template],$template_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
			}
			$this->components[$component]["forms"][$form_name]["template"]=$template_values;
		}
		if(IsSet($theme_values["class"]))
		{
			$theme_class_values=array();
			$required_types=array(
			);
			$optional_types=array(
				"input"=>"CONTEXT"
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["class"],$theme_class_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			if(IsSet($theme_class_values["input"]))
			{
				$theme_class_input_values=array();
				$required_types=array(
				);
				$optional_types=array(
					"text"=>"CONTEXT",
					"textarea"=>"CONTEXT",
					"submit"=>"CONTEXT",
					"image"=>"CONTEXT"
				);
				if(!$compiler->GetOptionalArgumentValues($theme_class_values["input"],$theme_class_input_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				$this->components[$component]["forms"][$form_name]["class"]=array("input"=>$theme_class_input_values);
			}
			else
				$template_values["class"]=array();
		}
		if(IsSet($theme_values["style"]))
		{
			$theme_style_values=array();
			$required_types=array(
			);
			$optional_types=array(
				"input"=>"CONTEXT"
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["style"],$theme_style_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			if(IsSet($theme_style_values["input"]))
			{
				$theme_style_input_values=array();
				$required_types=array(
				);
				$optional_types=array(
					"text"=>"CONTEXT",
					"textarea"=>"CONTEXT",
					"submit"=>"CONTEXT",
					"image"=>"CONTEXT"
				);
				if(!$compiler->GetOptionalArgumentValues($theme_style_values["input"],$theme_style_input_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				$this->components[$component]["forms"][$form_name]["style"]=array("input"=>$theme_style_input_values);
			}
			else
				$template_values["style"]=array();
		}
		if(IsSet($theme_values["layout"]))
		{
			$template_values=array();
			$required_types=array(
				"type"=>"DATA",
			);
			$optional_types=array(
			);
			if(!$compiler->GetOptionalArgumentValues($theme_values["layout"],$layout_values,$required_types,$optional_types,"persistence","form",1))
				return(0);
			$compiler->CopyEmptyContext($theme_values["layout"],$layout_values["Context"]);
			$this->components[$component]["forms"][$form_name]["layout"]=$layout_values;
		}
		switch($form_values["type"])
		{
			case "createobject":
				$parameter_values=array();
				$required_types=array(
					"object"=>"CONTEXT",
					"submit"=>"CONTEXT"
				);
				$optional_types=array(
					"cancel"=>"CONTEXT"
				);
				if(!$compiler->GetOptionalArgumentValues($form_values["parameters"],$parameter_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				if(!$this->ParseFormSubmitInput($compiler,$parameter_values["submit"],$this->components[$component]["forms"][$form_name]["parameters"]["submit"],"submit"))
					return(0);
				if(IsSet($parameter_values["cancel"])
				&& !$this->ParseFormSubmitInput($compiler,$parameter_values["cancel"],$this->components[$component]["forms"][$form_name]["parameters"]["cancel"],"cancel"))
					return(0);
				$object_values=array();
				$required_types=array(
					"class"=>"NAME"
				);
				$optional_types=array(
					"variable"=>"CONTEXT:MULTIPLE",
					"error"=>"CONTEXT:MULTIPLE"
				);
				if(!$compiler->GetOptionalArgumentValues($parameter_values["object"],$object_values,$required_types,$optional_types,"persistence","form",1))
					return(0);
				$this->components[$component]["forms"][$form_name]["parameters"]["class"]=$class=$object_values["class"];
				if(!IsSet($this->components[$component]["classes"][$class]))
					return($compiler->SetElementError($parameter_values["object"]->file,$parameter_values["object"]->path,$object_values["class"]." is not a valid ".$component." component class"));
				$this->components[$component]["forms"][$form_name]["variables"]=array();
				if(IsSet($object_values["variable"]))
				{
					$required_types=array(
						"name"=>"NAME"
					);
					$optional_types=array(
						"defaultvalueformvariable"=>"NAME",
						"label"=>"CONTEXT",
						"accesskey"=>"CONTEXT",
						"display"=>"CONTEXT",
						"verticalposition"=>"INTEGERDATA"
					);
					for($variable=0;$variable<count($object_values["variable"]);$variable++)
					{
						$variable_values=array();
						if(!$compiler->GetOptionalArgumentValues($object_values["variable"][$variable],$variable_values,$required_types,$optional_types,"persistence","form",1))
							return(0);
						$name=$variable_values["name"];
						if(!IsSet($this->components[$component]["classes"][$class]["variables"][$name]))
							return($compiler->SetElementError($object_values["variable"][$variable]->file,$object_values["variable"][$variable]->path,$name." is not a valid ".$component." component ".$class." class variable"));
						if(IsSet($this->components[$component]["classes"][$class]["variables"][$name][$auto="autocreate"])
						|| IsSet($this->components[$component]["classes"][$class]["variables"][$name][$auto="autoupdate"]))
							return($compiler->SetElementError($object_values["variable"][$variable]->file,$object_values["variable"][$variable]->path,"the ".$auto." variable ".$name." may not show in a createobject form because it is automatically assigned by its class"));
						if(IsSet($this->components[$component]["forms"][$form_name]["variables"][$name]))
							return($compiler->SetElementError($object_values["variable"][$variable]->file,$object_values["variable"][$variable]->path,$name." is already declared form class variable"));
						$this->components[$component]["forms"][$form_name]["variables"][$name]=$variable_values;
					}
				}

				$this->components[$component]["forms"][$form_name]["errors"]=array();
				if(IsSet($object_values["error"]))
				{
					for($error=0;$error<count($object_values["error"]);$error++)
					{
						$error_values=array();
						$required_types=array(
							"code"=>"INTEGERDATA",
							"message"=>"CONTEXT",
						);
						$optional_types=array(
						);
						if(!$compiler->GetOptionalArgumentValues($object_values["error"][$error],$error_values,$required_types,$optional_types,"persistence","form",1))
							return(0);
						$code=$error_values["code"];
						if(IsSet($this->components[$component]["forms"][$form_name]["errors"][$code]))
							return($compiler->SetElementError($object_values["error"][$error]->file,$object_values["error"][$error]->path,"form error code ".$code." is already declared"));
						$error_values["variables"]=array();
						$this->components[$component]["forms"][$form_name]["errors"][$code]=$error_values;
					}
				}

				$this->components[$component]["forms"][$form_name]["validations"]=array();
				if(IsSet($this->components[$component]["classes"][$class]["validation"]))
				{
					for($validation=0;$validation<count($this->components[$component]["classes"][$class]["validation"]);$validation++)
					{
						$type=$this->components[$component]["classes"][$class]["validation"][$validation]["type"];
						switch($type)
						{
							case "unique":
							case "notempty":
								$variable_values=array();
								$required_types=array(
									"variable"=>"NAME:MULTIPLE"
								);
								$optional_types=array();
								if(!$compiler->GetOptionalArgumentValues($this->components[$component]["classes"][$class]["validation"][$validation]["parameters"],$variable_values,$required_types,$optional_types,"persistence","form",1))
									return(0);
								for($variable=0;$variable<count($variable_values["variable"]);$variable++)
								{
									$variable_name=$variable_values["variable"][$variable];
									$code=$this->components[$component]["classes"][$class]["validation"][$validation]["errorcode"];
									if(!IsSet($this->components[$component]["forms"][$form_name]["errors"][$code]))
										return($compiler->SetElementError($parameter_values["object"]->file,$parameter_values["object"]->path,"it was not defined the message for error code ".$code." for ".$type." validation of variable ".$variable_name));
									$this->components[$component]["forms"][$form_name]["validations"][$variable_name][]=array(
										"type"=>$type,
										"errorcode"=>$code,
										"message"=>$this->components[$component]["forms"][$form_name]["errors"][$code]["message"]
									);
									$this->components[$component]["forms"][$form_name]["errors"][$code]["variables"][$variable_name]="";
								}
								break;
							default:
								return($compiler->SetElementError($this->components[$component]["classes"][$class]["validation"][$validation]["parameters"]->file,$this->components[$component]["classes"][$class]["validation"][$validation]["parameters"]->path,$this->components[$component]["classes"][$class]["validation"][$validation]["type"]." is not a supported validation criteria"));
						}
					}
				}
				break;
			default:
				return($compiler->SetElementError($context->file,$context->path,$form_values["type"]." is not yet a supported component form type"));
		}
		return(1);
	}

	Function AddObjectTables(&$classes, &$tables)
	{
		Reset($classes);
		$end=(GetType($table=Key($classes))!="string");
		for(;!$end;)
		{
			$classes[$table]["table"]=$table;
			$tables[$table]=$classes[$table]["class"];
			Next($classes);
			$end=(GetType($table=Key($classes))!="string");
		}
	}

	Function GetQueryObjects(&$compiler, &$definitions, &$objects, $default_component, $function)
	{
		$objects=array();
		for($object=0;$object<count($definitions);$object++)
		{
			$object_values=array();
			$required_types=array(
				"name"=>"NAME",
				"class"=>"NAME"
			);
			$optional_types=array(
				"component"=>"NAME"
			);
			if(!$compiler->GetOptionalArgumentValues($definitions[$object],$object_values,$required_types,$optional_types,"persistence",$function,1))
				return(0);
			$object_name=$object_values["name"];
			if(IsSet($objects[$object_name]))
				return($compiler->SetElementError($definitions[$object]->file,$definitions[$object]->path,"it was defined a report query object with the name ".$query_name." more than once"));
			if(IsSet($object_values["component"]))
			{
				$object_component=$object_values["component"];
				if(!IsSet($this->components[$object_component]))
					return($compiler->SetElementError($definitions[$object]->file,$definitions[$object]->path,$component." is not an existing component"));
			}
			else
				$object_component=$default_component;
			$class=$object_values["class"];
			if(!IsSet($this->components[$object_component]["classes"][$class]))
				return($compiler->SetElementError($definitions[$object]->file,$definitions[$object]->path,$class." is not an existing component class"));
			$objects[$object_name]=array(
				"class"=>$class,
				"component"=>$object_component
			);
		}
		return(1);
	}

	Function ParseReportDefinition(&$compiler,&$context,$component)
	{
		$report_values=array();
		$required_types=array(
			"name"=>"NAME",
			"query"=>"CONTEXT:MULTIPLE",
			"function"=>"CONTEXT:MULTIPLE"
		);
		$optional_types=array(
		);
		if(!$compiler->GetOptionalArgumentValues($context,$report_values,$required_types,$optional_types,"persistence","report",1))
			return(0);
		$report_name=$report_values["name"];
		if(IsSet($this->components[$component]["reports"][$report_name]))
			return($compiler->SetElementError($context->file,$context->path,"it was defined a report with the name ".$report_name." more than once"));
		for($query=0;$query<count($report_values["query"]);$query++)
		{
			$query_values=array();
			$required_types=array(
				"name"=>"NAME",
				"object"=>"CONTEXT:MULTIPLE",
				"column"=>"CONTEXT:MULTIPLE"
			);
			$optional_types=array(
				"filter"=>"CONTEXT",
				"order"=>"CONTEXT"
			);
			if(!$compiler->GetOptionalArgumentValues($report_values["query"][$query],$query_values,$required_types,$optional_types,"persistence","report query",1))
				return(0);
			$query_name=$query_values["name"];
			if(IsSet($this->components[$component]["reports"][$report_name]["queries"][$query_name]))
				return($compiler->SetElementError($context->file,$context->path,"it was defined a report query with the name ".$query_name." more than once"));
			$this->components[$component]["reports"][$report_name]["queries"][$query_name]=array(
				"objects"=>array(),
				"columns"=>array(),
			);
			if(IsSet($query_values["filter"]))
				$compiler->CopyEmptyContext($query_values["filter"],$this->components[$component]["reports"][$report_name]["queries"][$query_name]["filter"]);
			if(!$this->GetQueryObjects($compiler, $query_values["object"], $this->components[$component]["reports"][$report_name]["queries"][$query_name]["objects"], $component, "report query object"))
				return(0);
			for($column=0;$column<count($query_values["column"]);$column++)
			{
				$column_values=array();
				$required_types=array(
					"name"=>"NAME",
					"value"=>"CONTEXT"
				);
				$optional_types=array(
				);
				if(!$compiler->GetOptionalArgumentValues($query_values["column"][$column],$column_values,$required_types,$optional_types,"persistence","report query column",1))
					return(0);
				$column_name=$column_values["name"];
				if(IsSet($this->components[$component]["reports"][$report_name]["queries"][$query_name]["columns"][$column_name]))
					return($compiler->SetElementError($query_values["column"][$column]->file,$query_values["column"][$column]->path,"it was defined a report query column with the name ".$query_name." more than once"));
				$compiler->CopyEmptyContext($column_values["value"], $this->components[$component]["reports"][$report_name]["queries"][$query_name]["columns"][$column_name]["value"]);
			}
			if(IsSet($query_values["order"]))
			{
				$this->components[$component]["reports"][$report_name]["queries"][$query_name]["order"]=array();
				$order_values=array();
				$required_types=array(
					"column"=>"CONTEXT:MULTIPLE"
				);
				$optional_types=array(
				);
				if(!$compiler->GetOptionalArgumentValues($query_values["order"],$order_values,$required_types,$optional_types,"persistence","report query order",1))
					return(0);
				for($column_names=array(), $column=0;$column<count($order_values["column"]);$column++)
				{
					$column_values=array();
					$required_types=array(
						"name"=>"NAME"
					);
					$optional_types=array(
						"ascending"=>"DATA",
						"descending"=>"DATA"
					);
					if(!$compiler->GetOptionalArgumentValues($order_values["column"][$column],$this->components[$component]["reports"][$report_name]["queries"][$query_name]["order"][$column],$required_types,$optional_types,"persistence","report query order column",1))
						return(0);
					$column_name=$this->components[$component]["reports"][$report_name]["queries"][$query_name]["order"][$column]["name"];
					if(!IsSet($this->components[$component]["reports"][$report_name]["queries"][$query_name]["columns"][$column_name]))
						return($compiler->SetElementError($order_values["column"][$column]->file,$order_values["column"][$column]->path,$column_name." is not a valid report query order column"));
					if(IsSet($column_names[$column_name]))
						return($compiler->SetElementError($order_values["column"][$column]->file,$order_values["column"][$column]->path,"it was specified the report query order column with the name ".$column_name." more than once"));
				}
			}
		}
		$supported=array(
			"openreport"=>1,
			"getreportdata"=>1,
			"getallreportdata"=>1,
			"getreportfield"=>1,
			"getreportrow"=>1,
			"getreportsql"=>1
		);
		return($this->ParseFunctionsDefinition($compiler,$report_values["function"],$component,$supported,$this->components[$component]["reports"][$report_name]["functions"]));
	}

	Function GetComponentPath(&$compiler, $file, &$component_path)
	{
		$component_path=array(
			dirname($file),
			getcwd()
		);
		for($path=0;$path<count($compiler->include_path);$path++)
			$component_path[]=$compiler->include_path[$path];
	}

	Function ParseComponentDefinition(&$compiler,&$context,&$name)
	{
		$path=$context->path;
		$compiler->GetParser($parser,$context->file);
	  if(strcmp($parser->structure[$path]["Tag"],"component"))
			return($compiler->SetParsedElementError($context->file,$parser,$path,"it was not specified a valid component definition file"));
		$this->GetComponentPath($compiler, $context->file, $component_path);
		$values=array();
		$required_types=array(
			"name"=>"NAME",
			"description"=>"TEXT",
			"class"=>"CONTEXT:MULTIPLE",
			"schema"=>"CONTEXT",
			"factory"=>"CONTEXT"
		);
		$optional_types=array(
			"includeclass"=>"DATA:MULTIPLE",
			"form"=>"CONTEXT:MULTIPLE",
			"includeform"=>"DATA:MULTIPLE",
			"component"=>"CONTEXT:MULTIPLE",
			"report"=>"CONTEXT:MULTIPLE",
			"includereport"=>"DATA:MULTIPLE"
		);
		if(!$compiler->GetOptionalArgumentValues($context,$values,$required_types,$optional_types,"persistence","component",1))
			return(0);
		if(strlen($name))
		{
			if(strcmp($name,$values["name"]))
				return($compiler->SetElementError($context->file,$context->path,"the component defined by file ".$context->file." is not named \"".$name."\""));
		}
		else
			$name=$values["name"];
		$this->components[$name]=array(
			"description"=>$values["description"],
			"classes"=>array(),
			"id"=>array(),
			"components"=>array()
		);
		if(IsSet($values["component"]))
		{
			for($component=0;$component<count($values["component"]);$component++)
			{
				$component_values=array();
				$required_types=array(
					"name"=>"NAME",
					"file"=>"DATA"
				);
				$optional_types=array(
				);
				$compiler->CopyEmptyContext($values["component"][$component],$component_context);
				if(!$compiler->GetOptionalArgumentValues($component_context,$component_values,$required_types,$optional_types,"persistence","factory",1))
					return(0);
				$component_name=$component_values["name"];
				if(!strcmp($component_name,$name))
					return($compiler->SetElementError($component_context->file,$component_context->path,"it was specified an external component with the same name of this component"));
				if(IsSet($this->components[$name]["components"][$component_name]))
					return($compiler->SetElementError($component_context->file,$component_context->path,"it was specified external component with the name of an already specified component"));
				$this->components[$name]["components"][$component_name]=array(
					"file"=>$component_values["file"]
				);
			}
		}
		if(IsSet($values["includeclass"]))
		{
			for($class=0;$class<count($values["includeclass"]);$class++)
			{
				$include_class=$values["includeclass"][$class];
				if(!$compiler->ParseDefinitionFileFromPath($include_class,$component_path))
					return(0);
				$compiler->GetParser($parser,$include_class);
				if(strcmp($parser->structure["0"]["Tag"],"class"))
					return($compiler->SetElementError($include_class,"0","it was not specified a valid component class definition file"));
				$compiler->CreateFileContext($class_context,$include_class);
				if(!$this->ParseComponentClass($compiler,$name,$class_context))
					return(0);
				$this->dependencies[]=$compiler->GetIncludePathFromList($include_class,$component_path).$include_class;
			}
		}
		for($class=0;$class<count($values["class"]);$class++)
		{
			$compiler->CopyEmptyContext($values["class"][$class],$class_context);
			if(!$this->ParseComponentClass($compiler,$name,$class_context))
				return(0);
		}
		for($class=0,Reset($this->components[$name]["classes"]);$class<count($this->components[$name]["classes"]);Next($this->components[$name]["classes"]),$class++)
		{
			$class_name=Key($this->components[$name]["classes"]);
			$class_file=$this->components[$name]["classes"][$class_name]["file"];
			$class_path=$this->components[$name]["classes"][$class_name]["path"];
			for($variable=0,Reset($this->components[$name]["classes"][$class_name]["variables"]);$variable<count($this->components[$name]["classes"][$class_name]["variables"]);Next($this->components[$name]["classes"][$class_name]["variables"]),$variable++)
			{
				$variable_name=Key($this->components[$name]["classes"][$class_name]["variables"]);
				if(IsSet($this->components[$name]["classes"][$class_name]["variables"][$variable_name]["class"]))
				{
					$variable_class=$this->components[$name]["classes"][$class_name]["variables"][$variable_name]["class"];
					if(IsSet($this->components[$name]["classes"][$class_name]["variables"][$variable_name]["component"]))
					{
						$reference_component=$this->components[$name]["classes"][$class_name]["variables"][$variable_name]["component"];
						if(!IsSet($this->components[$reference_component]))
						{
							$component_file=$this->components[$name]["components"][$reference_component]["file"];
							$this->GetComponentPath($compiler, $context->file, $component_path);
							if(!$compiler->ParseDefinitionFileFromPath($component_file,$component_path))
								return(0);
							$compiler->CreateFileContext($component_context,$component_file);
							if(!$this->ParseComponentDefinition($compiler,$component_context,$reference_component))
								return(0);
							$this->dependencies[]=$compiler->GetIncludePathFromList($component_file,$component_path).$component_file;
						}
						if(!IsSet($this->components[$reference_component]["classes"][$variable_class]))
							return($compiler->SetElementError($class_file,$class_path,"it was specified a non-existing class \"".$variable_class."\" in the component \"".$reference_component."\" for the reference variable with the name \"$variable_name\""));
					}
					else
					{
						if(!IsSet($this->components[$name]["classes"][$variable_class]))
							return($compiler->SetElementError($class_file,$class_path,"it was specified a non-existing class ($variable_class) for variable with the name \"$variable_name\""));
					}
				}
			}
			for($collection=0,Reset($this->components[$name]["classes"][$class_name]["collections"]);$collection<count($this->components[$name]["classes"][$class_name]["collections"]);Next($this->components[$name]["classes"][$class_name]["collections"]),$collection++)
			{
				$collection_name=Key($this->components[$name]["classes"][$class_name]["collections"]);
				$collection_class=$this->components[$name]["classes"][$class_name]["collections"][$collection_name]["class"];
				$collection_component=(IsSet($this->components[$name]["classes"][$class_name]["collections"][$collection_name]["component"]) ? $this->components[$name]["classes"][$class_name]["collections"][$collection_name]["component"] : $name);
				$compiler->CopyEmptyContext($this->components[$name]["classes"][$class_name]["collections"][$collection_name]["Context"],$collection_context);
				if(!IsSet($this->components[$collection_component]))
				{
					$component_file=$this->components[$name]["components"][$collection_component]["file"];
					$this->GetComponentPath($compiler, $context->file, $component_path);
					$component_file=$compiler->GetIncludePathFromList($component_file, $component_path).$component_file;
					if(!$compiler->ParseDefinitionFileFromPath($component_file,$component_path))
						return(0);
					$compiler->CreateFileContext($component_context,$component_file);
					if(!$this->ParseComponentDefinition($compiler,$component_context,$collection_component))
						return(0);
					$this->dependencies[]=$compiler->GetIncludePathFromList($component_file,$component_path).$component_file;
				}
				if(!IsSet($this->components[$collection_component]["classes"][$collection_class]))
					return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a non-existing class ($collection_class) for collection with the name \"$collection_name\""));
				$collection_reference=$this->components[$name]["classes"][$class_name]["collections"][$collection_name]["reference"];
				if(IsSet($this->components[$collection_component]["classes"][$collection_class]["variables"][$collection_reference]))
				{
					if(!IsSet($this->components[$collection_component]["classes"][$collection_class]["variables"][$collection_reference]["class"]))
						return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a collection reference ($collection_reference) to non-object variable of the class \"$collection_class\" for collection with the name \"$collection_name\""));
					if(strcmp($this->components[$collection_component]["classes"][$collection_class]["variables"][$collection_reference]["class"],$class_name))
						return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a collection reference ($collection_reference) to a object variable of the class \"$collection_class\" for collection with the name \"$collection_name\" that is not of the same class ($class_name) that has the collection"));
				}
				elseif(IsSet($this->components[$collection_component]["classes"][$collection_class]["collections"][$collection_reference]))
				{
				}
				else
					return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a collection reference ($collection_reference) to non-existing class ($collection_class) variable or collection for collection with the name \"$collection_name\""));
				if(!strcmp($class_name,$collection_class)
				&& !strcmp($collection_name,$collection_reference))
					return($compiler->SetElementError($collection_context->file,$collection_context->path,"it was specified a collection reference ($collection_reference) to the same class collection ($collection_class)"));
			}
		}
		if(IsSet($values["factory"]))
		{
			$factory_values=array();
			$required_types=array(
			);
			$optional_types=array(
				"function"=>"CONTEXT:MULTIPLE"
			);
			if(!$compiler->GetOptionalArgumentValues($values["factory"],$factory_values,$required_types,$optional_types,"persistence","factory",1))
				return(0);
			$supported=array(
				"createobject"=>1,
				"getobject"=>1,
				"getallobjects"=>1
			);
			if(!$this->ParseFunctionsDefinition($compiler,$factory_values["function"],$name,$supported,$this->components[$name]["factory"]["functions"]))
				return(0);
			Reset($this->components[$name]["factory"]["functions"]);
			$end=(GetType($key=Key($this->components[$name]["factory"]["functions"]))!="string");
			for(;!$end;)
			{
				switch($this->components[$name]["factory"]["functions"][$key]["type"])
				{
					case "getallobjects":
						if(!$this->GetFactoryGetAllObjectsParameters($compiler,$name,$this->components[$name]["factory"]["functions"][$key],"factory",$parameter_values))
							return(0);
						$this->requirements[$name]["factory"]["GetCollection"][$parameter_values["class"]]=1;
						$this->requirements[$name]["class"][$parameter_values["class"]]["Query"]=1;
						$this->requirements[$name]["class"][$parameter_values["class"]]["Fetch"]=1;
						break;
					case "getobject":
						if(!$this->GetFactoryGetObjectParameters($compiler,$name,$this->components[$name]["factory"]["functions"][$key],"factory",$parameter_values))
							return(0);
						if(IsSet($parameter_values["id"]))
							$this->requirements[$name]["factory"]["GetObject"][$parameter_values["class"]]=1;
						else
							$this->requirements[$name]["factory"]["GetCollection"][$parameter_values["class"]]=1;
						$this->requirements[$name]["class"][$parameter_values["class"]]["Query"]=1;
						$this->requirements[$name]["class"][$parameter_values["class"]]["Fetch"]=1;
						break;
				}
				Next($this->components[$name]["factory"]["functions"]);
				$end=(GetType($key=Key($this->components[$name]["factory"]["functions"]))!="string");
			}
		}
		if(IsSet($values["schema"]))
		{
			$schema_values=array();
			$required_types=array(
			);
			$optional_types=array(
				"function"=>"CONTEXT:MULTIPLE"
			);
			if(!$compiler->GetOptionalArgumentValues($values["schema"],$schema_values,$required_types,$optional_types,"persistence","factory",1))
				return(0);
			$supported=array(
				"installschema"=>1
			);
			if(!$this->ParseFunctionsDefinition($compiler,$schema_values["function"],$name,$supported,$this->components[$name]["schema"]["functions"]))
				return(0);
			if(!IsSet($this->components[$name]["schema"]["functions"]["installschema"]))
					return($compiler->SetElementError($values["schema"]->file,$values["schema"]->path,"it was not defined the installschema type function for the schema class"));
		}
		if(IsSet($values["includeform"]))
		{
			for($form=0;$form<count($values["includeform"]);$form++)
			{
				$include_form=$values["includeform"][$form];
				if(!$compiler->ParseDefinitionFileFromPath($include_form,$component_path))
					return(0);
				$compiler->GetParser($parser,$include_form);
				if(strcmp($parser->structure["0"]["Tag"],"form"))
					return($compiler->SetElementError($include_form,"0","it was not specified a valid component form definition file"));
				$compiler->CreateFileContext($form_context,$include_form);
				if(!$this->ParseFormDefinition($compiler,$form_context,$name))
					return(0);
				$this->dependencies[]=$compiler->GetIncludePathFromList($include_form,$component_path).$include_form;
			}
		}
		if(IsSet($values["form"]))
		{
			for($form=0;$form<count($values["form"]);$form++)
			{
				if(!$this->ParseFormDefinition($compiler,$values["form"][$form],$name))
					return(0);
			}
		}
		if(IsSet($values["includereport"]))
		{
			for($report=0;$report<count($values["includereport"]);$report++)
			{
				$include_report=$values["includereport"][$report];
				if(!$compiler->ParseDefinitionFileFromPath($include_report,$component_path))
					return(0);
				$compiler->GetParser($parser,$include_report);
				if(strcmp($parser->structure["0"]["Tag"],"report"))
					return($compiler->SetElementError($include_report,"0","it was not specified a valid component report definition file"));
				$compiler->CreateFileContext($report_context,$include_report);
				if(!$this->ParseReportDefinition($compiler,$report_context,$name))
					return(0);
				$this->dependencies[]=$compiler->GetIncludePathFromList($include_report,$component_path).$include_report;
			}
		}
		if(IsSet($values["report"]))
		{
			for($report=0;$report<count($values["report"]);$report++)
			{
				if(!$this->ParseReportDefinition($compiler,$values["report"][$report],$name))
					return(0);
			}
		}
		return(1);
	}

	Function Setup(&$compiler,&$code,&$attributes)
	{
		$compiler->CreateClassObject($this->generator,"metal_persistence_generator_class");
		$compiler->CreateClassObject($this->factory_generator,"metal_persistence_factory_generator_class");
		$compiler->CreateClassObject($this->schema_generator,"metal_persistence_schema_generator_class");
		$compiler->CreateClassObject($this->form_generator,"metal_persistence_form_generator_class");
		$compiler->CreateClassObject($this->report_generator,"metal_persistence_report_generator_class");
		$values=array();
		$required_types=array(
			"component"=>"DATA",
			"definitions"=>"DATA"
		);
		$optional_types=array(
			"themesfile"=>"DATA"
		);
		if(!$compiler->GetOptionalArgumentValues($attributes,$values,$required_types,$optional_types,"persistence","Setup",1))
			return(0);
		if(strlen($values["component"])==0)
			return($compiler->SetElementError($attributes->file,$attributes->path,"it was specified an empty component file name"));
		if(IsSet($values["themesfile"]))
		{
			$themes_file=$values["themesfile"];
			if(!$compiler->ParseDefinitionFile($themes_file))
				return(0);
			$compiler->GetParser($parser,$themes_file);
			if(strcmp($parser->structure["0"]["Tag"],"themes"))
				return($compiler->SetElementError($themes_file,"0","it was not specified a valid themes list file"));
			$compiler->CreateFileContext($theme_context,$themes_file);
			$themes_values=array();
			$required_types=array(
				"theme"=>"CONTEXT:MULTIPLE",
			);
			$optional_types=array(
			);
			if(!$compiler->GetOptionalArgumentValues($theme_context,$themes_values,$required_types,$optional_types,"persistence","Setup",1))
				return(0);
			for($theme=0;$theme<count($themes_values["theme"]);$theme++)
			{
				$theme_values=array();
				$required_types=array(
					"name"=>"DATA",
					"file"=>"DATA"
				);
				$optional_types=array(
				);
				if(!$compiler->GetOptionalArgumentValues($themes_values["theme"][$theme],$theme_values,$required_types,$optional_types,"persistence","Setup",1))
					return(0);
				if(IsSet($this->themes[$theme_values["name"]]))
					return($compiler->SetElementError($themes_values["theme"][$theme]->file,$themes_values["theme"][$theme]->path,$theme_values["name"]." theme is already listed"));
				$file=$theme_values["file"];
				$base_path=array(dirname($values["themesfile"]));
				$this->themes[$theme_values["name"]]=$compiler->GetBaseIncludePath($file,$base_path).$file;
			}
			$this->dependencies[]=$compiler->GetIncludePath($themes_file).$themes_file;
		}

		$component_file=$values["component"];
		if(!$compiler->ParseDefinitionFile($component_file))
			return(0);
		$compiler->CreateFileContext($component_context,$component_file);
		$this->component_name="";
		if(!$this->ParseComponentDefinition($compiler,$component_context,$this->component_name))
			return(0);
		$this->dependencies[]=$compiler->GetIncludePath($component_file).$component_file;
		$this->definitions=$values["definitions"];
		return(1);
	}

	Function OutputDependencies(&$context)
	{
		for($dependency=0;$dependency<count($this->dependencies);$dependency++)
		{
			$context->result[]=array(
				"Data"=>array(
					"Name"=>$this->dependencies[$dependency],
					"Type"=>"DEPENDENCY"
				),
				"Type"=>"FILE"
			);
		}
	}

	Function EvaluatePaths(&$compiler,&$context,&$values,$class,$form="")
	{
		$functions=array(
			"componentname",
			"classname"
		);
		if(strlen($form))
			$functions[]="formname";
		if(!$compiler->AddContextFunctions($context,$this->object_name,$functions,$level))
			return(0);
		$current_form=$this->current_form;
		$current_class=$this->current_class;
		$component=$this->component_name;
		$this->current_form=$form;
		$this->current_class=$class;
		$success=($compiler->GetValidData($values["basefilepath"]->file,$values["basefilepath"]->path,$values["basefilename"],"it was not specified a valid component class base file path")
		&& $compiler->GetValidData($values["filename"]->file,$values["filename"]->path,$values["file"],"it was not specified a valid component class filename"));
		if($success
		&& IsSet($values["schemafile"])
		&& !IsSet($values["schemafilename"]))
			$success=$compiler->GetValidData($values["schemafile"]->file,$values["schemafile"]->path,$values["schemafilename"],"it was not specified a valid component schema file");
		if(strlen($form)
		&& $success
		&& IsSet($values["baseformpath"]))
			$success=$compiler->GetValidData($values["baseformpath"]->file,$values["baseformpath"]->path,$values["formfilename"],"it was not specified a valid component base form file path");
		$this->current_class=$current_class;
		$this->current_form=$current_form;
		if(!$success
		|| !$compiler->RemoveContextFunctions($context,$this->object_name,$level))
			return(0);
		return(1);
	}

	Function GenerateFunctionDeclaration(&$compiler, &$context, $name, &$arguments, $node)
	{
		$context->result[]=array(
			"Data"=>array(
				"Node"=>($function_node=$compiler->GetXMLNode()),
				"Parent"=>$node,
				"Tag"=>"function",
				"Indent"=>1
			),
			"Type"=>"XMLTAG"
		);
		$context->result[]=array(
			"Data"=>array(
				"Parent"=>$function_node,
				"Tag"=>"name",
				"Data"=>$name
			),
			"Type"=>"XMLTAG"
		);
		if(IsSet($arguments["arguments"]))
		{
			Reset($arguments["arguments"]);
			$end=(GetType($argument=Key($arguments["arguments"]))!="string");
			for(;!$end;)
			{
				$context->result[]=array(
					"Data"=>array(
						"Node"=>($argument_node=$compiler->GetXMLNode()),
						"Parent"=>$function_node,
						"Tag"=>"argument",
						"Indent"=>1
					),
					"Type"=>"XMLTAG"
				);
				$context->result[]=array(
					"Data"=>array(
						"Parent"=>$argument_node,
						"Tag"=>"name",
						"Data"=>$argument
					),
					"Type"=>"XMLTAG"
				);
				if(IsSet($arguments["arguments"][$argument]["type"]))
				{
					$context->result[]=array(
						"Data"=>array(
							"Node"=>($type_node=$compiler->GetXMLNode()),
							"Parent"=>$argument_node,
							"Tag"=>"type"
						),
						"Type"=>"XMLTAG"
					);
					$context->result[]=array(
						"Data"=>array(
							"Parent"=>$type_node,
							"Tag"=>"databasetype".$arguments["arguments"][$argument]["type"]
						),
						"Type"=>"XMLTAG"
					);
				}
				else
				{
					$context->result[]=array(
						"Data"=>array(
							"Parent"=>$argument_node,
							"Tag"=>"type",
							"Data"=>(IsSet($arguments["arguments"][$argument]["oid"]) ? "INTEGER" : "OBJECT:".$arguments["arguments"][$argument]["class"])
						),
						"Type"=>"XMLTAG"
					);
				}
				Next($arguments["arguments"]);
				$end=(GetType($argument=Key($arguments["arguments"]))!="string");
			}

		}
		return($function_node);
	}

	Function GenerateFunctionDotArgumentList(&$arguments)
	{
		if(!IsSet($arguments["arguments"]))
			return("");
		Reset($arguments["arguments"]);
		$end=(GetType($argument=Key($arguments["arguments"]))!="string");
		for($argument_list="";!$end;)
		{
			if(strlen($argument_list))
				$argument_list.=", ";
			$argument_list.=$argument.":".(IsSet($arguments["arguments"][$argument]["type"]) ? $arguments["arguments"][$argument]["type"] : $arguments["arguments"][$argument]["class"]);
			Next($arguments["arguments"]);
			$end=(GetType($argument=Key($arguments["arguments"]))!="string");
		}
		return($argument_list);
	}

	Function GenerateTableList(&$tables)
	{
		Reset($tables);
		$end=(GetType($table_name=Key($tables))!="string");
		for($table_list="";!$end;)
		{
			if(strlen($tables[$table_name]))
			{
				if(strlen($table_list))
					$table_list.=", ";
				if(strcmp($tables[$table_name], $table_name))
					$table_list.=$tables[$table_name]." ";
				$table_list.=$table_name;
			}
			Next($tables);
			$end=(GetType($table_name=Key($tables))!="string");
		}
		return($table_list);
	}

	Function VerifyUnusedArguments(&$compiler,$function_name,&$arguments)
	{
		if(!IsSet($arguments["arguments"]))
			return(1);
		Reset($arguments["arguments"]);
		$end=(GetType($argument=Key($arguments["arguments"]))!="string");
		for(;!$end;)
		{
			if($arguments["arguments"][$argument]["Uses"]==0)
			{
				$compiler->SetElementError($arguments["arguments"][$argument]["Context"]->file,$arguments["arguments"][$argument]["Context"]->path,"the argument ".$argument." of function ".$function_name." was declared but is not used in the function");
				return(0);
			}
			Next($arguments["arguments"]);
			$end=(GetType($argument=Key($arguments["arguments"]))!="string");
		}
		return(1);
	}

	Function GenerateComponent(&$compiler, &$context, &$values, $function, $component, &$factory_input)
	{
		$values["part"]="input";
		if(!$this->EvaluatePaths($compiler,$context,$values,$component))
			return(0);
		$context->result[]=array(
			"Data"=>array(
				"File"=>($factory_input=$values["basefilename"].".input")
			),
			"Type"=>"XMLFILE"
		);
		$this->generator->GenerateStorageClass($this,$compiler,$context,$class,$component,$component."class",$values);
		$input=$class;
		for($external=0,Reset($this->components[$component]["components"]);$external<count($this->components[$component]["components"]);Next($this->components[$component]["components"]),$external++)
		{
			$component_name=Key($this->components[$component]["components"]);
			if(IsSet($this->components[$component_name]))
			{
				if(!$this->EvaluatePaths($compiler,$context,$values,$component_name))
					return(0);
				$context->result[]=array(
					"Data"=>array(
						"Parent"=>$input,
						"Tag"=>"includeonce",
						"Data"=>$values["basefilename"].".input"
					),
					"Type"=>"XMLTAG"
				);
			}
		}

		for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
		{
			$class_name=Key($this->components[$component]["classes"]);
			if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
				return(0);
			$context->result[]=array(
				"Data"=>array(
					"Parent"=>$input,
					"Tag"=>"includeonce",
					"Data"=>$values["basefilename"].".input"
				),
				"Type"=>"XMLTAG"
			);
		}

		for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
		{
			$class_name=Key($this->components[$component]["classes"]);
			if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
				return(0);
			$context->result[]=array(
				"Data"=>array(
					"File"=>$values["basefilename"].".input"
				),
				"Type"=>"XMLFILE"
			);
			$this->generator->GenerateStorageClass($this,$compiler,$context,$input,$component,$class_name."class",$values);
			$context->result[]=array(
				"Data"=>array(
					"Parent"=>$input,
					"Tag"=>"includeonce",
					"Data"=>$factory_input
				),
				"Type"=>"XMLTAG"
			);
		}

		$values["part"]="output";
		if(!$this->EvaluatePaths($compiler,$context,$values,$component))
			return(0);
		$context->result[]=array(
			"Data"=>array(
				"File"=>$values["basefilename"].".output"
			),
			"Type"=>"XMLFILE"
		);
		$this->generator->GenerateStorageClass($this,$compiler,$context,$class,$component,$component."class",$values);
		for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
		{
			$class_name=Key($this->components[$component]["classes"]);
			if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
				return(0);
			$context->result[]=array(
				"Data"=>array(
					"File"=>$values["basefilename"].".output"
				),
				"Type"=>"XMLFILE"
			);
			$this->generator->GenerateStorageClass($this,$compiler,$context,$output,$component,$class_name."class",$values);
		}

		$values["part"]="class";
		if(!$this->EvaluatePaths($compiler,$context,$values,$component))
			return(0);
		$context->result[]=array(
			"Data"=>array(
				"File"=>$values["basefilename"].".class"
			),
			"Type"=>"XMLFILE"
		);
		if(!$this->factory_generator->GenerateClass($this, $compiler, $context, $component, $values, $function))
			return(0);

		for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
		{
			$class_name=Key($this->components[$component]["classes"]);
			if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
				return(0);
			$context->result[]=array(
				"Data"=>array(
					"File"=>$values["basefilename"].".class"
				),
				"Type"=>"XMLFILE"
			);
			if(!$this->generator->GenerateClass($this, $compiler, $context, $component, $class_name, $values, $function))
				return(0);
		}

		return(1);
	}

	Function Execute(&$compiler,&$code,$function,&$context,&$arguments_code,&$arguments)
	{
		switch($function)
		{
			case "generateschema":
				$this->schema_generator->Generate($this,$compiler,$context);
				$this->OutputDependencies($context);
				break;
			case "generatecomponent":
				$types=array(
					"basepath"=>"NAME",
					"basefilepath"=>"CONTEXT",
					"filename"=>"CONTEXT"
				);
				$optional_types=array(
					"baseformpath"=>"CONTEXT",
					"schemafile"=>"CONTEXT",
					"images"=>"DATA"
				);
				$values=array();
				if(!$compiler->GetOptionalArgumentValues($arguments,$values,$types,$optional_types,"persistence",$function,0)
				|| !$compiler->GetOptionalArgumentValues($context,$values,$types,$optional_types,"persistence",$function,1))
					return(0);
				$component=$this->component_name;

				$values["part"]="schema";
				if(!$this->EvaluatePaths($compiler,$context,$values,$component))
					return(0);
				if(IsSet($values["schemafilename"]))
				{
					$context->result[]=array(
						"Data"=>array(
							"File"=>$values["schemafilename"]
						),
						"Type"=>"XMLFILE"
					);
					$this->schema_generator->Generate($this,$compiler,$context);
				}

				if(!$this->GenerateComponent($compiler, $context, $values, $function, $component, $factory_input))
					return(0);

				for($components=array(),$external=0,Reset($this->components[$component]["components"]);$external<count($this->components[$component]["components"]);Next($this->components[$component]["components"]),$external++)
				{
					$component_name=Key($this->components[$component]["components"]);
					if(IsSet($this->components[$component_name]))
						$components[]=$component_name;
				}
				for($external=0;$external<count($components);$external++)
				{
					if(!$this->GenerateComponent($compiler, $context, $values, $function, $components[$external], $input))
						return(0);
				}

				if(IsSet($values["schemafile"]))
				{
					$values["part"]="input";
					if(!$this->EvaluatePaths($compiler,$context,$values,$component."schema"))
						return(0);
					$context->result[]=array(
						"Data"=>array(
							"File"=>$values["basefilename"].".input"
						),
						"Type"=>"XMLFILE"
					);

					$this->generator->GenerateStorageClass($this,$compiler,$context,$input,$component,$component."schemaclass",$values);
					for($schema_function=0,Reset($this->components[$component]["schema"]["functions"]);$schema_function<count($this->components[$component]["schema"]["functions"]);$schema_function++,Next($this->components[$component]["schema"]["functions"]))
					{
						$function_name=Key($this->components[$component]["schema"]["functions"]);
						$function_arguments=$this->components[$component]["schema"]["functions"][$function_name];
						if($function_arguments["type"]=="installschema")
						{
							$context->result[]=array(
								"Data"=>array(
									"Node"=>($function_node=$compiler->GetXMLNode()),
									"Parent"=>$input,
									"Tag"=>"function",
									"Indent"=>1
								),
								"Type"=>"XMLTAG"
							);
							$context->result[]=array(
								"Data"=>array(
									"Parent"=>$function_node,
									"Tag"=>"name",
									"Data"=>"installschemafunctionname"
								),
								"Type"=>"XMLTAG"
							);
							$context->result[]=array(
								"Data"=>array(
									"Parent"=>$function_node,
									"Tag"=>"object",
									"Data"=>"expressions"
								),
								"Type"=>"XMLTAG"
							);
							$context->result[]=array(
								"Data"=>array(
									"Parent"=>$function_node,
									"Tag"=>"call",
									"Data"=>"data"
								),
								"Type"=>"XMLTAG"
							);
							$context->result[]=array(
								"Data"=>array(
									"Parent"=>$function_node,
									"Tag"=>"arguments",
									"Data"=>$function_name
								),
								"Type"=>"XMLTAG"
							);
							break;
						}
					}

					$values["part"]="class";
					if(!$this->EvaluatePaths($compiler,$context,$values,$component."schema"))
						return(0);
					$context->result[]=array(
						"Data"=>array(
							"File"=>$values["basefilename"].".class"
						),
						"Type"=>"XMLFILE"
					);
					if(!$this->schema_generator->GenerateClass($this, $compiler, $context, $component, $values, $function))
						return(0);

					$values["part"]="output";
					if(!$this->EvaluatePaths($compiler,$context,$values,$component."schema"))
						return(0);
					$context->result[]=array(
						"Data"=>array(
							"File"=>$values["basefilename"].".output"
						),
						"Type"=>"XMLFILE"
					);
					$this->generator->GenerateStorageClass($this,$compiler,$context,$class,$component,$component."schemaclass",$values);
				}

				if(IsSet($this->components[$component]["forms"]))
				{
					if(!IsSet($values["baseformpath"]))
						return($compiler->SetElementError($context->file,$context->path,"it was not specified baseformpath argument"));
					for($form=0,Reset($this->components[$component]["forms"]);$form<count($this->components[$component]["forms"]);Next($this->components[$component]["forms"]),$form++)
					{
						$form_name=Key($this->components[$component]["forms"]);
						$values["part"]="form";
						if(!$this->EvaluatePaths($compiler,$context,$values,$component,$form_name))
							return(0);
						$form_file_name=$values["formfilename"].".form";
						$context->result[]=array(
							"Data"=>array(
								"File"=>$form_file_name
							),
							"Type"=>"XMLFILE"
						);
						if(!$this->form_generator->Generate($this,$compiler,$context,$component,$form_name))
							return(0);

						$class_name=$form_name."form";
						$values["part"]="class";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".class"
							),
							"Type"=>"XMLFILE"
						);
						if(!$this->form_generator->GenerateClass($this, $compiler, $context, $component, $class_name, $form_name, $values))
							return(0);

						$values["part"]="input";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".input"
							),
							"Type"=>"XMLFILE"
						);
						$this->generator->GenerateStorageClass($this,$compiler,$context,$class_node,$component,$class_name."class",$values);
						$this->form_generator->GenerateClassInput($this,$compiler,$context,$class_node,$component,$form_name,$factory_input,$form_file_name);
						$values["part"]="output";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".output"
							),
							"Type"=>"XMLFILE"
						);
						$this->generator->GenerateStorageClass($this,$compiler,$context,$class_node,$component,$class_name."class",$values);
					}
				}

				if(IsSet($this->components[$component]["reports"]))
				{
					for($report=0,Reset($this->components[$component]["reports"]);$report<count($this->components[$component]["reports"]);Next($this->components[$component]["reports"]),$report++)
					{
						$report_name=Key($this->components[$component]["reports"]);
						$class_name=$report_name."report";
						$values["part"]="class";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".class"
							),
							"Type"=>"XMLFILE"
						);
						if(!$this->report_generator->GenerateClass($this, $compiler, $context, $component, $class_name, $report_name, $values, $function))
							return(0);

						$values["part"]="input";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".input"
							),
							"Type"=>"XMLFILE"
						);
						$this->generator->GenerateStorageClass($this,$compiler,$context,$class_node,$component,$class_name."class",$values);
						$this->report_generator->GenerateClassInput($this,$compiler,$context,$class_node,$component,$report_name,$factory_input);

						$values["part"]="output";
						if(!$this->EvaluatePaths($compiler,$context,$values,$class_name))
							return(0);
						$context->result[]=array(
							"Data"=>array(
								"File"=>$values["basefilename"].".output"
							),
							"Type"=>"XMLFILE"
						);
						$this->generator->GenerateStorageClass($this,$compiler,$context,$class_node,$component,$class_name."class",$values);
					}
				}

				$this->OutputDependencies($context);
				break;
			case "forfactoryclass":
				$functions=array(
					"componentname",
					"classname"
				);
				if(!$compiler->AddContextFunctions($context,$this->object_name,$functions,$level))
					return(0);
				$current_class=$this->current_class;
				$component=$this->component_name;
				$this->current_class=$this->component_name;
				$success=$compiler->GetOutputData($context->file,$context->path,$context->result,"it was not defined valid action for the component class",1);
				$this->current_class=$current_class;
				if(!$success
				|| !$compiler->RemoveContextFunctions($context,$this->object_name,$level))
					return(0);
				break;
			case "forschemaclass":
				$functions=array(
					"componentname",
					"classname"
				);
				if(!$compiler->AddContextFunctions($context,$this->object_name,$functions,$level))
					return(0);
				$current_class=$this->current_class;
				$component=$this->component_name;
				$this->current_class=$this->component_name."schema";
				$success=$compiler->GetOutputData($context->file,$context->path,$context->result,"it was not defined valid action for the component class",1);
				$this->current_class=$current_class;
				if(!$success
				|| !$compiler->RemoveContextFunctions($context,$this->object_name,$level))
					return(0);
				break;
			case "forallclasses":
				$functions=array(
					"componentname",
					"classname"
				);
				if(!$compiler->AddContextFunctions($context,$this->object_name,$functions,$level))
					return(0);
				$current_class=$this->current_class;
				$component=$this->component_name;
				for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
				{
					$this->current_class=Key($this->components[$component]["classes"]);
					if(!($success=$compiler->GetOutputData($context->file,$context->path,$context->result,"it was not defined valid action for the component classes",1)))
						break;
				}
				if($success
				&& IsSet($this->components[$component]["forms"]))
				{
					for($class=0,Reset($this->components[$component]["forms"]);$class<count($this->components[$component]["forms"]);Next($this->components[$component]["forms"]),$class++)
					{
						$this->current_class=Key($this->components[$component]["forms"])."form";
						if(!($success=$compiler->GetOutputData($context->file,$context->path,$context->result,"it was not defined valid action for the component classes",1)))
							break;
					}
				}
				if($success
				&& IsSet($this->components[$component]["reports"]))
				{
					for($class=0,Reset($this->components[$component]["reports"]);$class<count($this->components[$component]["reports"]);Next($this->components[$component]["reports"]),$class++)
					{
						$this->current_class=Key($this->components[$component]["reports"])."report";
						if(!($success=$compiler->GetOutputData($context->file,$context->path,$context->result,"it was not defined valid action for the component classes",1)))
							break;
					}
				}
				$this->current_class=$current_class;
				if(!$success
				|| !$compiler->RemoveContextFunctions($context,$this->object_name,$level))
					return(0);
				break;
			case "componentname":
				$context->result[]=array(
					"Data"=>$this->component_name,
					"Type"=>"DATA"
				);
				break;
			case "classname":
				if(strlen($this->current_class)==0)
					return($compiler->SetElementError($context->file,$context->path,"it was requested the $function outside a component class iteration scope"));
				$context->result[]=array(
					"Data"=>$this->current_class,
					"Type"=>"DATA"
				);
				break;
			case "formname":
				if(strlen($this->current_form)==0)
					return($compiler->SetElementError($context->file,$context->path,"it was requested the $function outside a component form iteration scope"));
				$context->result[]=array(
					"Data"=>$this->current_form,
					"Type"=>"DATA"
				);
				break;
			case "generategraph":
				$types=array(
					"graphfile"=>"DATA"
				);
				$optional_types=array(
				);
				$values=array();
				if(!$compiler->GetOptionalArgumentValues($arguments,$values,$types,$optional_types,"persistence",$function,0)
				|| !$compiler->GetOptionalArgumentValues($context,$values,$types,$optional_types,"persistence",$function,1))
					return(0);
				if(!($dot=fopen($values["graphfile"],"w")))
					return($compiler->SetElementError($context->file,$context->path,"could not open the graph file for writing"));
				$linefeed="\n";
				$indent="\t";
				$component=$this->component_name;
				$graph="digraph ".$component." {".$linefeed;
				$graph.=$indent."node [shape=record,fontname=\"Helvetica\",fontsize=10,fillcolor=white]".$linefeed;
				$graph.=$indent."edge [fontname=\"Helvetica\",fontsize=10,labelfontname=\"Helvetica\",labelfontsize=10]".$linefeed;
				for($class=0,Reset($this->components[$component]["classes"]);$class<count($this->components[$component]["classes"]);Next($this->components[$component]["classes"]),$class++)
				{
					$current_class=Key($this->components[$component]["classes"]);
					if(!$this->generator->GenerateClassDiagram($compiler,$this,$component,$current_class,$indent,$linefeed,$graph))
						return(0);
				}
				if(!$this->factory_generator->GenerateClassDiagram($compiler,$this,$component,$indent,$linefeed,$graph))
					return(0);
				if(!$this->schema_generator->GenerateClassDiagram($compiler,$this,$component,$indent,$linefeed,$graph))
					return(0);
				if(IsSet($this->components[$component]["forms"]))
				{
					for($form=0,Reset($this->components[$component]["forms"]);$form<count($this->components[$component]["forms"]);Next($this->components[$component]["forms"]),$form++)
					{
						$current_form=Key($this->components[$component]["forms"]);
						if(!$this->form_generator->GenerateClassDiagram($compiler,$this,$component,$current_form,$indent,$linefeed,$graph))
							return(0);
					}
				}
				if(IsSet($this->components[$component]["reports"]))
				{
					for($report=0,Reset($this->components[$component]["reports"]);$report<count($this->components[$component]["reports"]);Next($this->components[$component]["reports"]),$report++)
					{
						$current_report=Key($this->components[$component]["reports"]);
						if(!$this->report_generator->GenerateClassDiagram($compiler,$this,$component,$current_report,$indent,$linefeed,$graph))
							return(0);
					}
				}
				$graph.="}".$linefeed;
				$success=(fwrite($dot,$graph) && fflush($dot));
				fclose($dot);
				if(!$success)
					return($compiler->SetElementError($context->file,$context->path,"could not write to the graph file"));
				$context->result[]=array(
					"Data"=>array(
						"Name"=>$values["graphfile"],
						"Type"=>"GENERATED"
					),
					"Type"=>"FILE"
				);
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$function." is not a supported persistence object function");
				return(0);
		}
		return(1);
	}

};

}
?>
