<?php
if(!defined("METAL_SCRIPT_INCLUDED"))
{
	define("METAL_SCRIPT_INCLUDED",1);

/*
 * metal_script.php
 *
 * @(#) $Header: /opt2/ena/metal/metal/metal_script.php,v 1.28 2002/10/20 07:54:33 mlemos Exp $
 *
 */
 
class metal_script_class extends metal_base_class
{
	Function GetScript(&$compiler,&$code,&$context,&$file,&$name,&$skip_white_space,$require_file)
	{
		$file=$name="";
		$skip_white_space=0;
		$elements=$code->structure[$context->path]["Elements"];
		for($element=0;$element<$elements;$element++)
		{
			$element_path=$context->path.",$element";
			$data=$code->structure[$element_path];
			if(GetType($data)=="array")
			{
				switch($data["Tag"])
				{
					case "file":
						$error="it was not specified a valid script output file name";
						if(!$compiler->GetValidData($context->file,$element_path,$file,$error))
							return(0);
						if(!strcmp($file,""))
						{
							$compiler->SetElementError($context->file,$context->path,$error);
							return(0);
						}
						break;
					case "name":
						$error="it was not specified a valid script name";
						if(!$compiler->GetValidData($context->file,$element_path,$name,$error))
							return(0);
						if(!strcmp($name,""))
						{
							$compiler->SetElementError($context->file,$context->path,$error);
							return(0);
						}
						break;
					case "skipwhitespace":
						if(!$compiler->GetValidData($context->file,$element_path,$skip_white_space,"it was not specified a valid script skipwhitespace option"))
							return(0);
						if(strcmp($skip_white_space,"1")
						&& strcmp($skip_white_space,"0"))
						{
							$compiler->SetElementError($context->file,$element_path,"it was not specified a valid boolean script skipwhitespace option value");
							return(0);
						}
						$this->skip_white_space=intval($skip_white_space);
						break;
					default:
						$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a supported script object attribute");
						return(0);
				}
			}
			else
			{
				if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
					return(0);
			}
		}
		if($require_file
		&& !strcmp($file,""))
		{
			$compiler->SetElementError($context->file,$context->path,"it was not specified the script output file name");
			return(0);
		}
		if(!strcmp($name,""))
		{
			$compiler->SetElementError($context->file,$context->path,"it was not specified the script name");
			return(0);
		}
		return(1);
	}

	Function GenerateOutput(&$compiler,&$code,&$context,&$arguments_code,&$arguments,&$file,$require_file)
	{
		$message=array(
			"output"=>"",
			"name"=>"",
			"evaluate"=>0,
			"result"=>array(),
			"variables"=>array(),
			"exceptions"=>array()
		);
		if(!$this->GetScript($compiler,$arguments_code,$arguments,$file,$message["name"],$skip_white_space,$require_file)
		|| !$compiler->StartScope($context,$scope,"global")
		|| !$compiler->GetOutputData($context->file,$context->path,$message["result"],"invalid script output",$skip_white_space)
		|| !$compiler->GetScopeVariables($context,$scope,$message["variables"])
		|| !$compiler->GetScopeExceptions($context,$scope,$message["exceptions"])
		|| !$compiler->EndScope($context,$scope))
			return(0);
		$message["evaluate"]=!strcmp($file,"");
		if(!$this->language_bindings->Execute($compiler,$context,$this,"output",$message))
			return(0);
		if(strcmp($file,""))
		{
			if(!$compiler->CreateContextOutputDirectory(dirname($file),$context,"it was not possible to create the directory for the output file \"".dirname($file)."\""))
				return(0);
			if(!($output_file=@fopen($file,"w")))
			{
				$compiler->SetElementError($context->file,$context->path,"it was not possible to open the script output file \"$file\"");
				return(0);
			}
			$written=fwrite($output_file,$message["output"]);
			$closed=fclose($output_file);
			if($written!=strlen($message["output"])
			|| !$closed)
			{
				$compiler->SetElementError($context->file,$context->path,"it was not possible to write to the script output file");
				return(0);
			}
			$context->result[]=array(
				"Data"=>array(
					"Name"=>$file,
					"Type"=>"GENERATED"
				),
				"Type"=>"FILE"
			);
		}
		else
		{
			$context->result[]=array(
				"Data"=>$message["output"],
				"Type"=>"DATA"
			);
		}
		return(1);
	}

	Function Execute(&$compiler,&$code,$function,&$context,&$arguments_code,&$arguments)
	{
		switch($function)
		{
			case "output":
				if(!$this->GenerateOutput($compiler,$code,$context,$arguments_code,$arguments,$file,0))
					return(0);
				break;
			case "input":
				Unset($file);
				$elements=$code->structure[$context->path]["Elements"];
				for($element=0;$element<$elements;$element++)
				{
					$element_path=$context->path.",$element";
					$data=$code->structure[$element_path];
					if(GetType($data)=="array")
					{
						switch($data["Tag"])
						{
							case "file":
								if(!$compiler->GetValidData($context->file,$element_path,$file,"it was not specified a valid script input file name"))
									return(0);
								$include_path=$compiler->GetIncludePath($file);
								if(!strcmp($file,"")
								|| !file_exists($include_path.=$file))
								{
									$compiler->SetElementError($context->file,$context->path,"it was not specified a existing script input file (\"$file\")");
									return(0);
								}
								break;
							default:
								$compiler->SetElementError($context->file,$element_path,$data["Tag"]." is not a supported script input argument");
								return(0);
						}
					}
					else
					{
						if(!$compiler->CheckWhiteSpace($context->file,$data,$element_path))
							return(0);
					}
				}
				if(!IsSet($file))
				{
					$compiler->SetElementError($context->file,$element_path,"it was not defined the script input file argument");
					return(0);
				}
				if(!$compiler->ExecuteOutputFile($file,$context->result,0))
					return(0);
				break;
			case "include":
				if(!$this->GenerateOutput($compiler,$code,$context,$arguments_code,$arguments,$file,1)
				|| !$compiler->ConvertData($context,"STRING",$file,$file_string))
					return(0);
				$context->result[]=array(
					"Data"=>"include($file_string);",
					"Type"=>"COMMAND"
				);
				break;
			case "flush":
				$context->result[]=array(
					"Data"=>array(
						"Type"=>"FLUSH"
					),
					"Type"=>"CONTROL"
				);
				break;
			default:
				$compiler->SetElementError($context->file,$context->path,$function." is not a supported script object function");
				return(0);
		}
		return(1);
	}
};
 
}
?>