<?php
/*
 * read_arguments.php
 *
 * @(#) $Id: read_arguments.php,v 1.1 2004/08/19 03:54:58 mlemos Exp $
 *
 */

/*
{metadocument}<?xml version="1.0" encoding="ISO-8859-1" ?>
<class>

	<package>net.manuellemos.readarguments</package>

	<version>@(#) $Id: read_arguments.php,v 1.1 2004/08/19 03:54:58 mlemos Exp $</version>
	<copyright>Copyright  (C) Manuel Lemos 2004</copyright>
	<title>Read arguments</title>
	<author>Manuel Lemos</author>
	<authoraddress>mlemos-at-acm.org</authoraddress>

	<documentation>
		<idiom>en</idiom>
		<purpose>Parse the command line arguments and fill a list of option
			values according to the definition of the options of your
			program.</purpose>
		<usage>Use the <functionlink>AddOption</functionlink> function to
			specify the definition of each of the options supported by your
			program.<paragraphbreak />
			Then use the <functionlink>ProcessArguments</functionlink> function
			to process the command line arguments and extract the options
			values.<paragraphbreak />
			If you need to present your program options to the user, call
			<functionlink>Usage</functionlink> function to generate the list
			of options in a user readable format.</usage>
	</documentation>

{/metadocument}
*/

class read_arguments_class
{
/*
{metadocument}
	<variable>
		<name>error</name>
		<type>STRING</type>
		<value></value>
		<documentation>
			<purpose>Store the message that is returned when an error
				occurs.</purpose>
			<usage>Check this variable to understand what happened when a call to
				any of the class functions has failed.<paragraphbreak />
				This class uses cumulative error handling. This means that if one
				class functions that may fail is called and this variable was
				already set to an error message due to a failure in a previous call
				to the same or other function, the function will also fail and does
				not do anything.<paragraphbreak />
				This allows programs using this class to safely call several
				functions that may fail and only check the failure condition after
				the last function call.<paragraphbreak />
				Just set this variable to an empty string to clear the error
				condition.</usage>
		</documentation>
	</variable>
{/metadocument}
*/
	var $error='';

	/* Private variables */
	var $options=array();
	var $keywords=array();

/*
{metadocument}
	<function>
		<name>AddOption</name>
		<type>BOOLEAN</type>
		<documentation>
			<purpose>Add the definition of one argument option.</purpose>
			<usage>Pass an associative array with the <argumentlink>
					<function>AddOption</function>
					<argument>parameters</argument>
				</argumentlink> to define the
				option.</usage>
			<returnvalue>This function returns <booleanvalue>0</booleanvalue> if
				there was an inconsistency error in the definition of the argument
				option.<paragraphbreak />
				This return value may be safely ignored if the option arguments are
				defined correctly.</returnvalue>
		</documentation>
		<argument>
			<name>parameters</name>
			<type>HASH</type>
			<documentation>
				<purpose>Associative array that lists the parameters that define
					the option to be added.<paragraphbreak />
					The array entry indexes are the names of the parameters and the
					array entry values are the respective parameter values. Here
					follows the list of supported option parameters:<paragraphbreak />
					<tt><stringvalue>Name</stringvalue></tt> - (required) Name of the
					option. Each option must have an unique name.<paragraphbreak />
					<tt><stringvalue>Type</stringvalue></tt> - (required) Type of
					option. Currently supported types are:<paragraphbreak />
					- <stringvalue>text</stringvalue> for normal text
					arguments.<paragraphbreak />
					- <stringvalue>switch</stringvalue> for boolean arguments that
					are enabled by the presence of an option keyword.<paragraphbreak />
					- <stringvalue>usage</stringvalue> is like
					<stringvalue>switch</stringvalue> except that it instructs the
					program to present an usage help message. When this type of
					option is found in the command line arguments, the
					<functionlink>ProcessArguments</functionlink> function ignores
					any other arguments.<paragraphbreak />
					<tt><stringvalue>Description</stringvalue></tt> - (required)
					Short phrase with a human readable description of the purpose of
					the option. This description is used to compose the command usage
					help message returned by the <functionlink>Usage</functionlink>
					function.<paragraphbreak />
					<tt><stringvalue>Required</stringvalue></tt> - Boolean parameter
					to specify that the option values must be present in the command
					line arguments, otherwise the
					<functionlink>ProcessArguments</functionlink> will fail.<paragraphbreak />
					<tt><stringvalue>Keyword</stringvalue></tt> - Text that
					identifies an option in the command line. Usually it is a text
					string starting with <stringvalue>-</stringvalue> or
					<stringvalue>--</stringvalue> on Unix systems or
					<stringvalue>/</stringvalue> on Windows, but this class does not
					use or require any prefix for the options keyword text.<paragraphbreak />
					A keyword text string may not be associated to more than one
					option.<paragraphbreak />
					For <stringvalue>text</stringvalue> options, the keyword text
					must preceed the actual value in the command line arguments
					array. For <stringvalue>switch</stringvalue> and
					<stringvalue>usage</stringvalue> options, this parameter is
					required. For these types of option, the use of the keyword in
					the command line arguments indicates that the option was
					specified.<paragraphbreak />
					<tt><stringvalue>KeywordRequired</stringvalue></tt> - Boolean
					parameter to specify that the option value is only considered
					when it is preceeded by the keyword text. It only applies to
					<stringvalue>text</stringvalue> type options.<paragraphbreak />
					<tt><stringvalue>Aliases</stringvalue></tt> - Array of one or
					more strings to specify other keywords that may be used as
					equivalent alternatives to the option main keyword
					text.<paragraphbreak />
					Like the main keyword text string, an alias text may not be
					associated to more than one option.<paragraphbreak />
					</purpose>
			</documentation>
		</argument>
		<do>
{/metadocument}
*/
	Function AddOption($parameters)
	{
		if(strlen($this->error))
			return(0);
		if(!IsSet($parameters['Name'])
		|| strlen($parameters['Name'])==0)
		{
			$this->error='it was not specified the option name';
			return(0);
		}
		$name=$parameters['Name'];
		if(IsSet($this->options[$name]))
		{
			$this->error='it was already specified an option with the name '.$name;
			return(0);
		}
		if(!IsSet($parameters['Type']))
		{
			$this->error='it was not specified the type of option '.$name;
			return(0);
		}
		if(!IsSet($parameters['Description']))
		{
			$this->error='it was not specified the description of option '.$name;
			return(0);
		}
		switch($type=$parameters['Type'])
		{
			case 'text':
				$requires_keyword=0;
				break;
			case 'switch':
			case 'usage':
				$requires_keyword=1;
				break;
			default:
				$this->error=$type.' is not a supported type of option for '.$name;
				return(0);
		}
		$option=array(
			'Type'=>$type,
			'Description'=>$parameters['Description']
		);
		if(IsSet($parameters['Required'])
		&& $parameters['Required'])
			$option['Required']=1;
		if(IsSet($parameters['Aliases']))
			$keywords=$option['Aliases']=$parameters['Aliases'];
		else
			$keywords=array();
		for($k=0;$k<count($keywords);$k++)
		{
			$alias=$keywords[$k];
			if(IsSet($this->keywords[$alias]))
			{
				$this->error='the keyword alias '.$alias.' of option '.$name.' is already associated to another option';
				return(0);
			}
		}
		if(IsSet($parameters['Keyword']))
		{
			if(IsSet($parameters['KeywordRequired'])
			&& $parameters['KeywordRequired'])
				$option['KeywordRequired']=$parameters['KeywordRequired'];
			$keyword=$parameters['Keyword'];
			if(IsSet($this->keywords[$keyword]))
			{
				$this->error='the keyword '.$keyword.' of option '.$name.' is already associated to another option';
				return(0);
			}
			$keywords[]=$option['Keyword']=$parameters['Keyword'];
			for($k=0;$k<count($keywords);$k++)
				$this->keywords[$keywords[$k]]=$name;
		}
		else
		{
			if($requires_keyword)
			{
				$this->error='it was not specified the keyword for the option '.$name;
				return(0);
			}
		}
		$this->options[$name]=$option;
		return(1);
	}
/*
{metadocument}
		</do>
	</function>
{/metadocument}
*/

/*
{metadocument}
	<function>
		<name>ProcessArguments</name>
		<type>BOOLEAN</type>
		<documentation>
			<purpose>Process the command line arguments to extract the option
				values.</purpose>
			<usage>Pass to this function the array of strings of the command line
				arguments to the <argumentlink>
					<function>ProcessArguments</function>
					<argument>argv</argument>
				</argumentlink> argument as it is passed to the current script.<paragraphbreak />
				This function starts processing the arguments from the string in
				the position <integervalue>1</integervalue> of the array. The
				argument in the position <integervalue>0</integervalue> is usually
				the current script name.<paragraphbreak />
				If the specified arguments comply with all the option requirements,
				this function succeeds and the <argumentlink>
					<function>ProcessArguments</function>
					<argument>values</argument>
				</argumentlink> argument returns an associative array with the
				values of the options that were used.<paragraphbreak />
				If this function fails, check the
				<variablelink>error</variablelink> to determine why the given
				command line arguments do not comply with options requirements.</usage>
			<returnvalue>This function returns <booleanvalue>0</booleanvalue> if
				this the given command line arguments do not comply with the
				requirements of the defined options.</returnvalue>
		</documentation>
		<argument>
			<name>argv</name>
			<type>ARRAY</type>
			<documentation>
				<purpose>List of command line argument strings.</purpose>
			</documentation>
		</argument>
		<argument>
			<name>values</name>
			<type>HASH</type>
			<out />
			<documentation>
				<purpose>Associative array used to return the values of the options
					used in the command line arguments.<paragraphbreak />
					The indexes of the array entries are the names of options that
					were used. The respective entry values are set according to the
					type of option.<paragraphbreak />
					For <stringvalue>text</stringvalue> options, the values are the
					actual arguments' text. For <stringvalue>switch</stringvalue> and
					<stringvalue>usage</stringvalue> options, the values are set to
					the number of the argument on which the respective option
					keyword text was used.<paragraphbreak />
					When an option that is not required is not used, the respective
					value entry is not set.<paragraphbreak />
					When a <stringvalue>usage</stringvalue> option is used, no other
					option values are returned even if they were used.</purpose>
			</documentation>
		</argument>
		<do>
{/metadocument}
*/
	Function ProcessArguments($argv, &$values)
	{
		$values=array();
		if(strlen($this->error))
			return(0);
		$fill=$this->options;
		Reset($fill);
		$end_fill=(GetType($next_fill=Key($fill))!='string');
		for($argument=1;$argument<count($argv);$argument++)
		{
			$value=$argv[$argument];
			if(IsSet($this->keywords[$value]))
			{
				$option=$this->keywords[$value];
				switch($this->options[$option]['Type'])
				{
					case 'usage':
						$values=array($option=>$argument);
						return(1);
					case 'text':
						if(IsSet($this->options[$option]['KeywordRequired']))
						{
							$argument++;
							if($argument>=count($argv))
							{
								$this->error='it was not specified the value of the option '.$option;
								return(0);
							}
							$values[$option]=$argv[$argument];
							break;
						}
					default:
						if(IsSet($values[$option]))
						{
							$this->error='it was specified the option '.$option.' more than once';
							return(0);
						}
						$values[$option]=$argument;
						break;
				}
			}
			else
			{
				for(;;)
				{
					if($end_fill)
					{
						$this->error='it were specified too many arguments';
						return(0);
					}
					switch($this->options[$next_fill]['Type'])
					{
						case 'text':
							if(IsSet($this->options[$next_fill]['KeywordRequired']))
								break;
							$values[$next_fill]=$value;
							Next($fill);
							$end_fill=(GetType($next_fill=Key($fill))!='string');
							break 2;
					}
					Next($fill);
					$end_fill=(GetType($next_fill=Key($fill))!='string');
				}
			}
		}
		Reset($this->options);
		$end=(GetType($option=Key($this->options))!='string');
		for(;!$end;)
		{
			if(IsSet($this->options[$option]['Required'])
			&& !IsSet($values[$option]))
			{
				$this->error='it was not specified the required option '.$option;
				return(0);
			}
			Next($this->options);
			$end=(GetType($option=Key($this->options))!='string');
		}
		return(1);
	}
/*
{metadocument}
		</do>
	</function>
{/metadocument}
*/

/*
{metadocument}
	<function>
		<name>Usage</name>
		<type>BOOLEAN</type>
		<documentation>
			<purpose>Generate the list of options of a command in a user readable
				format.</purpose>
			<usage>Pass the name of the current script to the <argumentlink>
				<function>Usage</function>
				<argument>command</argument>
			</argumentlink> argument. Usually this is the first element
			(<integervalue>0</integervalue>) of the command line argument strings
			array.</usage>
			<returnvalue>Text that lists the options.</returnvalue>
		</documentation>
		<argument>
			<name>command</name>
			<type>STRING</type>
			<documentation>
				<purpose>Name of the command.</purpose>
			</documentation>
		</argument>
		<do>
{/metadocument}
*/
	Function Usage($command)
	{
		$usage=$command;
		Reset($this->options);
		$end=(GetType($option=Key($this->options))!='string');
		for($keywords_length=0;!$end;)
		{
			$alias='';
			if(IsSet($this->options[$option]['Keyword']))
			{
				$keyword=$this->options[$option]['Keyword'];
				if(IsSet($this->options[$option]['Aliases']))
				{
					for($a=0;$a<count($this->options[$option]['Aliases']);$a++)
						$alias.=', '.$this->options[$option]['Aliases'][$a];
				}
			}
			else
				$keyword=$option;
			switch($this->options[$option]['Type'])
			{
				case 'text':
					if(IsSet($this->options[$option]['Required']))
					{
						$usage.=' '.$keyword;
						break;
					}
				case 'switch':
				case 'usage':
					$usage.=' ['.$keyword.']';
					break;
			}
			$keyword.=$alias;
			$keywords_length=max($keywords_length,strlen($keyword));
			$keywords[$option]=$keyword;
			Next($this->options);
			$end=(GetType($option=Key($this->options))!='string');
		}
		$usage.="\n\n";
		Reset($this->options);
		$end=(GetType($option=Key($this->options))!='string');
		for(;!$end;)
		{
			$keyword=$keywords[$option];
			for($pad='',$length=strlen($keyword); $length<$keywords_length+2; $pad.=' ', $length++);
			$usage.=$keyword.$pad.$this->options[$option]['Description']."\n";
			Next($this->options);
			$end=(GetType($option=Key($this->options))!='string');
		}
		return($usage);
	}

/*
{metadocument}
		</do>
	</function>
{/metadocument}
*/

};

/*

{metadocument}
</class>
{/metadocument}

*/

?>
